<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\AccountGroup;
use backend\modules\master\models\AccountGroupSearch;
use backend\modules\master\models\ParentGroup;
use backend\modules\master\models\Types;
use backend\modules\master\models\TypesSearch;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;
use yii2tech\spreadsheet\Spreadsheet;

/**
 * AccountGroupController implements the CRUD actions for AccountGroup model.
 */
class AccountGroupController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all AccountGroup models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=19;
        $searchModel = new AccountGroupSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        
        return $this->render('/account-group/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single AccountGroup model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new AccountGroup model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=19;
        $model = new AccountGroup();
        $parent_id=ParentGroup::find()->all();
        $type_id=Types::find()->all();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            Yii::$app->getSession()->setFlash('success', 'Created Successfully');
            return $this->redirect(['account-group/index']);        }

        return $this->render('/account-group/_form', [
            'model' => $model,
            'parent_id'=>$parent_id,
            'type_id'=>$type_id,
            'update'=>0,
        ]);
    }

    /**
     * Updates an existing AccountGroup model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=19;
        $model = $this->findModel($id);
        $parent_id=ParentGroup::find()->all();
        $type_id=Types::find()->all();
        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
            return $this->redirect(['account-group/index']);
        }

        return $this->render('/account-group/_form', [
            'model' => $model,
            'parent_id'=>$parent_id,
            'type_id'=>$type_id,
            'update'=>1,
        ]);
    }

    /**
     * Deletes an existing AccountGroup model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $model =AccountGroup::findOne($id);
        $this->findModel($id)->delete();
        Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');

        return $this->redirect(['account-group/index']);
    }
    
    public function actionExcelExport($detail=null)
    {
        $dataProvider = new ActiveDataProvider([
            'query' => AccountGroup::find(),
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'group_name', 
            //'parent_id',
            //'type_id',
             [
                'label'=>'Parent Name',
               'attribute' => 'parent_group',
               'value' => function ($data) {
                $value=ParentGroup::find()->where(['parent_id'=>$data->parent_id])->one();
                return $value?$value->parent_name:'';
               }
            ],
            [
                'label'=>'Type Name',
               'attribute' => 'types',
               'value' => function ($data) {
                $value=Types::find()->where(['type_id'=>$data->type_id])->one();
                return $value?$value->type_name:'';
               }
            ],
            'created_at:date',
            'updated_at:date'
                         
        ],
    ]);
        }
        else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    'group_name', 
                    [
                        'label'=>'Parent Name',
                       'attribute' => 'parent_id',
                       'value' => function ($data) {
                        $value=ParentGroup::find()->where(['parent_id'=>$data->parent_id])->one();
                        return $value?$value->parent_name:'';
                       }
                    ],
                    [
                        'label'=>'Type Name',
                       'attribute' => 'type_id',
                       'value' => function ($data) {
                        $value=Types::find()->where(['type_id'=>$data->type_id])->one();
                        return $value?$value->type_name:'';
                       }
                    ],
                                                       
                ],
            ]);
        }
    if($detail)
    return $exporter->send('account_group_details.xlsx');
    return $exporter->send('account_group.xlsx');
    }
    public function actionExportPdf()
    {
        $dataProvider = new ActiveDataProvider([
            'query' => AccountGroup::find()->where(['status' => 1]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->WriteHTML($this->renderPartial('/account-group/exportpdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('account_group.pdf', 'I');
    }
    public function actionDeleteAccountGroup()
    {
        // Yii::$app->view->params['active']=5;
        $ids=Yii::$app->request->post('data');
        if($ids){
        foreach($ids as $id){
        $this->findModel($id)->delete();
        }
        Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
        return $this->redirect(['index']);
    }
    
    }
    /**
     * Finds the AccountGroup model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return AccountGroup the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = AccountGroup::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
