<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\AccountingLedger;
use backend\modules\master\models\AccountType;
use backend\modules\master\models\AccountingLedgerSearch;
use backend\modules\master\models\GroupName;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;
use yii\helpers\ArrayHelper;
use yii2tech\spreadsheet\Spreadsheet;

/**
 * AccountingLedgerController implements the CRUD actions for AccountingLedger model.
 */
class AccountingLedgerController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all AccountingLedger models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=17;
        $searchModel = new AccountingLedgerSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $model = new AccountingLedger();
        $ledger=AccountingLedger::find()->all();
        $accounttype=AccountType::find()->all();
       
       
        return $this->render('index', [
            'model' => $model,
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'ledger'  => $ledger,
            'accounttype'  => $accounttype,
        ]);
    }

    /**
     * Displays a single AccountingLedger model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new AccountingLedger model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=17;
        $model = new AccountingLedger();
        $groupname=GroupName::find()->all();
        $accounttype=AccountType::find()->all();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {

            Yii::$app->getSession()->setFlash('success', 'Created Successfully');
            return $this->redirect(['accounting-ledger/index']);
        }

        return $this->render('/accounting-ledger/_form', [
            'model' => $model,
            'groupname'=>$groupname,
            'accounttype'=>$accounttype,
            'update'=>0,
        ]);
    }

    /**
     * Updates an existing AccountingLedger model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=17;
        $model = $this->findModel($id);

        $groupname=GroupName::find()->all();
        $accounttype=AccountType::find()->all();
        if ($model->load(Yii::$app->request->post()) && $model->save()) {

            Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
            return $this->redirect(['accounting-ledger/index']);
        }

        return $this->render('/accounting-ledger/_form', [
            'model' => $model,
            'groupname'=>$groupname,
            'accounttype'=>$accounttype,
            'update'=>1,            
        ]);
    }

    /**
     * Deletes an existing AccountingLedger model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=17;
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            if($ids){
            foreach($ids as $id){
            $this->findModel($id)->delete();
            }
            }
           Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
           return $this->redirect(['index']);
        }
        else
        {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['accounting-ledger/index']); 
        }  
    }
    public function actionExcelExport($detail=null)
    {  
        Yii::$app->view->params['active']=17; 
        $query=AccountingLedger::find()
        ->orderby(['accounting_ledger_id'=>SORT_DESC]);
        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'taxable_ledger', 
            [
                'label'=>'Account Name',
               'attribute' => 'ledger_id',
               'value' => function ($data) {
                $value=AccountingLedger::find()->where(['ledger_id'=>$data->ledger_id])->one();
                return $value?$value->ledger_id:'';
               }
            ],
            [
                'label'=>' Ledger Name',
               'attribute' => 'name',
               'value' => function ($data) {
                $value=AccountingLedger::find()->where(['name'=>$data->name])->one();
                return $value?$value->name:'';
               }
            ],
            'opening_balance',
             
             [
                'label'=>'Account Type',
               'attribute' => 'account_type_id',
               'value' => function ($data) {
                $value=AccountType::find()->where(['account_type_id'=>$data->account_type_id])->one();
                return $value?$value->account_name:'';
               }
            ],
            
            [
                'label'=>'Group',
               'attribute' => 'group_name_id',
               'value' => function ($data) {
                $value=GroupName::find()->where(['group_name_id'=>$data->group_name_id])->one();
                return $value?$value->name_group:'';
               }
            ],
               
               
           
            'alias_name',
            'msic_code',
            // 'status',
            'created_at:date',
            'updated_at:date',
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [ 
                    [
                        'label'=>'Account Name',
                       'attribute' => 'ledger_id',
                       'value' => function ($data) {
                        $value=AccountingLedger::find()->where(['ledger_id'=>$data->ledger_id])->one();
                        return $value?$value->ledger_id:'';
                       }
                    ],
                    [
                        'label'=>' Ledger Name',
                       'attribute' => 'name',
                       'value' => function ($data) {
                        $value=AccountingLedger::find()->where(['name'=>$data->name])->one();
                        return $value?$value->name:'';
                       }
                    ],
                    'opening_balance',
                     [
                        'label'=>'Account Type',
                       'attribute' => 'account_type_id',
                       'value' => function ($data) {
                        $value=AccountType::find()->where(['account_type_id'=>$data->account_type_id])->one();
                        return $value?$value->account_name:'';
                       }
                    ],
                       
                       
                   
                    // 'alias_name',
                     'msic_code',
                    // // 'status',
                     'created_at:date',
                     'updated_at:date',
                ],
            ]);
        }
    if($detail)
    return $exporter->send('accounting_ledger_dtails.xlsx');
    return $exporter->send('accounting_ledger.xlsx');
    }
    public function actionExportPdf()
    {
        Yii::$app->view->params['active']=17;
        $query=AccountingLedger::find()
        ->orderby(['accounting_ledger_id'=>SORT_DESC]);
        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->WriteHTML($this->renderPartial('/accounting-ledger/exportpdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('Accouonting_ledger.pdf', 'I');
    }
    // public function actionDependentDropdown(){
    //     $model1= new GroupName();
         
    //     if($model->load(Yii::$app->request->post())){
    //         echo "<pre>";
    //         print_r($model);
    //         echo "</pre>";
    //         die();
    //     }
         
    //     return $this->render('/accounting-ledger/_form',[
    //             'model1'=>$model1,
    //         ]);
    //     }
        public function actionAccountList(){
            Yii::$app->view->params['active']=17;
            Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            $id = Yii::$app->request->post ( 'id' );

                 $groups = GroupName::find()
                            ->where(['account_type_id' => $id])
                            ->orderBy('group_name_id DESC')
                            ->all();
                    $count = count($groups);    
          
            
            // if($count > 0){
            //     foreach($groups as $group){
            //         echo "<option value='".$group->group_name_id."'>".$group->name_group."</option>";
                    
            //     }
            // }else{
            //     echo "<option>-</option>";
            // }
            return ArrayHelper::map ( $groups , 'group_name_id' , 'name_group' );

         
        }
        public function actionGroupList(){
            Yii::$app->view->params['active']=17;
            Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
            $id = Yii::$app->request->post ( 'id' );

                 $groups = GroupName::find()
                            ->where(['account_type_id' => $id,'is_deleted'=>0])
                            ->orderBy('group_name_id DESC')
                            ->all();
                    $count = count($groups);    
          
            
            // if($count > 0){
            //     foreach($groups as $group){
            //         echo "<option value='".$group->group_name_id."'>".$group->name_group."</option>";
                    
            //     }
            // }else{
            //     echo "<option>-</option>";
            // }
            return ArrayHelper::map ( $groups , 'group_name_id' , 'name_group' );

         
        }
   
    /**
     * Finds the AccountingLedger model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return AccountingLedger the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = AccountingLedger::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
