<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\CallStatus;
use backend\modules\master\models\CallStatusSearch;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;

/**
 * CallStatusController implements the CRUD actions for CallStatus model.
 */
class CallStatusController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    // 'delete' => ['POST'],
                ],
            ],
        ];
    }
    /**
     * Lists all CallStatus models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=14;
        $model = new CallStatus();
        $searchModel = new CallStatusSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        if ($model->load(Yii::$app->request->post()) ) {
            $model->call_status_name=$_POST['CallStatus']['call_status_name'];
            $model->created_at=date('Y-m-d H:i:s');
             $model->updated_at=date('Y-m-d H:i:s');

             if($model->validate() && $model->save(false)){
               Yii::$app->getSession()->setFlash('success', 'Call Status Created Successfully');
                return $this->redirect(['/master/call-status/index']);
            }
            else
            {
                Yii::$app->getSession()->setFlash('error', "Can't create duplicate Call status");
                return $this->redirect(['/master/call-status/index']);
            }
            

        }

        return $this->render('/leads/call-status/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'model' => $model,
        ]);
    }

    /**
     * Displays a single CallStatus model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=14;
        return $this->render('/leads/call-status/view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new CallStatus model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=14;
        $model = new CallStatus();

        if ($model->load(Yii::$app->request->post()) ) {
            $model->created_at=date('Y-m-d H:i:s');
             $model->updated_at=date('Y-m-d H:i:s');
            $model->save(false);
            return $this->redirect(['call-status/view', 'id' => $model->id]);
        }

        return $this->render('/leads/call-status/create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing CallStatus model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=14;
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) ) {
            $model->updated_at=date('Y-m-d H:i:s');

            if($model->validate() && $model->save(false)){
               Yii::$app->getSession()->setFlash('success', 'Call Status updated Successfully');
                return $this->redirect(['/master/call-status/index']);
            }

        }

        return $this->render('/leads/call-status/update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing CallStatus model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            foreach($ids as $id){
            $this->findModel($id)->delete();
            }
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted selected items');
            return $this->redirect(['call-status/index']);
        }
        else
        {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted');
            return $this->redirect(['call-status/index']);
        }
        
    }

    public function actionExportPdf()
    {
        $dataProvider = new ActiveDataProvider([
            'query' => CallStatus::find()->orderBy(['id' => SORT_DESC]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/leads/call-status/export-pdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('call_status.pdf', 'I');
    }

    public function actionExcelExport($detail=null)
    {
        $dataProvider = new ActiveDataProvider([
            'query' => CallStatus::find()->orderBy(['id' => SORT_DESC]),
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'call_status_name',
            'created_at:date',
        ],
    ]);
        }else{
            // print_r("xxxxx");
            // exit();
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
            'call_status_name',
                ],
            ]);
        }
    return $exporter->send('call_status.xlsx');
    }


    /**
     * Finds the CallStatus model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return CallStatus the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = CallStatus::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
