<?php

namespace backend\modules\master\controllers;

use Yii;
use yii\helpers\Url;
use backend\modules\master\models\Category;
use backend\modules\master\models\CategorySearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\web\UploadedFile;
use yii\data\ActiveDataProvider;
use Mpdf\Mpdf;
use yii2tech\spreadsheet\Spreadsheet;
/**
 * CategoryController implements the CRUD actions for Category model.
 */
class CategoryController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Category models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=6;
        $searchModel = new CategorySearch();
        $category=Category::find()->all();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('/product/category/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'category'=>$category,
        ]);
    }

    /**
     * Displays a single Category model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=6;
        return $this->render('/product/category/view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Category model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=6;
        $model = new Category();
        $category=Category::find()->all();
        if ($model->load(Yii::$app->request->post()) ) {
            $model->image = UploadedFile::getInstance($model, 'image');
            if($model->image){
            $nameimage=(new \DateTime())->format('u');
            $model->image->saveAs('setting/category_images/' .$nameimage.'.'.$model->image->extension);
            $model->image=$nameimage. '.' . $model->image->extension;
            }
            if($model->save())
            {
                Yii::$app->getSession()->setFlash('success', 'Created Successfully');
            return $this->redirect(['index']);
            }
        }

        return $this->render('/product/category/_form', [
            'model' => $model,
            'category'=>$category,
            'update'=>0,
        ]);
    }

    /**
     * Updates an existing Category model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=6;
        $model = $this->findModel($id);
        $oldimage=$model->image;
        $category=Category::find()->all();
        if ($model->load(Yii::$app->request->post())) {
            $model->image = UploadedFile::getInstance($model, 'image');
                if($model->image){
                $nameimage=(new \DateTime())->format('u');
                $model->image->saveAs('setting/category_images/' .$nameimage.'.'.$model->image->extension);
                $model->image=$nameimage. '.' . $model->image->extension;
                }else{
                    $model->image=$oldimage;   
                }
                if($model->save()){
                    Yii::$app->getSession()->setFlash('success', 'Updated Successfully');

            return $this->redirect(['index']);
                }
        }

        return $this->render('/product/category/_form', [
            'model' => $model,
            'category'=>$category,
            'update'=>1,
        ]);
    }

    /**
     * Deletes an existing Category model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=6;
        if($id == null)
        {
            $ids=Yii::$app->request->post('data');
            foreach($ids as $id){
            $this->findModel($id)->delete();
            }
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        else
        {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }        
    }
    public function actionUniqueValidation($name,$update=null,$id=null)
    {    
    if($update){
    $count1=Category::find()->where(['name'=>$name,'id'=>$id])->count();
    if($count1)
    return 1;
    }
    else{
        $count=Category::find()->where(['name'=>$name])->count();
        if($count>1)
        return 0;
        else
        return 1;
    }

    }
    public function actionExcelExport($detail=null)
    {
        Yii::$app->view->params['active']=6;
        $dataProvider = new ActiveDataProvider([
            'query' => Category::find(),
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'name',
            'parent_category',
            [
                'attribute'=>'image',
                'label'=>'Image',
                'format' => 'raw',
                'value' => function ($model, $key, $index, $widget) {
                    return "=HYPERLINK(\"" . Url::base(true) .'/setting/category_images/'.$model->image. "\", \"" . Yii::t('app', $model->image) . "\")";
                }
            ],
            'created_at:date',
            'updated_at:date',
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    'name',
                    'parent_category',
                ],
            ]);
        }
        if($detail)
    return $exporter->send('category_details.xlsx');
    return $exporter->send('category.xlsx');

    }
    public function actionExportPdf()
    {
        Yii::$app->view->params['active']=6;
        $dataProvider = new ActiveDataProvider([
            'query' => Category::find(),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->WriteHTML($this->renderPartial('/product/category/exportpdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('category.pdf', 'I');
    }
    /**
     * Finds the Category model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Category the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        Yii::$app->view->params['active']=6;
        if (($model = Category::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
