<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\EnquiryModes;
use backend\modules\master\models\EnquiryModesSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
/**
 * EnquiryModesController implements the CRUD actions for EnquiryModes model.
 */
class EnquiryModesController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    //  'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all EnquiryModes models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=15;
        $model = new EnquiryModes();
        $searchModel = new EnquiryModesSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

         if ($model->load(Yii::$app->request->post()) ) {


            $model->created_at=date('Y-m-d H:i:s');
             $model->updated_at=date('Y-m-d H:i:s');
             if($model->validate() && $model->save(false)){
                Yii::$app->getSession()->setFlash('success', 'Successfully created new Mode of Enquiry');
                 return $this->redirect(['enquiry-modes/index']);
            }
            else
            {
                Yii::$app->getSession()->setFlash('error', "Can't create duplicate Mode of Enquiry");
                 return $this->redirect(['enquiry-modes/index']);
            }
        }

        return $this->render('/leads/enquiry-modes/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'model' => $model,
        ]);
    }

    /**
     * Displays a single EnquiryModes model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=15;
        return $this->render('/leads/enquiry-modes/view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new EnquiryModes model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=15;
        $model = new EnquiryModes();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['enquiry-modes/view', 'id' => $model->id]);
        }

        return $this->render('/leads/enquiry-modes/create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing EnquiryModes model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=15;
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) ) {
            $model->updated_at=date('Y-m-d H:i:s');
            if($model->validate() && $model->save(false)){
                Yii::$app->getSession()->setFlash('success', 'Successfully updated Mode of Enquiry');
            return $this->redirect(['enquiry-modes/index']);
            }            
        }

        return $this->render('/leads/enquiry-modes/update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing EnquiryModes model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            foreach($ids as $id){
            $this->findModel($id)->delete();
            }
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted selected items');
            return $this->redirect(['enquiry-modes/index']);
        }
        else
        {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted');
            return $this->redirect(['enquiry-modes/index']);
        } 
    }
     public function actionExportPdf()
    {
        $dataProvider = new ActiveDataProvider([
            'query' => EnquiryModes::find()->orderBy(['id' => SORT_DESC]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/leads/enquiry-modes/export-pdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('modes_of_enquiry.pdf', 'I');
    }

    public function actionExcelExport($detail=null)
    {
        $dataProvider = new ActiveDataProvider([
            'query' => EnquiryModes::find()->orderBy(['id' => SORT_DESC]),
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'mode_of_enquiry',
            'created_at:date',
        ],
    ]);
        }else{
            // print_r("xxxxx");
            // exit();
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                  'mode_of_enquiry',
                ],
            ]);
        }
    return $exporter->send('modes_of_enquiry.xlsx');
    }



    /**
     * Finds the EnquiryModes model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return EnquiryModes the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = EnquiryModes::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
