<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\GroupName;
use backend\modules\master\models\AccountType;
use backend\modules\master\models\GroupNameSearch;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;
use yii2tech\spreadsheet\Spreadsheet;

/**
 * GroupNameController implements the CRUD actions for GroupName model.
 */
class GroupNameController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all GroupName models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new GroupNameSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single GroupName model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new GroupName model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new GroupName();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            Yii::$app->getSession()->setFlash('success', ' Group Created Successfully');
            return $this->redirect(['/master/group-name/index']);        }

        return $this->render('create', [
            'model' => $model,
            'update'=>'add',
        ]);
    }

    /**
     * Updates an existing GroupName model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
            return $this->redirect(['group-name/index']);
        }

        return $this->render('update', [
            'model' => $model,
            'update'=>'update',
        ]);
    }

    /**
     * Deletes an existing GroupName model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            if($ids){
            foreach($ids as $id){
            $data=$this->findModel($id);
            $data->is_deleted=1;
            $data->updated_by=Yii::$app->user->identity->id;
            $data->save();       
             }
             }
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        else
        {
            $data=$this->findModel($id);
            $data->is_deleted=1;
            $data->updated_by=Yii::$app->user->identity->id;
            $data->save();
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        
    }
    public function actionExportPdf()
    {
        $dataProvider = new ActiveDataProvider([
            'query' => GroupName::find()->where(['is_deleted' => 0]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->WriteHTML($this->renderPartial('/group-name/exportpdf', [
            'dataProvider' => $dataProvider,

        ]));
            
        $mpdf->Output('account_group.pdf', 'I');
    }
    public function actionExcelExport($detail=null)
    {
        $dataProvider = new ActiveDataProvider([
            'query' => GroupName::find()->where(['is_deleted' => 0]),
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'name_group',
            [
                'label'=>'Account Type',
                'value'=>function($data){
                    return AccountType::find()->where(['account_type_id'=>$data->account_type_id])->one()->account_name;
                }
            ],
            [
            'label'=>'Created At',
            'value'=>function($data){
                return date('d-m-Y H:i:s',strtotime($data->created_at));
            }
           ],
           [
            'label'=>'Updated At',
            'value'=>function($data){
                return date('d-m-Y H:i:s',strtotime($data->updated_at));
            }
           ],
                         
        ],
    ]);
        }
        else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
              
                    'name_group',
            [
                'label'=>'Account Type',
                'value'=>function($data){
                    return AccountType::find()->where(['account_type_id'=>$data->account_type_id])->one()->account_name;
                }
            ],                                                      
                ],
            ]);
        }
    if($detail){
     ob_clean();
     ob_start();
    return $exporter->send('account_group_details.xlsx');
    ob_flush();
    }
    ob_clean();
     ob_start();
    return $exporter->send('account_group.xlsx');
    ob_flush();

    }

    /**
     * Finds the GroupName model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return GroupName the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = GroupName::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
