<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\Leads;
use backend\modules\master\models\LeadsSearch;
use backend\modules\master\models\Category;
use backend\modules\master\models\Product;
use backend\modules\master\models\BusinessType;
use backend\modules\master\models\CallStatus;
use backend\modules\master\models\EnquiryModes;
use common\models\User;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\Json;
use common\components\AccessRule;
use yii\filters\AccessControl;
use backend\modules\master\models\LeadsEnquiry;
use Mpdf\Mpdf;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use yii\helpers\ArrayHelper;
use DateTime;

/**
 * LeadsController implements the CRUD actions for Leads model.
 */
class LeadsController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Leads models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=18;
        $model = new Leads();
        $searchModel = new LeadsSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        if($model->load(Yii::$app->request->get())){
            $data=Yii::$app->request->post();
            $query=Leads::find()->where(['is_disqualify' =>1])->orderBy([
                'id' => SORT_DESC,
              ]);
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
            ]);
           //echo "<pre>";print_r($model);die();
            if($model->search!=""){
                $query
                ->andFilterWhere(['like', 'name', $model->search])
                ->orFilterWhere(['like', 'email', $model->search])
                ->orFilterWhere(['like', 'city', $model->search])
                ->orFilterWhere(['like', 'mobile', $model->search]);
            }
            else {
               // echo "<pre>";print_r($model);die();
                if($model->lead_type=="postponed"){
                    $query=Leads::find()->where(['not',['postponed_date'=>null]])->orderBy([
                'id' => SORT_DESC,
              ]);
                }
                if($model->lead_status=="Disqualified"){
                    $query=Leads::find()->where(['is_disqualify' =>0])->orderBy([
                'id' => SORT_DESC,
              ]);
                     }
                $query->andFilterWhere(['between', 'lead_date', $model->from_date,$model->to_date])
                 ->andFilterWhere(['like', 'enquiry_no', $model->enquiry_no])
                ->andFilterWhere(['like', 'business_type', $model->business_type])
                ->andFilterWhere(['like', 'userid', $model->userid])
                 ->andFilterWhere(['like', 'city', $model->city])
                 ->andFilterWhere(['like', 'lead_executive', $model->lead_executive])
                 ->andFilterWhere(['like', 'product_model', $model->product_model])
                 ->andFilterWhere(['like', 'status', $model->status]);
            }
            // else{
            //     $dataProvider = new ActiveDataProvider(['query' => $query]);
    
            // }
            $dataProvider = new ActiveDataProvider(['query' => $query]);

        }

        //$dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        return $this->render('/leads/leads/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'model' => $model,
        ]);
    }

    /**
     * Displays a single Leads model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=18;
        return $this->render('/leads/leads/view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Leads model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate($id=null,$from=null)
    {

        Yii::$app->view->params['active']=18;
        $model = new Leads();
        $last_lead=Leads::find()->orderby(['id' => SORT_DESC])->one();
      

        if($id)
        {
            if($from=='leads-enquiry')
            {
                $enquiry_details = LeadsEnquiry::find()->where(['id'=>$id])->one();
                $model->name = $enquiry_details->name;
                $model->email = $enquiry_details->email;
                $model->mobile = $enquiry_details->mobile;
                $model->city = $enquiry_details->city;
                $model->address = $enquiry_details->address;
                
                $business= BusinessType::find()->where(['id'=>$enquiry_details->business_type])->one();
                $model->business_type = $business->business_type_name;
                $status = CallStatus::find()->where(['id'=>$enquiry_details->call_status])->one();
                $model->status = $status->call_status_name;
                $model->remark = $enquiry_details->messages;
                $mode = EnquiryModes::find()->where(['id'=>$enquiry_details->mode_of_enquiry])->one();
                $model->mode_enquiry =  $mode->mode_of_enquiry;
            }
        }
        if ($model->load(Yii::$app->request->post()) ) {
            $data= Yii::$app->request->post();
           // echo "<pre>";print_r($data);die();

            if($from=='leads-enquiry')
            {
                $enquiry = LeadsEnquiry::find()->where(['id'=>$id])->one();
                $enquiry->converted_to = "Leads";
                $enquiry->save(false);
            }
            $model->lead_date= date('Y-m-d');
            if(isset($data['Leads']['product_make'])){
            $model->product_make = $data['Leads']['product_make'];
            }
            if(isset($data['Leads']['product_model'])){
                $model->product_model = $data['Leads']['product_model'];
                }
            if(isset($data['product-div'])  && $data['product-div']!=1){
            $model->product_model = $data['product-div'];
            }
            if(isset(Yii::$app->user->identity->id)){
                $user_id=Yii::$app->user->identity->id;
                $model->userid= $user_id;

            }
            if($model->save(false)){
                Yii::$app->getSession()->setFlash('success', 'Successfully created the leads');
                return $this->redirect(['index']);
            }
            else{
                print_r($model->getErrors());die();
            }
        }

        return $this->render('/leads/leads/create', [
            'model' => $model,
            'last_lead'=>$last_lead,
        ]);
    }

    /**
     * Updates an existing Leads model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=18;
        $last_lead=Leads::find()->orderby(['id' => SORT_DESC])->one();
        $model = $this->findModel($id);
 
        if ($model->load(Yii::$app->request->post()) && $model->save(false)) {
            return $this->redirect(['leads/index']);
        }

        return $this->render('/leads/leads/update', [
            'model' => $model,
            
        ]);
    }
    public function actionPostpond($id)
    {
        Yii::$app->view->params['active']=18;
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post())) {
           $data= Yii::$app->request->post();
           $model->status="postponed";
           //echo "<pre>";print_r($data['Leads']['postponed_date']);die();
            $model->save(false);
            return $this->redirect(['index']);
        }

        return $this->render('/leads/leads/postpond', [
            'model' => $model,
        ]);
    }
    public function actionGetdata()
    {
        Yii::$app->view->params['active']=18;
       $name=Yii::$app->request->get()['parent_category'];
       $category= Category::find()->where(['name' =>$name])->one();
       $product= Product::find()->where(['category' => $category->id])->all();
       $data=[];
       foreach ($product as $key => $value) {
           array_push($data,$value->product_name);
       }
       echo JSON::encode($data);
    }

    /**
     * Deletes an existing Leads model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=18;
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            foreach($ids as $id){
            $this->findModel($id)->delete();
            }
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted selected leads');
            return $this->redirect(['leads/index']); 
        }
        else
        {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['leads/index']);
        }
        
    }
    public function actionDisqualify($id)
    {
        Yii::$app->view->params['active']=18;
        $model = $this->findModel($id);
        $model->is_disqualify = 0;
        Yii::$app->getSession()->setFlash('success', 'Disqualify Successfully');
        $model->save(false);

        return $this->redirect(['index']);
    }
    public function actionExportPdf()
    {
        Yii::$app->view->params['active']=18;
        $dataProvider = new ActiveDataProvider([
            'query' => Leads::find()->where(['is_disqualify' =>1]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->SetFont('Font family', '', 20);
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/leads/leads/export-pdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('leads.pdf', 'D');
    }

    public function actionExcelExport($detail=null)
    {
        Yii::$app->view->params['active']=18;
        $dataProvider = new ActiveDataProvider([
            'query' => Leads::find()->where(['is_disqualify' =>1]),
        ]);
        if($detail){
        $exporter = new Spreadsheet([
            'dataProvider' => $dataProvider,
            'columns' => [
                [
                    'header'=> 'Enquiry',
                    'value'=>'enquiry_no',
                ],
                [
                    'header'=> 'Appointment Date',
                    'value'=>'appoint_date',
                ],
                [
                    'header'=> 'Set Reminder For',
                    'value'=>'set_reminder',
                ],
                [
                    'header'=> 'Name',
                    'value'=>'name',
                ],
                [
                    'header'=> 'Email',
                    'value'=>'email',
                ],
                [
                    'header'=> 'Mobile',
                    'value'=>'mobile',
                ],
                [
                    'header'=> 'City',
                    'value'=>'city',
                ],
                [
                    'header'=> 'Address',
                    'value'=>'address',
                ],
                [
                    'header'=> 'Status',
                    'value'=>'status',
                ],
                [
                    'header'=> 'Bussiness Type',
                    'value'=>'business_type',
                ],
                [
                    'header'=> 'Product Make',
                    'value'=>'product_make',
                ],
                [
                    'header'=> 'Product',
                    'value'=> function($data){
                        return $data->product_model != null ? $data->product_model : '';
                    } 
                ],
                [
                    'header'=> 'Assigned',
                    'value'=> function($data){

                        $user= User::find()->where(['id' =>$data->userid])->one();
                        if(isset($user)){
                        return $user->username;
                        }
                        else{
                            return "";
                        }
                    }                     
                ],
                [
                    'header'=> 'Mode of Enquiry',
                    'value'=>'mode_enquiry',
                ],
                [
                    'header'=> 'Remark',
                    'value'=>'remark',
                ],
                 [
                        'header'=> 'Postpond?',
                        'value'=>'status',
    
                    ],
                    [
                        'header'=> 'Postpond Date',
                        'value'=>'postponed_date',
    
                    ],
                    [
                        'header'=> 'Postpond Note',
                        'value'=>'postponed_note',
    
                    ],
                    [
                        'header'=> 'Age of prospect',
                        'value'=> function($data){
                            $bday = new DateTime('1.10.2020');//dd.mm.yyyy
                            $today = new DateTime('00:00:00'); // Current date
                            $diff = $today->diff($bday);
                            return  $diff->y." year ".$diff->m." month ".$diff->d." days ";
                        } 
                    ],
                    [
                        'header'=> 'Status?',
                        'value'=>'status',
                    ],
                    [
                        'header'=> 'Converted',
                        'value'=>'status',
    
                    ],
                
            ],
        ]);
        }else{
            // print_r("xxxxx");
            // exit();
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    [
                        'header'=> 'Enquiry',
                       // 'value'=>'enquiry_no',
                       'value'=> function($data){
                           return $data->enquiry_no != null ? $data->enquiry_no : '';
                       }
    
                    ],
                    [
                        'header'=> 'Lead Date',
                        'value'=> function($data){
                            return $data->created_at != null ? $data->created_at : '';
                        }               
                     ],
                    [
                        'header'=> 'Name',
                        'value'=>'name',
                    ],
                    [
                        'header'=> 'Product',
                        'value'=> function($data){
                            return $data->product_model != null ? $data->product_model : '';
                        } 
                    ],
                    [
                        'header'=> 'Remark',
                        'value'=>'remark',
                    ],
                    [
                        'header'=> 'Assigned',
                        'value'=> function($data){
    
                            $user= User::find()->where(['id' =>$data->userid])->one();
                            if(isset($user)){
                            return $user->username;
                            }
                            else{
                                return "";
                            }
                        }                     
                    ],
                    [
                        'header'=> 'Email',
                        'value'=>'email',
                    ],
                    [
                        'header'=> 'City',
                        'value'=>'city',
                    ],
                    [
                        'header'=> 'Postpond?',
                        'value'=>'status',
    
                    ],
                    [
                        'header'=> 'Age of prospect',
                        'value'=> function($data){
                            $bday = new DateTime('1.10.2020');//dd.mm.yyyy
                            $today = new DateTime('00:00:00'); // Current date
                            $diff = $today->diff($bday);
                            return  $diff->y." year ".$diff->m." month ".$diff->d." days ";
                        } 
                    ],
                    [
                        'header'=> 'Status?',
                        'value'=>'status',
                    ],
                    [
                        'header'=> 'Converted',
                        'value'=>'status',
    
                    ],
                ],
            ]);
        }
    return $exporter->send('leads.xlsx');
    }

    /**
     * Finds the Leads model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Leads the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Leads::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionProductdetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $product=Product::find()->limit(10)->all();
        else
        $product=Product::find()->where(["like","product_name",$searchTerm])->limit(10)->all();

        $productlist=ArrayHelper::map($product, 'product_name', 'product_name');
        return json_encode($productlist);
    }
}
