<?php

namespace backend\modules\master\controllers;

use Yii;
use yii\helpers\ArrayHelper;
use backend\modules\master\models\ProductDiscount;
use backend\modules\master\models\DiscountSetting;
use backend\modules\master\models\DiscountCustomer;
use backend\modules\master\models\Category;
use backend\modules\master\models\ProductDiscountSearch;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\base\Model;
use Mpdf\Mpdf;
use yii2tech\spreadsheet\Spreadsheet;
/**
 * ProductDiscountController implements the CRUD actions for ProductDiscount model.
 */
class ProductDiscountController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all ProductDiscount models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=8;
        $searchModel = new ProductDiscountSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('/product/product-discount/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single ProductDiscount model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=8;
        return $this->render('/product/product-discount/view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new ProductDiscount model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=8;
        $model = new ProductDiscount();
        $count1 = count(Yii::$app->request->post('DiscountSetting', []));
        $count2 = count(Yii::$app->request->post('DiscountCustomer', []));
        $category=Category::find()->all();
        $dis_customer = [new DiscountCustomer()];
        $dis_setting = [new DiscountSetting()];
        for($i = 1; $i < $count1; $i++) {
            $dis_setting[] = new DiscountSetting();
        }
        for($i = 1; $i < $count2; $i++) {
            $dis_customer[] = new DiscountCustomer();
        }
        if ($model->load(Yii::$app->request->post()) ) {
            if($model->save()){
                if (Model::loadMultiple($dis_customer, Yii::$app->request->post()) ) {
                    foreach ($dis_customer as $dis_customerind) {
                        $dis_customerind->discount_id=$model->id;
                        $dis_customerind->save(false);
        
                    }
                }
                if (Model::loadMultiple($dis_setting, Yii::$app->request->post()) ) {
                    foreach ($dis_setting as $dis_settingind) {
                        $dis_settingind->discount_id=$model->id;
                        $dis_settingind->save(false);
        
                    }
                }
                Yii::$app->getSession()->setFlash('success', 'Created Successfully');
                return $this->redirect(['index']);
            }
        }

        return $this->render('/product/product-discount/_form', [
            'model' => $model,
            'update'=>0,
            'dis_customer'=>$dis_customer,
            'dis_setting'=>$dis_setting,
            'category'=>$category,
        ]);
    }

    /**
     * Updates an existing ProductDiscount model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=8;
        $model = $this->findModel($id);
        $category=Category::find()->all();
        $dis_customer = DiscountCustomer::find()->where(['discount_id'=>$id])->all();
        $dis_setting = DiscountSetting::find()->where(['discount_id'=>$id])->all();
        $oldsettingid=ArrayHelper::map($dis_setting, 'id', 'id');
        $oldcustomerid=ArrayHelper::map($dis_customer, 'id', 'id');
        if ($model->load(Yii::$app->request->post()) ) {
            $newsettingid=ArrayHelper::map(Yii::$app->request->post('DiscountSetting'), 'id', 'id');
            $newcustomerid=ArrayHelper::map(Yii::$app->request->post('DiscountCustomer'), 'id', 'id');
            $diffsettingid=array_diff($oldsettingid,$newsettingid);
            $diffcustomerid=array_diff($oldcustomerid,$newcustomerid);
            foreach(Yii::$app->request->post('DiscountSetting') as $value){
                if(!$value['id'])
                $dis_setting[]=new DiscountSetting();
            }
            foreach(Yii::$app->request->post('DiscountCustomer') as $value){
                if(!$value['id'])
                $dis_customer[]=new DiscountCustomer();
            }
            DiscountSetting::deleteAll(['in','id',$diffsettingid]);
            DiscountCustomer::deleteAll(['in','id',$diffcustomerid]);
            if($model->save()){
                if (Model::loadMultiple($dis_customer, Yii::$app->request->post()) ) {
                    foreach ($dis_customer as $dis_customerind) {
                        $dis_customerind->discount_id=$model->id;
                        $dis_customerind->save(false);
        
                    }
                }if (Model::loadMultiple($dis_setting, Yii::$app->request->post()) ) {
                    foreach ($dis_setting as $dis_settingind) {
                        $dis_settingind->discount_id=$model->id;
                        $dis_settingind->save(false);
        
                    }
                }
                Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
                return $this->redirect(['index']);
            }
        }

        return $this->render('/product/product-discount/_form', [
            'model' => $model,
            'update'=>1,
            'dis_customer'=>$dis_customer,
            'dis_setting'=>$dis_setting,
            'category'=>$category,
        ]);
    }

    /**
     * Deletes an existing ProductDiscount model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        Yii::$app->view->params['active']=8;
        $this->findModel($id)->delete();
        DiscountCustomer::deleteAll(['discount_id'=>$id]);
        DiscountSetting::deleteAll(['discount_id'=>$id]);
        Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
        return $this->redirect(['index']);
    }
    public function actionDeleteDiscount()
    {
        Yii::$app->view->params['active']=8;
        $ids=Yii::$app->request->post('data');
        ProductDiscount::deleteAll(['in','id',$ids]);
        DiscountCustomer::deleteAll(['in','discount_id',$ids]);
        DiscountSetting::deleteAll(['in','discount_id',$ids]);
        Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
        return $this->redirect(['index']);
    }
    public function actionExcelExport($detail=null)
    {
        Yii::$app->view->params['active']=8;
        $dataProvider = new ActiveDataProvider([
            'query' => ProductDiscount::find(),
        ]);
        if(!$detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'dis_type',
            'dis_name',
            'start_date',
            'till_date',
            [
                'label' => 'Status',
                'attribute' => 'status',
                'value' => function ($data) {
                    return $data->status?'Active':'Inactive';
                }
            ],
            
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    'dis_type',
                    'dis_name',
                    'id_code',
                    'start_date',
                    'till_date',
                    [
                        'label' => 'Status',
                        'attribute' => 'status',
                        'value' => function ($data) {
                            return $data->status?'Active':'Inactive';
                        }
                    ],
                    'access',
                    [
                        'label' => 'Category Include/Exclude',
                        'attribute' => 'category_include_exclude',
                        'value' => function ($data) {
                            return $data->category_include_exclude==1?'Include':'Exclude';
                        }
                    ],
                    'created_at:date',
                    'updated_at:date',
                ],
            ]);
        }
    if($detail)
    return $exporter->send('product_discount_details.xlsx');
    return $exporter->send('product_discount.xlsx');
    }
    public function actionExportPdf()
    {
        Yii::$app->view->params['active']=8;
        $dataProvider = new ActiveDataProvider([
            'query' => ProductDiscount::find(),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->WriteHTML($this->renderPartial('/product/product-discount/exportpdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('product_discount.pdf', 'D');
    }
    /**
     * Finds the ProductDiscount model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ProductDiscount the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = ProductDiscount::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
