<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\PurchaseTemplates;
use backend\modules\master\models\PurchaseTemplatesSearch;
use backend\modules\master\models\PurchaseOrder;
use backend\modules\users\models\PurchasePayment;
use backend\modules\users\models\DebitNote;
use backend\modules\users\models\PurchaseBill;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;


/**
 * PurchaseTemplatesController implements the CRUD actions for PurchaseTemplates model.
 */
class PurchaseTemplatesController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all PurchaseTemplates models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new PurchaseTemplatesSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $dataProvider->sort->defaultOrder=['id'=>SORT_DESC];

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single PurchaseTemplates model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new PurchaseTemplates model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new PurchaseTemplates();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['index']);
            
        }

        return $this->render('create', [
            'model' => $model,
            'update'=> 0,
        ]);
    }

    /**
     * Updates an existing PurchaseTemplates model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = PurchaseTemplates::find()->where(['id'=>$id])->one();
        $oldname = $model->template_name;
        $oldsetting = $model->setting_type;
        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            if($oldsetting =='Purchase Order')
            {
            $allpurchase = PurchaseOrder::find()->where(['setting_type'=>$oldname])->all();
                foreach($allpurchase as $val)
                {
                $val->setting_type = $model->template_name;
                $val->save(false);
                }

            }
            elseif($oldsetting =='Payment')
            {
                $allpurchase = PurchasePayment::find()->where(['setting_type'=>$oldname])->all();
                foreach($allpurchase as $val)
                {
                $val->setting_type = $model->template_name;
                $val->save(false);
                }
            }
            elseif($oldsetting =='Debit Note')
            {
                $allpurchase = DebitNote::find()->where(['setting_type'=>$oldname])->all();
                foreach($allpurchase as $val)
                {
                $val->setting_type = $model->template_name;
                $val->save(false);
                }
            }
            else
            {
                $allpurchase = PurchaseBill::find()->where(['setting_type'=>$oldname])->all();
                foreach($allpurchase as $val)
                {
                $val->setting_type = $model->template_name;
                $val->save(false);
                }
            }
            

            return $this->redirect(['index']);
        }

        return $this->render('_form', [
            'model' => $model,
            'update'=> 1,
        ]);
    }

    /**
     * Deletes an existing PurchaseTemplates model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        
        if($id==null)
        {
           $ids=Yii::$app->request->post('data');
          // print_r( $ids); die();
            if($ids)
            {
                foreach ($ids as $id) { 
                   $data=$this->findModel($id)->delete();
                }
            }
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted selected items');
            return $this->redirect(['purchase-templates/index']);
        }
        else
        {
            $dd = PurchaseTemplates::find()->where(['id'=>$id])->one();
            $dd->delete();
            // $data=$this->findModel($id);
             Yii::$app->getSession()->setFlash('success', 'Successfully deleted'); 
            return $this->redirect(['purchase-templates/index']);
        }
        
    }

    public function actionFindsetting($setting=null,$template=null)
    {
        $setting=Yii::$app->request->post('setting');
        $template=Yii::$app->request->post('template');
        $find = PurchaseTemplates::find()->where(['setting_type'=>$setting])->andWhere(['template_name'=>$template])->count();
        if($find>0)
        {
            echo '0';
        }
        else
        {
            echo '1';
        }
    }

    public function actionFindusedtemplates()
    {   
        // print_r($setting);die();
        $setting=Yii::$app->request->post('setting');
        $from=Yii::$app->request->post('from');
       $findsetting = PurchaseTemplates::find()->where(['id'=>$setting])->one();
       if($findsetting->setting_type=='Purchase Order')
       {
          $find = PurchaseOrder::find()->where(['setting_type'=>$findsetting->template_name])->andWhere(['is_deleted'=>0])->count();
          if($find>0)
        {
            echo '0'; 
        }
        else
        {
            echo '1';
        }
       }
       else if($findsetting->setting_type=='Debit Note')
       {
        $find = DebitNote::find()->where(['setting_type'=>$findsetting->template_name])->andWhere(['is_deleted'=>0])->count();
        if($find>0)
        {
            echo '0'; 
        }
        else
        {
            echo '1';
        }
       }
       else if($findsetting->setting_type=='Payment')
       {
        $find = PurchasePayment::find()->where(['setting_type'=>$findsetting->template_name])->andWhere(['is_deleted'=>0])->count();
        if($find>0)
        {
            echo '0'; 
        }
        else
        {
            echo '1';
        }
       }
       else
       {
        $find = PurchaseBill::find()->where(['setting_type'=>$findsetting->template_name])->andWhere(['is_deleted'=>0])->count();  
        if($find>0)
        {
            echo '0'; 
        }
        else
        {
            echo '1';
        }
       }
       
        
    }

    /**
     * Finds the PurchaseTemplates model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return PurchaseTemplates the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = PurchaseTemplates::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
