<?php

namespace backend\modules\master\controllers;

use Yii;
use backend\modules\master\models\Taxtype;
use backend\modules\master\models\TaxtypeSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;


/**
 * TaxtypeController implements the CRUD actions for Taxtype model.
 */
class TaxtypeController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    // 'delete' => ['POST'],
                ],
            ],
        ];
    }
    public static $tax_type=[
    "Service Tax"=>"Service tax",
    "VAT Tax"=>"VAT Tax",
    "TDS"=>"TDS",
    "ED"=>"ED",
    "Basic Duty"=>"Basic Duty",
    "CESS"=>"CESS",
    "CST"=>"CST",
    "Import/Export"=>"Import/Export",
    "Custom Tax"=>"Custom tax",
    "CGST"=>"CGST",
    "SGST"=>"SGST",
    "IGST"=>"IGST",
    ];
    /**
     * Lists all Taxtype models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=11;
        $searchModel = new TaxtypeSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('/tax/taxtype/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Taxtype model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=11;
        return $this->render('/tax/taxtype/view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Taxtype model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=11;
        $model = new Taxtype();

        if ($model->load(Yii::$app->request->post())) {
             $model->created_at=date('Y-m-d H:i:s');
             $model->updated_at=date('Y-m-d H:i:s');

             if($model->validate() && $model->save(false)){
                Yii::$app->getSession()->setFlash('success', 'Successfully created new Tax Type');
                return $this->redirect(['taxtype/index']);
            }
            // else
            // {
            //     Yii::$app->getSession()->setFlash('error', 'Tax Type already exist Choose a new tax');
            //     return $this->redirect(['taxtype/create']);
            // }

        }

        return $this->render('/tax/taxtype/create', [
            'model' => $model,
            'taxtype'=> TaxtypeController::$tax_type,
        ]);
    }

    /**
     * Updates an existing Taxtype model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=11;
        $model = $this->findModel($id);
        if ($model->load(Yii::$app->request->post())) {
             // $current_date=date('Y-m-d H:i', time());
            $model->updated_at=date('Y-m-d H:i:s');

            if($model->validate() && $model->save(false)){
            Yii::$app->getSession()->setFlash('success', 'Successfully updated');
            return $this->redirect(['taxtype/index']);
        }

        }

        return $this->render('/tax/taxtype/update', [
            'model' => $model,
            'taxtype'=> TaxtypeController::$tax_type,
        ]);
    }

    /**
     * Deletes an existing Taxtype model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            foreach($ids as $id){
            $this->findModel($id)->delete();
            }
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted selected Tax Types');
            return $this->redirect(['taxtype/index']);
        }
        else
        {
            $this->findModel($id)->delete();
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted');
            return $this->redirect(['taxtype/index']);   
        }
        
    }
    public function actionExportPdf()
    {
        $dataProvider = new ActiveDataProvider([
            'query' => Taxtype::find()->orderBy(['id' => SORT_DESC]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/tax/taxtype/export-pdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('tax_types.pdf', 'I');
    }

    public function actionExcelExport($detail=null)
    {
        $dataProvider = new ActiveDataProvider([
            'query' => Taxtype::find()->orderBy(['id' => SORT_DESC]),
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            'tax_type',
            'tax_type_name',
            'created_at',
            'updated_at'
        ],
    ]);
        }else{
            // print_r("xxxxx");
            // exit();
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                  'tax_type',
                  'tax_type_name',
                ],
            ]);
        }
    return $exporter->send('tax_types.xlsx');
    }
    /**
     * Finds the Taxtype model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Taxtype the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Taxtype::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
