<?php

namespace backend\modules\master\models;

use Yii;
use backend\modules\master\models\AccountType;

/**
 * This is the model class for table "accounting_tax".
 *
 * @property int $accounting_tax_id
 * @property string $tax_name
 * @property string $tax_class
 * @property string|null $abbreviation
 * @property string $tax_type
 * @property string $tax_sub_type
 * @property string|null $supply_type
 * @property string $ledger_name
 * @property float $tax_rate
 * @property int $status
 * @property int $is_deleted
 * @property string $created_at
 * @property string $updated_at
 */
class AccountingTax extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'accounting_tax';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            // [['tax_name', 'tax_class', 'tax_type', 'tax_sub_type', 'ledger_name', 'tax_rate'], 'required'],
            [['tax_name', 'tax_class', 'tax_type', 'tax_sub_type', 'tax_rate'], 'required'],
            [['is_deleted','status'], 'integer'],
            [['tax_name', 'tax_rate'], 'unique', 'targetAttribute' => ['tax_name', 'tax_rate'] ,'message'=>'Combination of Tax name and Tax rate should be Unique'],
            // [['tax_rate'], 'number','min'=>0,'max'=>100,'message'=> 'Integer or decimal value upto 3 digit is required'],
            [['created_at', 'updated_at','status'], 'safe'],
            [['tax_name', 'tax_class', 'abbreviation', 'tax_sub_type', 'supply_type', 'ledger_name','tax_type'], 'string', 'max' => 50],
        ];
    }
    public static function getHierarchy() {
        $options = [];
        
        $ledgers = AccountType::find()->all();
        foreach($ledgers as $id => $p) {
            $children = AccountingLedger::find()->where(["account_type_id"=>$p->account_type_id])->all();
            $child_options = [];
            foreach($children as $child) {
                $child_options[$child->ledger_id] = $child->ledger_id;
            }
            $options[$p->account_name] = $child_options;
        }
        return $options;
    }
    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'accounting_tax_id' => 'Accounting Tax ID',
            'tax_name' => 'Tax Name',
            'tax_class' => 'Tax Class',
            'abbreviation' => 'Abbreviation',
            'tax_type' => 'Tax Type',
            'tax_sub_type' => 'Tax Sub Type',
            'supply_type' => 'Supply Type',
            'ledger_name' => 'Ledger Name',
            'tax_rate' => 'Tax Rate(%)',
            'status' => 'Status',
            'is_deleted' => 'Is Deleted',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }
}
