<?php

namespace backend\modules\master\models;

use Yii;

/**
 * This is the model class for table "company_profile".
 *
 * @property int $id
 * @property string|null $company_name
 * @property string|null $email_id
 * @property string $f_year_start
 * @property string $f_year_end
 * @property string|null $f_year
 * @property int|null $gstin_no
 * @property int|null $pan_no
 * @property int|null $mobile_no
 * @property int|null $org_contact_no
 * @property string|null $address
 * @property string $created_at
 * @property string $updated_at
 */
class CompanyProfile extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'company_profile';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['f_year_start', 'f_year_end','gstin_no','org_contact_no','pan_no','company_name','email_id'], 'required'],
            [['f_year_start', 'f_year_end', 'created_at', 'updated_at'], 'safe'],
            [['address'], 'string'],
            [['company_name', 'f_year','account_no','ifsc_code','bank_name'], 'string', 'max' => 256],
            [['email_id'], 'email'],
            [['gstin_no'], 'string','min'=>15, 'max' => 15],
            [['gstin_no'], 'match' ,'pattern'=>'/^([0-9]){2}([A-Z]){4}([0-9A-Z]){1}([0-9]){4}([A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}?$/', 'message'=> 'GSTN Format should be like  12ABCDE1234Z1Z1.'],

            [['pan_no',], 'string','min'=>10, 'max' => 10],
            [['pan_no'], 'match' ,'pattern'=>'/^([A-Z]){4}([0-9A-Z]){1}([0-9]){4}([A-Z]){1}?$/', 'message'=> 'PAN Format should be like  ABCDE1234Z.'],

            [['f_year_end'], 'compare' ,'compareAttribute' => 'f_year_start','operator'=>'>=', 'message'=> 'To Date must be equal/greater than Financial starting date'],
             ['f_year_end', 'required', 'when' => function($model) {
                return $model->f_year_start;
            },'whenClient' => "function (attribute, value) {
                return $('#companyprofile-f_year_start').val();
            }"],
            ['f_year_start', 'required', 'when' => function($model) {
                return $model->f_year_end;
            },'whenClient' => "function (attribute, value) {
                return $('#companyprofile-f_year_end').val();
            }"], 

             [['mobile_no'], 'string','min'=>10, 'max' => 10],
             [['mobile_no'], 'match' ,'pattern'=>'/^[0-9]+$/u', 'message'=> 'Mobile No can Contain only numeric characters.'], 
             [['mobile_no'],'validatemob'],
             [['org_contact_no'], 'string','min'=>10, 'max' => 10],
             [['org_contact_no'], 'match' ,'pattern'=>'/^[0-9]+$/u', 'message'=> 'Contact No can Contain only numeric characters.'], 
             [['org_contact_no'],'validatecontact'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'company_name' => 'Company Name',
            'email_id' => 'Email ID',
            'f_year_start' => 'F Year Start',
            'f_year_end' => 'F Year End',
            'f_year' => 'F Year',
            'gstin_no' => 'Gstin No',
            'pan_no' => 'Pan No',
            'mobile_no' => 'Mobile No',
            'org_contact_no' => 'Org Contact No',
            'address' => 'Address',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    public function validatemob()
    { 
       $words = $this->mobile_no;
       $first_l = $words[0];
       if($first_l==0)
       {
         $this->addError('mobile_no','Enter valid mobile number');
         return false;
       }

        
    }
    public function validatecontact()
    { 
       $words = $this->org_contact_no;
       $first_l = $words[0];
       if($first_l==0)
       {
         $this->addError('org_contact_no','Enter valid contact number');
         return false;
       }

        
    }
}
