<?php

namespace backend\modules\reports\controllers;

use Mpdf\Mpdf;
use Yii;
use yii\helpers\ArrayHelper;
use backend\modules\reports\models\CashFlowReport;
use backend\modules\users\models\PurchasePayment;
use backend\modules\users\models\ReceiptDetails;
use yii\data\ActiveDataProvider;
use backend\modules\master\models\Vendor;
use backend\modules\master\models\Customer;
use PHPExcel;
use PHPExcel_IOFactory;
class CashFlowController extends \yii\web\Controller
{
    public function actionIndex($print=null,$excel=null,$pdf=null)
    {
        $model = new CashFlowReport();
        $now = new \DateTime('now');
        $month = $now->format('m');
        $year = $now->format('Y');
        $query = CashFlowReport::find();
        $query->andFilterWhere(['month_year'=>$year.'-' .$month]);
        if(Yii::$app->request->get()) {
            $query = CashFlowReport::find();
            $model->load(Yii::$app->request->queryParams);
            $query->andFilterWhere(['between','month_year', $model->from,$model->to]);
        }
        $dataProvider = new ActiveDataProvider([
            'query'=>$query,
        ]);
        if($print!=null)
        {
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
                'dataProvider' => $dataProvider,
                'model'=>$model,

            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'I');
        }
        if($pdf!=null)
        {
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
                'dataProvider' => $dataProvider,
                'model'=>$model,

            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'D');
        }
        if($excel !=null){
            $this->actionExcelReport($dataProvider);
        }

        return $this->render('index',[
            'dataProvider'=>$dataProvider,
            'model'=> $model,
        ]);
    }
    public function actionDetailedTransaction($first_day,$last_day,$print=null,$pdf=null,$excel=null)
    {
        $model = new CashFlowReport();
        
        if($model->load(Yii::$app->request->queryParams))
        {
            $query1 = ReceiptDetails::find()->where(['between','receipt_date',$model->from_date,$model->to_date]);
            $query2 = PurchasePayment::find()->where(['between','date',$first_day,$last_day])
                                             ->andWhere(['=','is_deleted',0]);     
        }else{
            $query1 = ReceiptDetails::find()->where(['between','receipt_date',$first_day,$last_day]);
            $query2 = PurchasePayment::find()->where(['between','date',$first_day,$last_day])
                                             ->andWhere(['=','is_deleted',0]);        
        }
        
        $dataProvider1 = new ActiveDataProvider([
            'query'=>$query1,
            'pagination'=>false,
        ]);
        $dataProvider2 = new ActiveDataProvider([
            'query'=>$query2,
            'pagination'=>false,

        ]);   
        if($print!=null)
        {
            $dataProvider1->pagination = false;
            $dataProvider2->pagination = false;
            $mpdf = new mPDF;
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/detail-export', [
                'dataProvider1'=>$dataProvider1,
                'dataProvider2'=>$dataProvider2,
                'model'=>$model,

            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'I');
        }
        if($pdf!=null)
        {
            $dataProvider1->pagination = false;
            $dataProvider2->pagination = false;
            $mpdf = new mPDF;
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/cash-flow/detail-export', [
                'dataProvider1'=>$dataProvider1,
                'dataProvider2'=>$dataProvider2,
                'model'=>$model,

            ]));
            $mpdf->Output('Cash_flow_Report.pdf', 'D');
        }
        if($excel !=null){
            $this->actionDetailExcelReport($dataProvider1,$dataProvider2);
        }
        return $this->render('detailed-transaction',[
            'dataProvider1'=>$dataProvider1,
            'dataProvider2'=>$dataProvider2,
            // 'month_year'=> $month_year,
            'model'=> $model,

        ]);
    }

    public function actionExportPrint()
    {
        $model = new CashFlowReport(); 
        $query = CashFlowReport::find();
        $now = new \DateTime('now');
        $month = $now->format('m');
        $year = $now->format('Y');
        $query->andFilterWhere(['month_year'=>$year.'-' .$month]);
        $dataProvider = new ActiveDataProvider([
            'query'=>$query,
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
            'dataProvider' => $dataProvider,
            'model'=>$model,

        ]));
        $mpdf->Output('Cash_flow_Report.pdf', 'I');
    }
    public function actionExportPdf()
    {
        $model = new CashFlowReport(); 
        $query = CashFlowReport::find();
        $now = new \DateTime('now');
        $month = $now->format('m');
        $year = $now->format('Y');
        $query->andFilterWhere(['month_year'=>$year.'-' .$month]);
        $dataProvider = new ActiveDataProvider([
            'query'=>$query,
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/cash-flow/export-pdf', [
            'dataProvider' => $dataProvider,
            'model'=>$model,

        ]));
        $mpdf->Output('Cash_flow_Report.pdf', 'D');
    }
    public function actionExcelReport($data=null)
    {
        if ($data != null){
            $dataProvider = $data;
        }else{

            $query = CashFlowReport::find();
            $now = new \DateTime('now');
            $month = $now->format('m');
            $year = $now->format('Y');
            $query->andFilterWhere(['month_year'=>$year.'-' .$month]);
            $dataProvider = new ActiveDataProvider([
                'query'=> $query,
                'pagination'=>false,
            ]);

        }

        $phpExcel = new PHPExcel;
        $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
        $phpExcel ->getProperties()->setDescription("Excel SpreadSheet in PHP");
        $writer = PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
        $sheet = $phpExcel ->getActiveSheet();
        // $sheet->setTitle('My product list');
        $sheet ->getCell('A1')->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
        $sheet ->getCell('A4')->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
        $sheet ->getCell('A5')->setValue('BHUBANESWAR, Odisha');
        $sheet ->getCell('A7')->setValue('Cash Flow Report');
        $sheet ->getCell('A8')->setValue('');
        // Making headers text bold and larger
        $sheet->getStyle('A1:D1')->getFont()->setBold(true)->setSize(11);
        $sheet->getStyle('A2:D2')->getFont()->setSize(11);
        $sheet->getStyle('A3:D3')->getFont()->setSize(11);
        $sheet->getStyle('A7:D7')->getFont()->setSize(11);
        $sheet->getStyle('A5:D5')->getFont()->setSize(11);
        $sheet->getStyle('A8:D8')->getFont()->setSize(11);

        $sheet->getColumnDimension('A')->setWidth(10);
        $sheet->getColumnDimension('B')->setWidth(10);
        $sheet->getColumnDimension('C')->setWidth(10);
        $sheet->getColumnDimension('D')->setWidth(10);

        $sheet->mergeCells('A1:D1');
        $sheet->mergeCells('A2:D2');
        $sheet->mergeCells('A3:D3');
        $sheet->mergeCells('A4:D4');
        $sheet->mergeCells('A5:D5');
        $sheet->mergeCells('A6:D6');
        $sheet->mergeCells('A7:D7');
        $sheet->mergeCells('A8:D8');
        $sheet->mergeCells('B9:C9');


        $sheet->setCellValue('B9',  'Cash Movement');
        $sheet->setCellValue('A10',  'Particulars');
        $sheet->setCellValue('B10',  'Inflow');
        $sheet->setCellValue('C10',  'Outflow');
        $sheet->setCellValue('D10',  'Net Flow');

        $row=11;
        $total_outflow=0;$total_inflow=0;
        if (!empty($dataProvider->getModels())) {
            foreach ($dataProvider->getModels() as $key => $val) {
                $total_inflow+=$val->inflow;
                $total_outflow+=$val->outflow;
            }
        }

        foreach ($dataProvider->models as $model)
        {
            // $sheet->getStyle('A'.$row)->getAlignment()->setWrapText(true);

            $sheet->setCellValue('A' . $row,  date('F-Y',strtotime($model->month_year)));
            $sheet->setCellValue('B' . $row,  $model->inflow);
            $sheet->setCellValue('C' . $row,  $model->outflow);
            $sheet->setCellValue('D' . $row,  $model->net_flow);
            $row++;
        }
        $sheet->setCellValue('A' . $row,'Grand  Total');
        $sheet->setCellValue('B' . $row, $total_inflow);
        $sheet->setCellValue('C' . $row, $total_outflow);
        $sheet->setCellValue('D' . $row, $total_inflow-$total_outflow);


        ob_clean();
        ob_start();

        header('Content-type: application/vnd.ms-excel'); // It will be called file.xls
        header('Content-Disposition: attachment; filename="cash-flow.xlsx"'); // Write file to the browser

        $writer->save('php://output');
        ob_flush();
        exit();

    }
    public function actionDetailExcelReport($data1=null,$data2=null)
    {
        if ($data1 != null && $data2!=null){
            $dataProvider1 = $data1;
            $dataProvider2 = $data2;

        }

        $phpExcel = new PHPExcel;
        $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
        $phpExcel ->getProperties()->setDescription("Excel SpreadSheet in PHP");
        $writer = PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
        $sheet = $phpExcel ->getActiveSheet();
        // $sheet->setTitle('My product list');
        $sheet ->getCell('A1')->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
        $sheet ->getCell('A4')->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
        $sheet ->getCell('A5')->setValue('BHUBANESWAR, Odisha');
        $sheet ->getCell('A7')->setValue('Cash Flow Report');
        $sheet ->getCell('A8')->setValue('');
        // Making headers text bold and larger
        $sheet->getStyle('A8:C8')->getFont()->setBold(true)->setSize(11);
        $sheet->getStyle('A2:D2')->getFont()->setSize(11);
        $sheet->getStyle('A3:D3')->getFont()->setSize(11);
        $sheet->getStyle('A7:D7')->getFont()->setSize(11);
        $sheet->getStyle('A5:D5')->getFont()->setSize(11);
        $sheet->getStyle('A8:D8')->getFont()->setSize(11);

        $sheet->getColumnDimension('A')->setWidth(10);
        $sheet->getColumnDimension('B')->setWidth(10);
        $sheet->getColumnDimension('C')->setWidth(10);
        $sheet->getColumnDimension('D')->setWidth(10);

        $sheet->mergeCells('A8:B8');
        $sheet->mergeCells('C8:D8');
        $sheet->mergeCells('A1:D1');
        $sheet->mergeCells('A4:D4');
        $sheet->mergeCells('A5:D5');
        $sheet->mergeCells('A6:D6');
        $sheet->mergeCells('A7:D7'); 


        $sheet->setCellValue('A8',  'Inflow');
        $sheet->setCellValue('C8',  'Outflow');
        

        $row1=9;
        $total_inflow=0;$total_outflow=0;
        if (!empty($dataProvider1->getModels())) {
            foreach ($dataProvider1->getModels() as $key => $val) {
                $total_inflow+=$val->total_amount;
            }
        }
        if (!empty($dataProvider2->getModels())) {
            foreach ($dataProvider2->getModels() as $key => $val) {
                $total_outflow+=$val->total_amount;
            }
        }

        foreach ($dataProvider1->models as $model1)
        {
            // $sheet->getStyle('A'.$row)->getAlignment()->setWrapText(true);
            $customer=Customer::find()->where(['id'=>$model1->customer_name])->one();

            $sheet->setCellValue('A' . $row1,  $customer?$customer->business_name:$model1->customer_name);
            $sheet->setCellValue('B' . $row1,  $model1->total_amount);
            $row1++;
        }
        $row2=9;
        foreach ($dataProvider2->models as $model2)
        {
            // $sheet->getStyle('A'.$row)->getAlignment()->setWrapText(true);
            $sheet->setCellValue('C' . $row2,  Vendor::find()->where(['id'=>$model2->vendor_id])->one()->business_name);
            $sheet->setCellValue('D' . $row2,  $model2->total_amount);
            $row2++;
        }
        if($row1>$row2)
        {
            $row=$row1;
        }else{
            $row=$row2;
        }
        $sheet->getStyle('A'. (int)($row+1).':'.'D'. (int)($row+1))->getFont()->setBold(true)->setSize(11);
        $sheet->mergeCells('A'.(int)($row+1).':'.'C'.(int)($row+1)); 

        $sheet->setCellValue('A' . $row,'Inflow  Total');
        $sheet->setCellValue('B' . $row, $total_inflow);
        $sheet->setCellValue('C' . $row,'Outflow  Total');
        $sheet->setCellValue('D' . $row, $total_outflow);
        $sheet->setCellValue('A' . (int)($row+1), 'Netflow');
        $sheet->setCellValue('D' . (int)($row+1), $total_inflow-$total_outflow);


        ob_clean();
        ob_start();

        header('Content-type: application/vnd.ms-excel'); // It will be called file.xls
        header('Content-Disposition: attachment; filename="cash-flow.xlsx"'); // Write file to the browser

        $writer->save('php://output');
        ob_flush();
        exit();






    }

}
