<?php

namespace backend\modules\reports\controllers;

use backend\modules\reports\models\PurchaseVendorLedgerSearch;
use backend\modules\reports\models\VendorLedgerOverview;
use backend\modules\reports\models\VendorLedgerOverviewSearch;
use backend\modules\reports\models\VendorLedgerDetails;
use backend\modules\master\models\Vendor;
use backend\modules\users\models\PurchaseBill;
use backend\modules\users\models\PurchasePayment;
use backend\modules\users\models\CreditNote;
use backend\modules\users\models\DebitNote;
use backend\modules\users\models\AccountingDetails;
use yii2tech\spreadsheet\Spreadsheet;
use yii\data\ActiveDataProvider;
use backend\modules\users\models\AccountingJournal;
use yii\helpers\Url;
use Mpdf\Mpdf;
use Yii;


class PurchaseVendorLedgerController extends \yii\web\Controller
{
  public function actionIndex($print = null, $excel = null, $detailprint = null, $detailexcel = null)
  {
      $searchModel = new VendorLedgerOverviewSearch();
      $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

      if ($print !== null) {
          $dataProvider->pagination = false;
          $this->actionPdfexport($dataProvider, $searchModel, 'print');
      }
      if ($detailprint !== null) {
          $dataProvider->pagination = false;
          $this->actionPdfexport($dataProvider, $searchModel, 'detailprint');
      }
      if ($excel !== null) {
          $dataProvider->pagination = false;
          $this->actionExcelreport($dataProvider, $searchModel);
      }
      if ($detailexcel !== null) {
          $dataProvider->pagination = false;
          $this->actionExceldetailreport($dataProvider, $searchModel);
      }

      return $this->render('index', [
          'searchModel' => $searchModel,
          'dataProvider' => $dataProvider,
      ]);
  }

  public function actionPdfexport($dataProvider, $searchModel, $for)
  {
      $dataProvider->pagination = false;
      $mpdf = new Mpdf();
      $mpdf->showImageErrors = true;

      if ($for == 'print') {
        //print_r($dataProvider); die();
          $content = $this->renderPartial('/purchase-vendor-ledger/pdfexport', [
              'dataProvider' => $dataProvider,
              'model' => $searchModel,
          ]);
          
          $mpdf->WriteHTML($content);
          $mpdf->Output('Purchase-vendor-ledger.pdf', 'D');
      } else {
          $content = $this->renderPartial('/purchase-vendor-ledger/detailedpdf', [
              'dataProvider' => $dataProvider,
              'model' => $searchModel,
          ]);
          
          $mpdf->WriteHTML($content);
          $mpdf->Output('Detailed-Purchase-vendor-ledger.pdf', 'D');
      }

  }

  public function actionExcelreport($data = null, $mod = null)
  {
    if ($data != null) {
      $dataProvider = $data;
    } else {
      $dataProvider = new ActiveDataProvider([
        'query' => VendorLedgerOverview::find()->orderBy(['Date' => SORT_DESC]),
      ]);
    }

    $phpExcel = new \PHPExcel;
    $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
    $phpExcel->getProperties()->setDescription("Excel SpreadSheet in PHP");
    $writer = \PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
    $sheet = $phpExcel->getActiveSheet();

    // $sheet->setTitle('My product list');
    $sheet->getCell('A1')->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
    $sheet->getCell('A2')->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
    $sheet->getCell('A3')->setValue('BHUBANESWAR, Odisha');
    $sheet->getCell('A4')->setValue('');
    $sheet->getCell('A5')->setValue('Ledgers');
    $sheet->getStyle('A5')->getFont()->setBold(true)->setSize(11);


    $sheet->getStyle('A1')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A2')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A3')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A4')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A5')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    $sheet->getStyle('A6')->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);

    // Making headers text bold and larger
    $sheet->getStyle('A1:S1')->getFont()->setBold(true)->setSize(12);
    $sheet->getStyle('A2:S2')->getFont()->setSize(11);
    $sheet->getStyle('A3:S3')->getFont()->setSize(11);
    $sheet->getStyle('A4:S4')->getFont()->setSize(11);
    $sheet->getStyle('A5:S5')->getFont()->setSize(11);
    $sheet->getStyle('A6:S6')->getFont()->setSize(11);

    $sheet->getColumnDimension('A')->setWidth(60);
    $sheet->getColumnDimension('B')->setWidth(10);
    $sheet->getColumnDimension('C')->setWidth(10);

    $sheet->mergeCells('A1:C1');
    $sheet->mergeCells('A2:C2');
    $sheet->mergeCells('A3:C3');
    $sheet->mergeCells('A4:C4');
    $sheet->mergeCells('A5:C5');
    $sheet->mergeCells('A6:C6');

    if ($mod->from_date != null) {
      $sheet->setCellValue('A6', date('d/m/Y', strtotime($mod->from_date)) . ' to ' . date('d/m/Y', strtotime($mod->to_date)));
    } else {
      $sheet->setCellValue('A6', '');
    }


    $sheet->setCellValue('A7',  'Ledger Name');
    $sheet->setCellValue('B7',  'Debit');
    $sheet->setCellValue('C7',  'Credit');
    $sheet->getStyle('A7')->getFont()->setBold(true)->setSize(11);
    $sheet->getStyle('B7')->getFont()->setBold(true)->setSize(11);
    $sheet->getStyle('C7')->getFont()->setBold(true)->setSize(11);

    $row = 8;
    $total_debit_amount = 0;
    $total_credit_amount = 0;
    foreach ($dataProvider->models as $model) {

      if ($mod->from_date != '' && $mod->from_date != '') {
        $amount = 0;
        $vendor = Vendor::find()->where(['business_name' => $model->business_name])->one();
        $debit = VendorLedgerDetails::find()->andFilterWhere(['in', 'vendor', $vendor->id])->andFilterWhere(['between', 'dated', $mod['from_date'],$mod['to_date']])->orderBy(['dated' => SORT_ASC])->all();
        foreach ($debit as $key => $value) {

          if ($value->debit_amount != '') {
            $amount = (float)$amount + $value->debit_amount;
          } else {
            $amount = 0 + $value->debit_amount;
          }
        }

        $credit_amt = 0;
        $credit = VendorLedgerDetails::find()->andFilterWhere(['in', 'vendor', $vendor->id])->andFilterWhere(['between', 'dated', $mod['from_date'],$mod['to_date']])->orderBy(['dated' => SORT_ASC])->all();
        foreach ($credit as $key => $value) {

          if ($value->credit_amount != '') {
            $credit_amt = (float)$credit_amt + $value->credit_amount;
          } else {
            $credit_amt = 0 + $value->credit_amount;
          }
        }

        $tot_deb = $amount - $credit_amt;
        $tot_cre = $credit_amt - $amount;

        $sheet->setCellValue('B' . $row,  (string)$tot_deb);
        $sheet->setCellValue('C' . $row,  (string)$tot_cre);

        $total_credit_amount = $total_credit_amount + $tot_cre;
        $total_debit_amount = $total_debit_amount + $tot_deb;
      } else {

        $total_debit = $model->total_debit_amount - $model->total_credit_amount;
        $total_credit = $model->total_credit_amount - $model->total_debit_amount;

        if($total_debit<0)
        {
          $sheet->setCellValue('B' . $row,  (string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", '0.00'));
        }
        else{
          $sheet->getStyle('B' . $row)->getNumberFormat()->setFormatCode('#,##0.00');
          $sheet->setCellValue('B' . $row,(string)$total_debit);

          $total_debit_amount = $total_debit_amount + $total_debit;
        }

        if($total_credit<0)
        {
          $sheet->setCellValue('C' . $row,  (string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", '0.00'));
        }
        else{
          $sheet->getStyle('C' . $row)->getNumberFormat()->setFormatCode('#,##0.00');
          $sheet->setCellValue('C' . $row,(string)$total_credit);

          $total_credit_amount = $total_credit_amount + $total_credit;
        }
      }
      $sheet->setCellValue('A' . $row,  $model->business_name);

      $row++;
    }

    $sheet->setCellValue('A' . $row, 'Total');
    $sheet->setCellValue('B' . $row,  (string)$total_debit_amount);
    $sheet->setCellValue('C' . $row,  (string)$total_credit_amount);

    $sheet->getStyle('A' . $row)->getFont()->setBold(true)->setSize(11)->getColor()->setRGB('FF0000');
    $sheet->getStyle('B' . $row)->getFont()->setBold(true)->setSize(11)->getColor()->setRGB('FF0000');
    $sheet->getStyle('C' . $row)->getFont()->setBold(true)->setSize(11)->getColor()->setRGB('FF0000');

    ob_clean();
    ob_start();

    header('Content-type: application/vnd.ms-excel'); // It will be called file.xls  
    header('Content-Disposition: attachment; filename="purchase-vendor-ledger.xls"'); // Write file to the browser 

    $writer->save('php://output');
    ob_flush();
    exit();
  }
  public function actionExceldetailreport($data = null, $mod = null)
  {
    if ($data != null) {
      $dataProvider = $data;
    } else {
      $dataProvider = new ActiveDataProvider([
        'query' => PurchaseVendorLedgerSearch::find()->orderBy(['Date' => SORT_DESC]),
        'pagination'=>false
      ]);
    }

    $phpExcel = new \PHPExcel;
    $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
    $phpExcel->getProperties()->setDescription("Excel SpreadSheet in PHP");
    $writer = \PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
    $sheet = $phpExcel->getActiveSheet();
    $row = 1;
    $total_debit_amount = 0;
    $total_credit_amount = 0;
    foreach ($dataProvider->models as $model) {

      $viewmodel = PurchaseVendorLedgerSearch::find()->where(['ledgerhead_id' => $model->ledgerhead_id])->one();


      $sheet->getCell('A' . $row)->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
      $sheet->mergeCells("A" . ($row) . ":I" . ($row));
      $sheet->getCell('A' . ($row + 1))->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
      $sheet->mergeCells("A" . ($row + 1) . ":I" . ($row + 1));
      $sheet->getCell('A' . ($row + 2))->setValue('BHUBANESWAR, Odisha');
      $sheet->mergeCells("A" . ($row + 2) . ":I" . ($row + 2));

      $sheet->getStyle('A' . $row)->getFont()->setBold(true)->setSize(12);
      $sheet->getStyle('A' . $row)->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
      $sheet->getStyle('A' . ($row + 1))->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
      $sheet->getStyle('A' . ($row + 2))->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);

      $sheet->getCell('A' . ($row + 3))->setValue('');
      $sheet->mergeCells("A" . ($row + 3) . ":I" . ($row + 3));

      $model_address = Vendor::find()->where(['business_name' => $model->ledgerhead_id])->one();

      $sheet->getCell('A' . ($row + 4))->setValue($model->ledgerhead_id);
      $sheet->getStyle('A' . ($row + 4))->getFont()->setBold(true)->setSize(10);
      $sheet->mergeCells("A" . ($row + 4) . ":F" . ($row + 4));
      $sheet->getCell('A' . ($row + 5))->setValue($model_address->adderss_line_1 . ',' . $model_address->adderss_line_2 . ',' . $model_address->adderss_line_3);
      $sheet->mergeCells("A" . ($row + 5) . ":F" . ($row + 5));
      $sheet->getCell('A' . ($row + 6))->setValue($model_address->city);
      $sheet->mergeCells("A" . ($row + 6) . ":F" . ($row + 6));
      $sheet->getCell('A' . ($row + 7))->setValue($model_address->state);
      $sheet->mergeCells("A" . ($row + 7) . ":F" . ($row + 7));
      $sheet->getCell('A' . ($row + 8))->setValue($model_address->country . '-' . $model_address->pin_code);
      $sheet->mergeCells("A" . ($row + 8) . ":F" . ($row + 8));
      $sheet->getCell('A' . ($row + 9))->setValue('Phone No-' . (string)$model_address->phone_no);
      $sheet->mergeCells("A" . ($row + 9) . ":F" . ($row + 9));
      $sheet->mergeCells("G" . ($row + 4) . ":G" . ($row + 9));
      $sheet->getCell('H' . ($row + 4))->setValue('STATEMENT OF ACCOUNT');
      $sheet->getStyle('H' . ($row + 4))->getAlignment()->setVertical(\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_TOP);
      $sheet->getStyle('H' . ($row + 4))->getFont()->setBold(true)->setSize(10);
      $sheet->mergeCells("H" . ($row + 4) . ":H" . ($row + 9));

      if ($mod->from_date != null && $mod->to_date != '') {
        $sheet->getCell('A' . ($row + 10))->setValue(date('d/m/Y', strtotime($mod->from_date)) . ' to ' . date('d/m/Y', strtotime($mod->to_date)));
      } else {
        $sheet->getCell('A' . ($row + 10))->setValue('');
      }
      $sheet->mergeCells("A" . ($row + 10) . ":F" . ($row + 10));


      $sheet->getCell('A' . ($row + 11))->setValue('Date');
      $sheet->getColumnDimension('A')->setWidth(20);
      $sheet->getCell('B' . ($row + 11))->setValue('REF.NO.');
      $sheet->getColumnDimension('B')->setWidth(20);
      $sheet->getCell('C' . ($row + 11))->setValue('DESCRIPTION');
      $sheet->getColumnDimension('C')->setWidth(40);
      $sheet->getCell('D' . ($row + 11))->setValue('NOTES');
      $sheet->getColumnDimension('D')->setWidth(40);
      $sheet->getCell('E' . ($row + 11))->setValue('VCH TYPE');
      $sheet->getColumnDimension('E')->setWidth(10);
      $sheet->getCell('F' . ($row + 11))->setValue('DEBIT');
      $sheet->getColumnDimension('F')->setWidth(15);
      $sheet->getCell('G' . ($row + 11))->setValue('CREDIT');
      $sheet->getColumnDimension('G')->setWidth(15);
      $sheet->getCell('H' . ($row + 11))->setValue('BALANCE');
      $sheet->getColumnDimension('H')->setWidth(22);

      $balance = 0;

      if ($mod->from_date != '' && $mod->to_date != '') {

        $model_details = AccountingDetails::find()->where(['ledgerhead_id' => $model->ledgerhead_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->andFilterWhere(['>', 'created_at', $mod->from_date])->andFilterWhere(['<', 'updated_at', $mod->to_date])->orderBy(['date' => SORT_ASC])->all();
      } else {
        $model_details = AccountingDetails::find()->where(['ledgerhead_id' => $model->ledgerhead_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
      }


      foreach ($model_details as $key => $transaction) {

        $ledgerhead = AccountingDetails::find()->where(['transaction_id' => $transaction['transaction_id']])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();

        foreach ($ledgerhead as $key => $ledger) {

          if ($ledger->ledgerhead_id == $transaction->ledgerhead_id) {
            // $total_amount = $total_amount + $ledger->amount;
            if ($ledger->transaction_type == 'Payment' || $ledger->transaction_type == 'Credit Note') {
              $total_debit_amount = $total_credit_amount + $ledger->amount;
            } else if ($ledger->transaction_type == 'Bill' || $ledger->transaction_type == 'Debit Note') {
              $total_credit_amount = $total_credit_amount + $ledger->amount;
            } else {
              if ($ledger->accounting_type == 'Debit') {
                $total_debit_amount = $total_credit_amount + $ledger->amount;
              } else {
                $total_credit_amount = $total_credit_amount + $ledger->amount;
              }
            }
          }
        }
      }
      $i = $row + 13;
      $row_amount = 0;
      $k = 1800;

      $Debit_value = 0;
      $Credit_value = 0;

      if ($mod->from_date != '' && $mod->to_date != '') {
        $prev_debit_amount = 0;
        $previous_debit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
          ->andFilterWhere(['edit_status' => 0])
          ->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])
          ->andFilterWhere(['or', ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note']])->orderBy(['date' => SORT_ASC])->all();
        foreach ($previous_debit as $key => $prev_debit) {
          $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
        }

        $prev_credit_amount = 0;
        $previous_credit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
          ->andFilterWhere(['edit_status' => 0])
          ->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])
          ->andFilterWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
        foreach ($previous_credit as $key => $prev_credit) {
          $prev_credit_amount = $prev_credit_amount + $prev_credit->amount;
        }
        $before_balance = (($balance + $prev_credit_amount) - $prev_debit_amount);

        $sheet->getCell('H' . ($row + 12))->setValue((string)$before_balance);
      } else {
        $sheet->getCell('H' . ($row + 12))->setValue($balance);
      }


      foreach ($model_details as $key => $val) {

        if ($mod->from_date != '' && $mod->to_date != '') {
          $prev_debit_amount = 0;
          $previous_debit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
            ->andFilterWhere(['edit_status' => 0])
            ->andFilterWhere(['in', 'ledgerhead_id', $val->ledgerhead_id])
            ->andFilterWhere(['or', ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note']])->orderBy(['date' => SORT_ASC])->all();
          foreach ($previous_debit as $key => $prev_debit) {
            $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
          }
          $sheet->getCell('F' . ($row + 12))->setValue((string)$prev_debit_amount);
        } else {
          $sheet->getCell('F' . ($row + 12))->setValue(0);
        }

        if ($mod->from_date != '' && $mod->to_date != '') {
          $prev_debit_amount = 0;
          $previous_debit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
            ->andFilterWhere(['edit_status' => 0])
            ->andFilterWhere(['in', 'ledgerhead_id', $val->ledgerhead_id])
            ->andFilterWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
          foreach ($previous_debit as $key => $prev_debit) {
            $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
          }
          $sheet->getCell('G' . ($row + 12))->setValue((string)$prev_debit_amount);
        } else {
          $sheet->getCell('G' . ($row + 12))->setValue(0);
        }
        $count = 0;
        if ($val != '')
          $total_trancection = AccountingDetails::find()->where(['transaction_id' => $val->transaction_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC])->all();
        $row_amount = $row_amount + count($total_trancection);
        $count = count($total_trancection);


        if ($row_amount > 1)
          $sheet->mergeCells("A" . ($i) . ":A" . ($i + ($count - 2)));
        $sheet->mergeCells("B" . ($i) . ":B" . ($i + ($count - 2)));
        $sheet->mergeCells("D" . ($i) . ":D" . ($i + ($count - 2)));
        $sheet->mergeCells("E" . ($i) . ":E" . ($i + ($count - 2)));
        $sheet->mergeCells("F" . ($i) . ":F" . ($i + ($count - 2)));
        $sheet->mergeCells("G" . ($i) . ":G" . ($i + ($count - 2)));
        $sheet->mergeCells("H" . ($i) . ":H" . ($i + ($count - 2)));

        $final_debit_amount = 0;
        $final_credit_amount = 0;


        foreach ($total_trancection as $key => $value) {

          if ($value->accounting_type == 'Credit' && $value->ledgerhead_id == $val->ledgerhead_id) {
            $Credit_value = $Credit_value + $val->amount;
            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance + $val->amount;
              } else {
                $before_balance = 0 + $val->amount;
              }
            } else {

              if ($balance != '') {
                $balance = (float)$balance + $val->amount;
              } else {
                $balance = 0 + $val->amount;
              }
            }
          } else if ($value->accounting_type == 'Debit' && $value->ledgerhead_id == $val->ledgerhead_id) {

            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance - $val->amount;
              } else {
                $before_balance = 0 - $val->amount;
              }
            } else {
              if ($balance != '') {
                $balance = (float)$balance - $val->amount;
              } else {
                $balance = 0 - $val->amount;
              }
            }
            $Debit_value = $Debit_value + $val->amount;
          }
          if ($value->ledgerhead_id != $val->ledgerhead_id) {
            if ($value->transaction_type == 'Bill') {
              $findbill = PurchaseBill::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->vendor_id])->one();
                if ($vendor->business_name == $model->ledgerhead_id) {
                  $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                  $sheet->getCell('B' . ($i))->setValue($findbill->full_bill_name);
                  $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                  $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                  $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                }
              }
            } elseif ($value->transaction_type == 'Payment') {
              $findbill = PurchasePayment::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->vendor_id])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->prefix . ' ' . $findbill->payment_no . '/' . $findbill->suffix);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            }  elseif ($value->transaction_type == 'Debit Note') {
              $findbill = DebitNote::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->party_id])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->prefix . '/' . $findbill->po . '/' . $findbill->suffix);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } elseif ($value->transaction_type == 'Debit Note') {
              $findbill = DebitNote::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->party_id])->one();

                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->prefix . '/' . $findbill->po . '/' . $findbill->suffix);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } elseif ($value->transaction_type == 'Credit Note') {
              $findbill = CreditNote::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->customer_name])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->credit_note_id);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } else {
              if($value->transaction_type != 'Journal')
                 $sheet->getCell('B' . ($i))->setValue($value->transaction_id);
            }

            if ($value->transaction_type == 'Payment') {
              $findbill = PurchasePayment::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->vendor_id])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('F' . ($i))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
                    $final_debit_amount = $final_debit_amount + $val->amount;
                    $sheet->getStyle('G' . ($i))->getNumberFormat()->setFormatCode('0.00');
                    $sheet->getCell('G' . ($i))->setValue('0.00');
                  }
                }
              }
            }else {
              if($value->transaction_type != 'Journal'){
                $sheet->getStyle('F' . ($i))->getNumberFormat()->setFormatCode('0.00');
                $sheet->getCell('F' . ($i))->setValue('0.00');
                $sheet->getCell('G' . ($i))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
              }

              $final_credit_amount = $final_credit_amount + $val->amount;
            }
          } else if ($value->ledgerhead_id == $val->ledgerhead_id) {

            if ($value->transaction_type == 'Journal') {
              $str = ltrim($value->transaction_id, 'J');
              $findbill = AccountingJournal::find()->where(['jv_no' => $str])->one();
              // $findbill = PurchasePayment::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                  $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                  $sheet->getCell('B' . ($i))->setValue($value->transaction_id);
                  $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                  $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                  $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  if ($value->accounting_type == 'Credit') {
                    $sheet->getCell('G' . ($i))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
                    $sheet->getStyle('F' . ($i))->getNumberFormat()->setFormatCode('0.00');
                    $sheet->getCell('F' . ($i))->setValue("0.00");
                  } else {
                    $sheet->getCell('F' . ($i))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
                    $sheet->getStyle('G' . ($i))->getNumberFormat()->setFormatCode('0.00');
                    $sheet->getCell('G' . ($i))->setValue("0.00");
                  }
                // }
              }
            }
          }
          else 
          {
            if ($mod->from_date != '' && $mod->to_date != '') {
              $sheet->getStyle('H' . (($i)))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('H' . (($i)))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $before_balance));
            } else {
              $sheet->getStyle('H' . (($i)))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('H' . (($i)))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $balance));
            }

            $sheet->getStyle('H' . ($i))->getFont()->setBold(true)->setSize(10);
          }

          $i++;
        }
      }
      $sheet->getCell('E' . ($row + 13 + $row_amount))->setValue('Total (Rs)');
      $sheet->getStyle('E' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      $sheet->getStyle('F' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
      $sheet->getCell('F' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $viewmodel->total_debit_amount));
      $sheet->getStyle('F' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      $sheet->getStyle('G' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
      $sheet->getCell('G' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $viewmodel->total_credit_amount));
      $sheet->getStyle('G' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      if ($mod->from_date != '' && $mod->to_date != '') {
        $sheet->getStyle('H' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
        $sheet->getCell('H' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $viewmodel->total_debit_amount - $viewmodel->total_credit_amount));
      } else {
        $sheet->getStyle('H' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
        $sheet->getCell('H' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $viewmodel->total_debit_amount - $viewmodel->total_credit_amount));
      }

      $sheet->getStyle('H' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);


      $row = $row + 16 + $row_amount;
    }

    ob_clean();
    ob_start();

    header('Content-type: application/vnd.ms-excel'); // It will be called file.xls  
    header('Content-Disposition: attachment; filename="purchase-vendor-ledger-detailed.xls"'); // Write file to the browser 

    $writer->save('php://output');
    ob_flush();
    exit();
  }


  public function actionVendordetails($id = null, $totalexcel = null,$fromdate=null,$todate=null,$printpdf=null)
  {
    $model = new PurchaseVendorLedgerSearch();
    $vendor = Vendor::find()->where(['id' => $id])->one();
    $query = VendorLedgerDetails::find()->where(['vendor' => $vendor->id])->orderBy(['dated' => SORT_DESC]);
   
    if($fromdate != null && $todate != null)
    {
      $model->details_from_date  = $fromdate;
       $model->details_to_date = $todate;
      //  die();
       $query->andFilterWhere(['between', 'dated', $model->details_from_date, $model->details_to_date]);
      //  Yii::$app->request->get();
    }
    if ($model->load(Yii::$app->request->get())) {
      $query->andFilterWhere(['between', 'dated', $model->details_from_date, $model->details_to_date])
        ->andFilterWhere(['like', 'vch_type', $model->details_type])
        ->andFilterWhere(['between', 'credit_amount', $model->details_total_from, $model->details_total_to])
        ->orFilterWhere(['between', 'debit_amount', $model->details_total_from, $model->details_total_to]);
      // ->andFilterWhere(['between', 'Date', $model->from_date,$model->to_date]);    
    }
    $dataProvider = new ActiveDataProvider([
      'query' => $query,
      'pagination' => false
    ]);



    if ($totalexcel != null) {
      
      $query2 = AccountingDetails::find()->where(['ledgerhead_id' => $vendor->business_name])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note'], ['transaction_type' => 'Journal']])->orderBy(['date' => SORT_ASC]);
      $dataProvider2 = new ActiveDataProvider([
        'query' => $query2,
        'pagination' => false
      ]);
      $this->actionExcelvendordetailreport($dataProvider2, $model, $dataProvider2);
    }

    if ($printpdf != null) {
      $this->actionSinglepdf($dataProvider, $model,$vendor);
    }

    return $this->render('vendordetails', [
      'model' => $model,
      'vendor_id' => $id,
      'dataProvider' => $dataProvider,
      'user' => $vendor->business_name,
    ]);
  }

  public function actionExcelvendordetailreport($data = null, $mod = null, $provider = null)
  {
    if ($data != null) {
      $dataProvider = $data;
    } else {
      $dataProvider = new ActiveDataProvider([
        'query' => PurchaseVendorLedgerSearch::find()->orderBy(['Date' => SORT_DESC]),
      ]);
    }

    $phpExcel = new \PHPExcel;
    $phpExcel->getDefaultStyle()->getFont()->setName('Calibri');
    $phpExcel->getProperties()->setDescription("Excel SpreadSheet in PHP");
    $writer = \PHPExcel_IOFactory::createWriter($phpExcel, "Excel2007");
    $sheet = $phpExcel->getActiveSheet();
    $row = 1;
    foreach ($dataProvider->models as $model) {
      $sheet->getCell('A' . $row)->setValue('MOHAPATRA SCIENTIFIC PRIVATE LIMITED');
      $sheet->mergeCells("A" . ($row) . ":I" . ($row));
      $sheet->getCell('A' . ($row + 1))->setValue('F2G/30, INDRADHANU MARKET, IRC VILLAGE,');
      $sheet->mergeCells("A" . ($row + 1) . ":I" . ($row + 1));
      $sheet->getCell('A' . ($row + 2))->setValue('BHUBANESWAR, Odisha');
      $sheet->mergeCells("A" . ($row + 2) . ":I" . ($row + 2));

      $sheet->getStyle('A' . $row)->getFont()->setBold(true)->setSize(12);
      $sheet->getStyle('A' . $row)->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
      $sheet->getStyle('A' . ($row + 1))->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
      $sheet->getStyle('A' . ($row + 2))->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);

      $sheet->getCell('A' . ($row + 3))->setValue('');
      $sheet->mergeCells("A" . ($row + 3) . ":I" . ($row + 3));

      $model_address = Vendor::find()->where(['business_name' => $model->ledgerhead_id])->one();

      $sheet->getCell('A' . ($row + 4))->setValue($model->ledgerhead_id);
      $sheet->getStyle('A' . ($row + 4))->getFont()->setBold(true)->setSize(10);
      $sheet->mergeCells("A" . ($row + 4) . ":F" . ($row + 4));
      $sheet->getCell('A' . ($row + 5))->setValue($model_address->adderss_line_1 . ',' . $model_address->adderss_line_2 . ',' . $model_address->adderss_line_3);
      $sheet->mergeCells("A" . ($row + 5) . ":F" . ($row + 5));
      $sheet->getCell('A' . ($row + 6))->setValue($model_address->city);
      $sheet->mergeCells("A" . ($row + 6) . ":F" . ($row + 6));
      $sheet->getCell('A' . ($row + 7))->setValue($model_address->state);
      $sheet->mergeCells("A" . ($row + 7) . ":F" . ($row + 7));
      $sheet->getCell('A' . ($row + 8))->setValue($model_address->country . '-' . $model_address->pin_code);
      $sheet->mergeCells("A" . ($row + 8) . ":F" . ($row + 8));
      $sheet->getCell('A' . ($row + 9))->setValue('Phone No-' . (string)$model_address->phone_no);
      $sheet->mergeCells("A" . ($row + 9) . ":F" . ($row + 9));
      $sheet->mergeCells("G" . ($row + 4) . ":G" . ($row + 9));
      $sheet->getCell('H' . ($row + 4))->setValue('STATEMENT OF ACCOUNT');
      $sheet->getStyle('H' . ($row + 4))->getAlignment()->setVertical(\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_TOP);
      $sheet->getStyle('H' . ($row + 4))->getFont()->setBold(true)->setSize(10);
      $sheet->mergeCells("H" . ($row + 4) . ":H" . ($row + 9));

      if ($mod->from_date != null && $mod->to_date != '') {
        $sheet->getCell('A' . ($row + 10))->setValue(date('d/m/Y', strtotime($mod->from_date)) . ' to ' . date('d/m/Y', strtotime($mod->to_date)));
      } else {
        $sheet->getCell('A' . ($row + 10))->setValue('');
      }
      $sheet->mergeCells("A" . ($row + 10) . ":F" . ($row + 10));


      $sheet->getCell('A' . ($row + 11))->setValue('Date');
      $sheet->getColumnDimension('A')->setWidth(20);
      $sheet->getCell('B' . ($row + 11))->setValue('REF.NO.');
      $sheet->getColumnDimension('B')->setWidth(20);
      $sheet->getCell('C' . ($row + 11))->setValue('DESCRIPTION');
      $sheet->getColumnDimension('C')->setWidth(40);
      $sheet->getCell('D' . ($row + 11))->setValue('NOTES');
      $sheet->getColumnDimension('D')->setWidth(40);
      $sheet->getCell('E' . ($row + 11))->setValue('VCH TYPE');
      $sheet->getColumnDimension('E')->setWidth(10);
      $sheet->getCell('F' . ($row + 11))->setValue('DEBIT');
      $sheet->getColumnDimension('F')->setWidth(15);
      $sheet->getCell('G' . ($row + 11))->setValue('CREDIT');
      $sheet->getColumnDimension('G')->setWidth(15);
      $sheet->getCell('H' . ($row + 11))->setValue('BALANCE');
      $sheet->getColumnDimension('H')->setWidth(22);

      $balance = Vendor::find()->where(['business_name' => $model->ledgerhead_id])->one()->opening_as_on;


      $sheet->getCell('E' . ($row + 12))->setValue('Opening');


      // $model_details = AccountingDetails::find()->where(['ledgerhead_id'=>$model->ledgerhead_id])->all();
      $model_details = $provider;
      print_r($model_details);
      $i = $row + 13;
      $row_amount = 0;
      $k = 1800;

      $Debit_value = 0;
      $Credit_value = 0;

      if ($mod->from_date != '' && $mod->to_date != '') {
        $prev_debit_amount = 0;
        $previous_debit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
          ->andFilterWhere(['edit_status' => 0])
          ->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])
          ->andFilterWhere(['or', ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note']])->orderBy(['date' => SORT_ASC])->all();
        foreach ($previous_debit as $key => $prev_debit) {
          $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
        }

        $prev_credit_amount = 0;
        $previous_credit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
          ->andFilterWhere(['edit_status' => 0])
          ->andFilterWhere(['in', 'ledgerhead_id', $model->ledgerhead_id])
          ->andFilterWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC])->all();
        foreach ($previous_credit as $key => $prev_credit) {
          $prev_credit_amount = $prev_credit_amount + $prev_credit->amount;
        }
        $before_balance = (($balance + $prev_credit_amount) - $prev_debit_amount);

        $sheet->getCell('H' . ($row + 12))->setValue((string)$before_balance);
      } else {
        // print_r('x');
        $sheet->getCell('H' . ($row + 12))->setValue($balance);
      }


      foreach ($model_details as $key => $val) {


        if ($mod->from_date != '' && $mod->to_date != '') {
          $prev_debit_amount = 0;
          $previous_debit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
            ->andFilterWhere(['edit_status' => 0])
            ->andFilterWhere(['in', 'ledgerhead_id', $val->ledgerhead_id])
            ->andFilterWhere(['or', ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note']])->orderBy(['date' => SORT_ASC])->all();
          foreach ($previous_debit as $key => $prev_debit) {
            $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
          }
          $sheet->getCell('F' . ($row + 12))->setValue((string)$prev_debit_amount);
        } else {
          $sheet->getCell('F' . ($row + 12))->setValue(0);
        }

        if ($mod->from_date != '' && $mod->to_date != '') {
          $prev_debit_amount = 0;
          $previous_debit = AccountingDetails::find()->andFilterWhere(['<', 'created_at', $mod->from_date])
            ->andFilterWhere(['edit_status' => 0])
            ->andFilterWhere(['in', 'ledgerhead_id', $val->ledgerhead_id])
            ->andFilterWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC])->all();
          foreach ($previous_debit as $key => $prev_debit) {
            $prev_debit_amount = $prev_debit_amount + $prev_debit->amount;
          }
          $sheet->getCell('G' . ($row + 12))->setValue((string)$prev_debit_amount);
        } else {
          $sheet->getCell('G' . ($row + 12))->setValue(0);
        }
        $count = 0;
        if ($val != '')
          $total_trancection = AccountingDetails::find()->where(['transaction_id' => $val->transaction_id])->andWhere(['edit_status' => 0])->andWhere(['or', ['transaction_type' => 'Bill'], ['transaction_type' => 'Payment'], ['transaction_type' => 'Credit Note'], ['transaction_type' => 'Debit Note']])->orderBy(['date' => SORT_ASC])->all();
        $row_amount = $row_amount + count($total_trancection);
        $count = count($total_trancection);


        if ($row_amount > 1)
          $sheet->mergeCells("A" . ($i) . ":A" . ($i + ($count - 2)));
        $sheet->mergeCells("B" . ($i) . ":B" . ($i + ($count - 2)));
        $sheet->mergeCells("D" . ($i) . ":D" . ($i + ($count - 2)));
        $sheet->mergeCells("E" . ($i) . ":E" . ($i + ($count - 2)));
        $sheet->mergeCells("F" . ($i) . ":F" . ($i + ($count - 2)));
        $sheet->mergeCells("G" . ($i) . ":G" . ($i + ($count - 2)));
        $sheet->mergeCells("H" . ($i) . ":H" . ($i + ($count - 2)));

        foreach ($total_trancection as $key => $value) {

          if ($value->accounting_type == 'Credit' && $value->ledgerhead_id == $val->ledgerhead_id) {
            $Credit_value = $Credit_value + $val->amount;
            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance + $val->amount;
              } else {
                $before_balance = 0 + $val->amount;
              }
            } else {

              if ($balance != '') {
                $balance = (float)$balance + $val->amount;
              } else {
                $balance = 0 + $val->amount;
              }
            }
          } else if ($value->accounting_type == 'Debit' && $value->ledgerhead_id == $val->ledgerhead_id) {
            if ($mod->from_date != '' && $mod->to_date != '') {
              if ($before_balance != '') {
                $before_balance = (float)$before_balance - $val->amount;
              } else {
                $before_balance = 0 - $val->amount;
              }
            } else {
              if ($balance != '') {
                $balance = (float)$balance - $val->amount;
              } else {
                $balance = 0 - $val->amount;
              }
            }
            $Debit_value = $Debit_value + $val->amount;
          }
          if ($value->ledgerhead_id != $val->ledgerhead_id) {
            if ($value->transaction_type == 'Bill') {
              $findbill = PurchaseBill::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->vendor_id])->one();
                if ($vendor->business_name == $model->ledgerhead_id) {
                  $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                  $sheet->getCell('B' . ($i))->setValue($findbill->full_bill_name);
                  $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                  $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                  $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                }
              }
            } elseif ($value->transaction_type == 'Payment') {
              $findbill = PurchasePayment::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->vendor_id])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->prefix . ' ' . $findbill->payment_no . '/' . $findbill->suffix);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } elseif ($value->transaction_type == 'Debit Note') {
              $findbill = DebitNote::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->party_id])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->prefix . '/' . $findbill->po . '/' . $findbill->suffix);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } elseif ($value->transaction_type == 'Debit Note') {
              $findbill = DebitNote::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->party_id])->one();

                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->prefix . '/' . $findbill->po . '/' . $findbill->suffix);
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } elseif ($value->transaction_type == 'Credit Note') {
              $findbill = CreditNote::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->customer_name])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('A' . ($i))->setValue(date('d/m/Y', strtotime($value->created_at)));
                    $sheet->getCell('B' . ($i))->setValue($findbill->credit_note_id);
                    echo $findbill->credit_note_id;
                    $sheet->getCell('C' . ($i))->setValue($value->ledgerhead_id);
                    $sheet->getCell('D' . ($i))->setValue($val->ledgerhead_id);
                    $sheet->getCell('E' . ($i))->setValue($value->transaction_type);
                  }
                }
              }
            } else {
              $sheet->getCell('B' . ($i))->setValue($value->transaction_id);
            }

            if ($value->transaction_type == 'Payment') {
              $findbill = PurchasePayment::find()->where(['id' => $value->transaction_id])->one();
              if ($findbill) {
                $vendor = Vendor::find()->where(['id' => $findbill->vendor_id])->one();
                if ($vendor) {
                  if ($vendor->business_name == $model->ledgerhead_id) {
                    $sheet->getCell('F' . ($i))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
                    $sheet->getStyle('G' . ($i))->getNumberFormat()->setFormatCode('0.00');
                    $sheet->getCell('G' . ($i))->setValue('0.00');
                  }
                }
              }
            } else {
              $sheet->getStyle('F' . ($i))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('F' . ($i))->setValue('0.00');
              $sheet->getCell('G' . ($i))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $val->amount));
            }
          } else if ($value->ledgerhead_id == $val->ledgerhead_id) {
            if ($mod->from_date != '' && $mod->to_date != '') {
              $sheet->getStyle('H' . (($i)))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('H' . (($i)))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $before_balance));
            } else {
              $sheet->getStyle('H' . (($i)))->getNumberFormat()->setFormatCode('0.00');
              $sheet->getCell('H' . (($i)))->setValue((string)preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $balance));
            }

            $sheet->getStyle('H' . ($i))->getFont()->setBold(true)->setSize(10);
            // $total_balance = $total_balance + $balance;
          }
          $i++;
        }
      }
      $sheet->getCell('E' . ($row + 13 + $row_amount))->setValue('Total (Rs)');
      $sheet->getStyle('E' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      $sheet->getStyle('F' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
      $sheet->getCell('F' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $Debit_value));
      $sheet->getStyle('F' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      $sheet->getStyle('G' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
      $sheet->getCell('G' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $Credit_value));
      $sheet->getStyle('G' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);
      if ($mod->from_date != '' && $mod->to_date != '') {
        $sheet->getStyle('H' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
        $sheet->getCell('H' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $before_balance));
      } else {
        $sheet->getStyle('H' . ($row + 13 + $row_amount))->getNumberFormat()->setFormatCode('0.00');
        $sheet->getCell('H' . ($row + 13 + $row_amount))->setValue(preg_replace("/(\d+?)(?=(\d\d)+(\d)(?!\d))(\.\d+)?/i", "$1,", $balance));
      }

      $sheet->getStyle('H' . ($row + 13 + $row_amount))->getFont()->setBold(true)->setSize(10);


      $row = $row + 16 + $row_amount;
    }
    // die();

    ob_clean();
    ob_start();

    header('Content-type: application/vnd.ms-excel'); // It will be called file.xls  
    header('Content-Disposition: attachment; filename="purchase-vendor-ledger-detailed.xls"'); // Write file to the browser 

    $writer->save('php://output');
    ob_flush();
    exit();
  }

  public function actionSinglepdf($data=null,$mod = null,$vendor = null)
  {
    if ($data != null) {
      $dataProvider = $data;
    } else {
      $dataProvider = new ActiveDataProvider([
        'query' => PurchaseVendorLedgerSearch::find()->orderBy(['Date' => SORT_DESC]),
      ]);
    }
      $dataProvider->pagination = false;
      $mpdf = new mPDF;
      $mpdf->showImageErrors = true;
      $mpdf->WriteHTML($this->renderPartial('/purchase-vendor-ledger/singlepdf', [
        'dataProvider' => $dataProvider,
        'vendor' => $vendor
      ]));
      $mpdf->Output('Sale-Vendor-Ledger-details.pdf', 'D');
  }
}
