<?php

namespace backend\modules\reports\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use backend\modules\reports\models\SaleReportByProduct;
use backend\modules\users\models\SaleOrderProduct;
use yii\db\Query;
/**
 * AccountingLedgerSearch represents the model behind the search form of `backend\modules\master\models\AccountingLedger`.
 */
class SaleReportByProductSearch extends SaleReportByProduct
{
    /**
     * {@inheritdoc}
     */
    public $order_by;
    public $date_range;
    public $from_date;
    public $to_date;
    public $report_type;


    public function rules()
    {
        return [
            [['Date','Particulars', 'VChNo', 'GSTNo', 'product_name', 'Qty', 'Rate','GrossTotal','TaxAmount','VendorDEALER','Units','Description','Category','Executive','customer_type','date_range','report_type','order_by','created_by','from_date','to_date'], 'safe'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        // $query =SaleReportByProduct::find();


        $query = SaleOrderProduct::find()
      ->select([
        'CAST(sale_order.created_at AS DATE) AS Date',
        'customer.business_name AS Particulars',
        'sale_order.id AS sale_order_id',
        'sale_order_product.id AS sale_product_id',
        'sale_order.sale_order AS VChNo',
        'sale_order.receiver AS GSTNo',
        'product.product_name AS product_name',
        'product.bying_price AS purchase_price',
        'sale_order_product.quantity AS Qty',
        'sale_order_product.product_id AS product_id',
        'sale_order_product.unit_price AS Rate',
        'sale_order_product.discount AS ItemDiscount',
        'sale_order_product.price AS GrossTotal',
        'sale_order_product.tax AS TaxAmount',
        'sale_order_product.tax_included AS tax_included',
        'vendor.business_name AS VendorDEALER',
        'customer.customer_type AS customer_type',
        'sale_order_product.units AS Units',
        'sale_order_product.description AS Description',
        'category.name AS Category',
        'sale_order_product.discount_type AS discount_type',
        'sale_order_product.amount AS gross_amount',
        'lead_executive.name AS Executive',
        'sale_order.converted_to AS converted_to',
        // 'sale_order.created_by AS createdby',
        'user.username AS createdby',
    ])
    ->from('sale_order_product')
    ->leftJoin('sale_order', 'sale_order_product.sale_order_no = sale_order.id')
    ->leftJoin('user', 'sale_order.created_by = user.id')
    ->leftJoin('customer', 'customer.id = sale_order.customer_id')
    ->leftJoin('lead_executive', 'lead_executive.id = sale_order.executive_id')
    ->leftJoin('product', 'sale_order_product.product_id = product.id')
    ->leftJoin('vendor', 'vendor.id = product.vendor')
    ->leftJoin('category', 'category.id = product.category')
    ->where(['sale_order_product.is_deleted' => 0]);
 
       

        

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $query->andFilterWhere(['like', 'Executive', $this->Executive])
                  ->andFilterWhere(['like', 'Description', $this->Category])
                  ->andFilterWhere(['like', 'product_name', $this->product_name])
                  ->andFilterWhere(['like', 'customer_type', $this->customer_type])
                  ->andFilterWhere(['like', 'Particulars', $this->Particulars])
                  ->andFilterWhere(['like', 'VendorDEALER', $this->VendorDEALER]);

                if($this->date_range){
                    if($this->date_range=='today'){
                    $query->andFilterWhere(['like', 'Date', date('Y-m-d')]);
                    }elseif($this->date_range=='current_week') {
                        $query->andFilterWhere(['>=', 'Date',date('Y-m-d',strtotime('last monday')) ]);
                    }elseif($this->date_range=='current_month') {
                        $query->andFilterWhere(['>=', 'Date',date ('Y-m-d', strtotime ('first day of this month')) ]);
                    }
                    elseif($this->date_range=='current_year') {
                        $query->andFilterWhere(['>=', 'Date',date('Y-m-d',strtotime(date('Y-01-01')))]);
                    }elseif($this->date_range=='custom'){
    
                        $query->andFilterWhere(['between', 'Date', $this->from_date,$this->to_date]);
                    } 
                }              
                if($this->report_type){
                    if($this->report_type=='invoice'){
                        $query->andFilterWhere(['or',['converted_to' => 'Invoice'],['converted_to' => 'Receipt']]);
                    }elseif($this->report_type=='sale_order'){
                        $query->andFilterWhere(['converted_to' => 'SaleOrder']);
                    }
                }
    
                if($this->created_by){
                    $query->andFilterWhere(['created_by' => $this->created_by]);
                
                }

                $attributeMapping = [
                    'vasc' => ['CAST(VChNo as unsigned)' => SORT_ASC],
                    'vdsc' => ['CAST(VChNo as unsigned)' => SORT_DESC],
                    'dasc' => ['Date' => SORT_ASC],
                    'ddsc' => ['Date' => SORT_DESC],
                    'particulars_asc' => ['Particulars' => SORT_ASC],
                    'particulars_desc' => ['Particulars' => SORT_DESC],
                    'product_asc' => ['product_name' => SORT_ASC],
                    'product_desc' => ['product_name' => SORT_DESC],
                    'rate_asc' => ['Rate' => SORT_ASC],
                    'rate_desc' => ['Rate' => SORT_DESC],
                    'qty_asc' => ['Qty' => SORT_ASC],
                    'qty_desc' => ['Qty' => SORT_DESC],
                    'tax_asc' => ['TaxAmount' => SORT_ASC],
                    'tax_desc' => ['TaxAmount' => SORT_DESC],
                    // Add more mappings as needed
                ];
                
                if ($this->order_by && isset($attributeMapping[$this->order_by])) {
                    $query->orderBy($attributeMapping[$this->order_by]);
                } else {
                    $query->orderBy(['sale_order.id' => SORT_DESC]);
                } 

                $dataProvider = new ActiveDataProvider([
                    'query' => $query,
                    'sort' => [
                    'attributes' => ['Date','Particulars','sale_order_id','sale_product_id','VChNo','GSTNo','product_name','purchase_price','Qty','product_id','Rate','ItemDiscount','GrossTotal','TaxAmount',
                        'tax_included','VendorDEALER','customer_type','Units','Description','Category','discount_type','gross_amount','Executive','converted_to','createdby'
                    ],
                        'defaultOrder' => ['sale_product_id' => SORT_DESC], // Set your default sort order here
                    ],
                    'pagination' => [
                        'pageSize' => 20, 
                
                        
                    ],
                    ]);
            
        return $dataProvider;
    }


    
}