<?php

namespace backend\modules\users\controllers;

use Yii;
use backend\modules\users\models\DebitNote;
use backend\modules\users\models\DebitNoteDocument;
use backend\modules\users\models\DebitNoteProduct;
use backend\modules\master\models\Vendor;
use backend\modules\master\models\Customer;
use backend\modules\master\models\Product;
use backend\modules\users\models\AccountingDetails;
use backend\modules\users\models\DebitNoteSearch;
use backend\modules\master\models\AccountingTax;
use backend\models\AuthAssignment;
use yii\helpers\Json;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use backend\models\UserClient;
use Mpdf\Mpdf;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use yii\web\UploadedFile;
use backend\modules\master\models\PurchaseTemplates;
use backend\modules\master\models\GenericTemplate;
/**
 * DebitNoteController implements the CRUD actions for DebitNote model.
 */
class DebitNoteController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all DebitNote models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=102;
        $searchModel = new DebitNoteSearch();
        
        $model=new DebitNote();
        $product=Product::find()->all();
        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        }
        else
        {
            $accessed[] = Yii::$app->user->identity->id;
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams,$accessed);
        }
        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        if ($model->load(Yii::$app->request->get()) ) {

            if($user_type->item_name == 'Admin')
            {
               $query=DebitNote::find()->where(['is_deleted'=>0]);
            }
            else
            {
                $accessed = Yii::$app->user->identity->id;
                $vendors = UserClient::find()->where(['user_id'=>$accessed])->all();
                $vendor_ids= [];
                $i = 0;
                foreach($vendors as $data)
                {
                    $vendor_ids[$i] = $data->client_id;
                    $i++;
                }
                $query = DebitNote::find()->orderBy(['id' => SORT_DESC])->where(['in', 'party_id', $vendor_ids])->andWhere(['is_deleted' =>0]);
            }
            //$query=DebitNote::find()->where(['is_deleted'=>0]);
            $query->andFilterWhere(['between', 'date', $model->from_date,$model->to_date])
            ->andFilterWhere(['like', 'party_type', $model->partytype])
            ->andFilterWhere(['like', 'party_id', $model->party])
            ->andFilterWhere(['like', 'setting_type', $model->setting])
            ->andFilterWhere(['like', 'total', $model->amount]);  
            if($model->product){
                $productlists=DebitNoteProduct::find()->where(['product_id'=>$model->product,'is_deleted'=>0])->groupBy(['debit_note_id'])->all();
                $productid=ArrayHelper::map($productlists, 'bill_id','bill_id');
                $query->andFilterWhere(['in', 'id', $productid]) ; 
            }  
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
                'sort'=> ['defaultOrder' => ['id' => SORT_DESC]],
            ]);
        }

        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'model'=>$model,
            'product'=>$productlist,
            'searchModel'=>$searchModel,
        ]);
    }

    /**
     * Displays a single DebitNote model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=102;


        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = DebitNote::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->party_id])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['debit-note/index']);
            }
        }



        $data=DebitNoteProduct::find()->where(['debit_note_id'=>$id,'is_deleted'=>0])->all();
        $data1=DebitNoteDocument::find()->where(['debit_note_id'=>$id,'is_deleted'=>0])->all();
        return $this->render('view', [
            'model' => $this->findModel($id),
            'products'=>$data,
            'documents'=>$data1,
        ]);
    }

    /**
     * Creates a new DebitNote model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        Yii::$app->view->params['active']=102;
        $model = new DebitNote();
        $datalist=[];
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        //$count=DebitNote::find()->count();
        $selected_date=date('Y-m-d');
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if( strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=DebitNote::find()->where(['<=', 'date', $dynamic_date])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=DebitNote::find()->where(['>=', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }
        if ($model->load(Yii::$app->request->post())) {


            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$model->party_id])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['debit-note/index']);
                }
            }

            $model->sub_total=Yii::$app->request->post('subtotalamount');
            $model->total=Yii::$app->request->post('totalamount');
            $model->cgst=Yii::$app->request->post('cgst');
            $model->sgst=Yii::$app->request->post('sgst');
            $model->igst=Yii::$app->request->post('igst');
            $model->round_off=Yii::$app->request->post('roundoffval');
            $model->updated_by=Yii::$app->user->identity->id;
	    $model->full_debit_id = $model->prefix .'/'.$model->po.'/'.$model->suffix;

           // $model->po=DebitNote::find()->orderby(['id' => SORT_DESC])->one()->id+1;
            if($model->save(false)){
            $data=Yii::$app->request->post();
            for($i=1;$i<=$data['imagecount'];$i++){
                $document = new DebitNoteDocument();
                $document->document_name = UploadedFile::getInstanceByName('file'.$i);
                if($document->document_name){
                $nameimage=(new \DateTime())->format('u');
                $document->document_name->saveAs('usersdocument/debit_note/' .$nameimage.'.'.$document->document_name->extension);
                $document->document_name=$nameimage. '.' . $document->document_name->extension; 
                $document->debit_note_id=$model->id;
                $document->save();
                }
                } 
            for($i=1;$i<=$data['productcount'];$i++){
                $val=new DebitNoteProduct();
                //print_r($model->id)
                $val->debit_note_id=$model->id;
                $val->product_id=Yii::$app->request->post('product'.($i));
                $val->description=Yii::$app->request->post('desc'.($i));
                $val->qty=Yii::$app->request->post('quantity'.($i));
                // print_r(Yii::$app->request->post('unit'.($i)));
                // die();
                $val->unit=Yii::$app->request->post('unit'.($i));
                $val->price=Yii::$app->request->post('price'.($i));
                $val->discount=Yii::$app->request->post('discount'.($i));
                if($val->discount=='')
                {
                    $val->discount=0;
                }
                $val->discount_type=Yii::$app->request->post('discounttype'.($i));
                $val->tax=Yii::$app->request->post('tax'.($i));
                $val->incl_tax=Yii::$app->request->post('includetax'.($i));
                $val->amount=Yii::$app->request->post('total'.($i));
                $val->save(false);
                }


                 $ledger = new AccountingDetails();
                 $cgst = $model->cgst;
                  $igst = $model->igst;
                  $sgst = $model->sgst;
                  $roundoff = $model->round_off;
                  $totalamt = $model->total;
                  $amt = $model->sub_total;
                  $gsttype = $model->point_of_supply;
                  $tan_id = $model->prefix .'/'.$model->po.'/'.$model->suffix;
                  $tan_date = $model->date;
                  if($model->party_type=='vendor')
                  $paid_to = Vendor::find()->where(['id'=>$model->party_id])->one();
                  else
                   $paid_to = Customer::find()->where(['id'=>$model->party_id])->one(); 
                  $amtto = $paid_to->business_name;
                  $tcs = $model->tcs;
                  // $is_tcs = $model->is_tcs;
                  $ledger->leadgerentrydebit($cgst,$sgst,$igst,$roundoff,$totalamt,$gsttype,$tan_id,$amtto,$amt,$tcs,$tan_date);



                Yii::$app->getSession()->setFlash('success', 'Created Successfully');
                if(Yii::$app->request->post('createnew'))
                return $this->redirect(['create']);
                return $this->redirect(['index']);
        }
    }
        return $this->render('_form', [
            'model' => $model,
            'vendorlist'=>$vendorlist,
            'update'=>0,
            'datalist'=>$datalist,
            //'count'=>$count,
            'last_entry'=>$last_entry+1,
        ]);
    }

    /**
     * Updates an existing DebitNote model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=102;

        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

        $selected_date="2022-03-16";
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if( strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=DebitNote::find()->where(['<=', 'date', $dynamic_date])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=DebitNote::find()->where(['>=', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }

        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = DebitNote::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->party_id])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['debit-note/index']);
            }
        }

        $model = $this->findModel($id);
        if($model->is_deleted)
        throw new NotFoundHttpException('The requested page does not exist.');
        $count=DebitNote::find()->count();
        $data=DebitNoteProduct::find()->where(['debit_note_id'=>$id,'is_deleted'=>0])->all();
        if($model->party_type=='customer')
        $customer=Customer::find()->all();
        else
        $customer=Vendor::find()->all();
        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        $datalist=[];
        foreach($data as $index=>$value){
            $data[$index]['id']=Product::find()->where(['id'=>$value->product_id])->one()->product_name;
        }
        $product=Product::find()->all();
        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        $doc=DebitNoteDocument::find()->where(['debit_note_id'=>$id,'is_deleted'=>0])->all();
        if ($model->load(Yii::$app->request->post()) ) {
            $data=Yii::$app->request->post();
            $newids=[];
            for($i=1;$i<=$data['imagecount'];$i++){ 
                $newids[]=Yii::$app->request->post('file'.$i);
            }
            $oldids=ArrayHelper::map($doc, 'id', 'id');
            $diffids=array_diff($oldids,$newids);
            DebitNoteProduct::deleteAll(['debit_note_id'=>$id,'is_deleted'=>0]);
            // $model->po=((DebitNote::find()->count())+1);
            $model->sub_total=Yii::$app->request->post('subtotalamount');
            $model->total=Yii::$app->request->post('totalamount');
            $model->cgst=Yii::$app->request->post('cgst');
            $model->sgst=Yii::$app->request->post('sgst');
            $model->igst=Yii::$app->request->post('igst');
            $model->round_off=Yii::$app->request->post('roundoffval');
            // $model->tcs=Yii::$app->request->post('tcs');
            $model->updated_by=Yii::$app->user->identity->id;
            if($model->save()){
                DebitNoteDocument::deleteAll(['in','id',$diffids]);
                for($i=2;$i<=$data['imagecount']+1;$i++){ 
                if(!Yii::$app->request->post('file'.$i)){
                $document = new DebitNoteDocument();
                $document->document_name = UploadedFile::getInstanceByName('file'.$i);
                if($document->document_name){
                $nameimage=(new \DateTime())->format('u');
                $document->document_name->saveAs('usersdocument/debit_note/' .$nameimage.'.'.$document->document_name->extension);
                $document->document_name=$nameimage. '.' . $document->document_name->extension; 
                $document->debit_note_id=$model->id;
                $document->save(false);
                }
                }
                }
            for($i=1;$i<=$data['productcount'];$i++){
                $val=new DebitNoteProduct();

                $val->debit_note_id=$model->id;
                $val->product_id=Yii::$app->request->post('product'.($i));
                $val->description=Yii::$app->request->post('desc'.($i));
                $val->qty=Yii::$app->request->post('quantity'.($i));
                $val->unit=Yii::$app->request->post('unit'.($i));
                $val->price=Yii::$app->request->post('price'.($i));
                $val->discount=Yii::$app->request->post('discount'.($i));
                if($val->discount=='')
                {
                    $val->discount=0;
                }
                $val->discount_type=Yii::$app->request->post('discounttype'.($i));
                $val->tax=Yii::$app->request->post('tax'.($i));
                $val->incl_tax=Yii::$app->request->post('includetax'.($i));
                $val->amount=Yii::$app->request->post('total'.($i));

                $val->save();
                
                }
                Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
                if(Yii::$app->request->post('createnew'))
                return $this->redirect(['create']);
                return $this->redirect(['index']);
        }
    }

        return $this->render('_form', [
            'model' => $model,
            'datalist'=>$data,
            'update'=>1,
            'productlist'=>$productlist,
            'customerlist'=>$customerlist,
            'count'=>$count,
            'doc'=>$doc,
            'last_entry'=>$last_entry+1,
        ]);
    }
    public function actionGetDescription()
    {
        Yii::$app->view->params['active']=102;
        $product=Product::find()->where(['id'=>Yii::$app->request->post('data')])->one();
        return json_encode([
            'des'=>$product->category,
            'unit'=>$product->units,
            'price'=>$product->bying_price,
            'tax'=>$product->tax_rate,
            ]);
    }
    /**
     * Deletes an existing DebitNote model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=102;
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            if($ids){
            foreach($ids as $id){
            $document=DebitNoteProduct::find()->where(['debit_note_id'=>$id])->all();
            $document1=DebitNoteDocument::find()->where(['debit_note_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
            foreach($document1 as $value){$value->is_deleted=1;$value->save();}
            $data=$this->findModel($id);
            $data->is_deleted=1;$data->updated_by=Yii::$app->user->identity->id;$data->save();
            }
            }
             Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
             return $this->redirect(['index']);
        }
        else
        {
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_for_purchase = DebitNote::find()->where(['id'=>$id])->one();
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->party_id])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['debit-note/index']);
                }
            }
            $data=$this->findModel($id);
            $data->is_deleted=1;$data->updated_by=Yii::$app->user->identity->id;$data->save();
            $document=DebitNoteProduct::find()->where(['debit_note_id'=>$id])->all();
            $document1=DebitNoteDocument::find()->where(['debit_note_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
            foreach($document1 as $value){$value->is_deleted=1;$value->save();}
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);   
        }
        
        
    }
  public function actionExcelExport($detail=null)
    {
        Yii::$app->view->params['active']=102;

        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();

        if($user_type->item_name == 'Admin')
        {
           $query=DebitNote::find()->where(['is_deleted'=>0]);
        }
        else
        {
            $accessed = Yii::$app->user->identity->id;
            $vendors = UserClient::find()->where(['user_id'=>$accessed])->all();
            $vendor_ids= [];
            $i = 0;
            foreach($vendors as $data)
            {
                $vendor_ids[$i] = $data->client_id;
                $i++;
            }
            $query = DebitNote::find()->orderBy(['id' => SORT_DESC])->where(['in', 'party_id', $vendor_ids])->andWhere(['is_deleted' =>0]);
        }

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            [
                'label'=>'Debit Note #',
               'attribute' => 'po',
               'value' => function ($data) {
                return $data->po;
               }
            ],
            [
                'label'=>'Amount',
               'attribute' => 'total',
               'value' => function ($data) {
                return $data->total;
               }
            ],
            [
                'label'=>'Vendor Name / Financial Category',
               'attribute' => 'party_id',
               'value' => function ($data) {
                if($data->party_type=='customer')
                $val=Customer::find()->where(['id'=>$data->party_id])->one();
                else
                $val=Vendor::find()->where(['id'=>$data->party_id])->one();
                return $val->business_name;
               }
            ],
            'date:date',
            'setting_type',
            'sender_gst:ntext',
            'bill_type',
            'notes:ntext',
            'supply_type',
            [
                'label'=>'Supply Attracts Reverse Charge',
               'attribute' => 'supply_attracts',
               'value' => function ($data) {
                return $data->supply_attracts?"YES":"NO";
               }
            ],
            'pricelist',
            'point_of_supply',
            // 'ineligible_itc',
         
            [
                'label'=>'Is ineligible ITC?',
               'attribute' => 'ineligible_itc',
               'value' => function ($data) {
                return $data->ineligible_itc?"YES":"NO";
               }
            ],
            'prefix',
            'suffix',
            'bill_no',
            [
                'label'=>'Bill Date',
               'attribute' => 'bill_date',
               'value' => function ($data) {
                return $data->bill_date?date("d-m-Y", strtotime($data->bill_date)):'';
               }
            ],
            'product_code:ntext',
            'sub_total',
            'total',
            'match_bill',
            [
                'label'=>'Show To Store Keeper',
               'attribute' => 'show_to_store_keeper',
               'value' => function ($data) {
                return $data->show_to_store_keeper?"YES":"No";
               }
            ],
            [
                'label'=>'Order',
               'attribute' => 'full_order',
               'value' => function ($data) {
                return $data->full_order;
               }
            ],
            'debit_notes:ntext',
            'created_at:date',
            'updated_at:date',
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    [
                        'label'=>'Debit Note #',
                       'attribute' => 'po',
                       'value' => function ($data) {
                        return $data->po;
                       }
                    ],
                    [
                        'label'=>'Amount',
                       'attribute' => 'total',
                       'value' => function ($data) {
                        return $data->total;
                       }
                    ],
                    [
                        'label'=>'Vendor Name / Financial Category',
                       'attribute' => 'party_id',
                       'value' => function ($data) {
                        if($data->party_type=='customer')
                        $val=Customer::find()->where(['id'=>$data->party_id])->one();
                        else
                        $val=Vendor::find()->where(['id'=>$data->party_id])->one();
                        return $val->business_name;
                       }
                    ],
                    'date:date',
                ],
            ]);
        }
    if($detail)
    return $exporter->send('debit_note_details.xlsx');
    return $exporter->send('debit_note.xlsx');
    }
    public function actionDebitnotePdf($id=null,$from=null,$print=null)
    {
        Yii::$app->view->params['active']=102;

        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();

        if($user_type->item_name == 'Admin')
        {
           $query=DebitNote::find()->where(['is_deleted'=>0]);
        }
        else
        {
            $accessed = Yii::$app->user->identity->id;
            $vendors = UserClient::find()->where(['user_id'=>$accessed])->all();
            $vendor_ids= [];
            $i = 0;
            foreach($vendors as $data)
            {
                $vendor_ids[$i] = $data->client_id;
                $i++;
            }
            $query = DebitNote::find()->orderBy(['id' => SORT_DESC])->where(['in', 'party_id', $vendor_ids])->andWhere(['is_deleted' =>0]);
        }
        if($from=='all')
        {
            $dataProvider = new ActiveDataProvider([
            'query' => $query,
            ]);
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->WriteHTML($this->renderPartial('exportpdf', [
                'dataProvider' => $dataProvider,

            ]));
            $mpdf->Output('debit_note.pdf', 'I');
        }
        else
        {
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_for_purchase = DebitNote::find()->where(['id'=>$id])->one();
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->party_id])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['debit-note/index']);
                }
            }
             $model=$this->findModel($id);
             $products=DebitNoteProduct::find()->where(['debit_note_id'=>$id,'is_deleted'=>0])->all();
             $dataProvider = new ActiveDataProvider([
                'query' => DebitNote::find()->where(['id'=>$id]),
            ]);
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->WriteHTML($this->renderPartial('/debit-note/debitnote-pdf', [
                'dataProvider' => $dataProvider,
                'model'=>$model,  
                'products'=>$products,       
            ]));
            if($print=='print')
            {
                $mpdf->Output('debit_note.pdf', 'I');
            }
            else
            {
                $mpdf->Output('debit_note.pdf', 'D');
            }
            
        }
         
    }  
    public function actionGetCustomer()
    {
        $customer=Customer::find()->all();
        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    public function actionGetVendor()
    {
        $customer=Vendor::find()->all();
        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    public function actionGetVendorGst()
    {
        $vendor=Vendor::find()->where(['id'=>Yii::$app->request->post('data')])->one();
        return $vendor->gstin_no;
    }
    public function actionGetCustomerGst()
    {
        $customer=Customer::find()->where(['id'=>Yii::$app->request->post('data')])->one();
        return $customer->gstin_no;
    }
    Public function actionDownloadAttachment($id)
    {
     Yii::$app->view->params['active']=102;
    $data=DebitNoteDocument::find()->where(['debit_note_id'=>$id,'is_deleted'=>0])->all();
    if(!$data){
    Yii::$app->getSession()->setFlash('success', 'No Attachment Found');
    return $this->redirect(['index']);
    }
    $file = Yii::$app->basePath.'/web/usersdocument/debit_note/product_'.$id.'.zip';
    $zip = new \ZipArchive();
    $zip->open($file, \ZipArchive::CREATE | \ZipArchive::OVERWRITE) ;
    foreach($data as $value){
       $files[]=Yii::$app->basePath . '/web/usersdocument/debit_note/'.$value->document_name;
    }
    foreach($files as $val){
        $zip->addFromString(basename($val), file_get_contents($val));
    }
    $zip->close();
    return Yii::$app->response->sendFile($file);
    }

     public function actionCustomerdetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $customer=Customer::find()->limit(10)->all();
        else
        $customer=Customer::find()->where(["like","business_name",$searchTerm])->limit(10)->all();

        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    public function actionVendordetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $customer=Vendor::find()->limit(10)->all();
        else
        $customer=Vendor::find()->where(["like","business_name",$searchTerm])->limit(10)->all();

        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    public function actionProductdetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $product=Product::find()->limit(10)->all();
        else
        $product=Product::find()->where(["like","product_name",$searchTerm])->limit(10)->all();

        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        return json_encode($productlist);
    }
    /**
     * Finds the DebitNote model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return DebitNote the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = DebitNote::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
     public function actionFinddetails()
    { 
        $taxdetails= Accountingtax::find()->select(['tax_name','accounting_tax_id'])->where(['is_deleted'=>0])
              ->andWhere(['tax_sub_type'=>'Output'])
              ->andWhere(['status'=>1])
              ->andWhere(['or',['tax_class'=>'IGST'],['tax_class'=>'TCS']])->all();      
        $taxlist=ArrayHelper::map($taxdetails, 'accounting_tax_id','tax_name');
        return Json::encode($taxlist);
    }
    public function actionTaxdetails($item)
    {
       $taxdetails= AccountingTax::find()->where(['accounting_tax_id' =>$item])->one();
        echo Json::encode($taxdetails);
    }
    public function actionFielddebitnotesettingtype($setting=null,$template=null)
    {   
        //print_r(1);die();
        $find = GenericTemplate::find()->where(['template_for'=>'Debit Note'])->andWhere(['gt_id'=>$setting])->count();
        echo Json::encode($find);
    }
    public function actionSelectdate($selected_date=null)
    {   
        $selected_date=$_GET['selected_date']; 
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
        $count_before_year=DebitNote::find()->where(['<=', 'date', $dynamic_date])->count();
        $count_after_year=DebitNote::find()->where(['>=', 'date', $dynamic_date])->count();
        if(strtotime($selected_date) <= strtotime($dynamic_date)) { 
            $last_entry=$count_before_year+1;
        }
        else if(strtotime($selected_date) > strtotime($dynamic_date)){
            $last_entry=$count_after_year+1;
        } 
        else{
            $last_entry=0;
        }
        echo $last_entry;
    }


}
