<?php

namespace backend\modules\users\controllers;

use Yii;
use backend\modules\users\models\ProductReceived;
use backend\modules\users\models\ReceivedProductList;
use backend\modules\master\models\Product;
use backend\modules\master\models\Vendor;
use yii\helpers\ArrayHelper;
use backend\modules\users\models\ProductReceivedSearch;
use backend\modules\master\models\PurchaseOrder;
use backend\modules\master\models\PurchaseOrderProduct;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use Mpdf\Mpdf;
use backend\modules\users\models\ProductInventory;
use backend\modules\master\models\PurchaseAgainstGr;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use backend\models\UserClient;
use backend\models\AuthAssignment;
/**
 * ProductReceivedController implements the CRUD actions for ProductReceived model.
 */
class ProductReceivedController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all ProductReceived models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active']=100;
        $searchModel = new ProductReceivedSearch();
        $model=new ProductReceived();

        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        }
        else
        {
            $accessed[] = Yii::$app->user->identity->id;
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams,$accessed);
        }


        // $query=ProductReceived::find()->where(['is_deleted'=>0])->orderBy(['id' => SORT_DESC]);
        
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        if ($model->load(Yii::$app->request->get()) ) {

            if($user_type->item_name == 'Admin')
            {
                $query=ProductReceived::find()->where(['is_deleted'=>0])->orderBy(['id'=>SORT_DESC]);
            }
            else
            {
                $accessed = Yii::$app->user->identity->id;
                $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
                $customer_ids = [];
                $j = 0;
                foreach($customers as $data)
                {
                    $customer_ids[$j] = $data->client_id;
                    $j++;
                }
                $query = ProductReceived::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
            }
            
            $query->andFilterWhere(['between', 'date', $model->from_date,$model->to_date]);
            $query->andFilterWhere(['like', 'vendor_id', $model->vendor])
            ->andFilterWhere(['like', 'shippment_no', $model->shippment]);  
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
            ]);  
        }
        
        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'model' => $model,
            'vendorlist'=>$vendorlist,
        ]);
    }

    /**
     * Displays a single ProductReceived model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        Yii::$app->view->params['active']=100;
        $data=ReceivedProductList::find()->where(['received_id'=>$id,'is_deleted'=>0])->all();
        return $this->render('view', [
            'model' => $this->findModel($id),
            'products'=>$data,
        ]);
    }

    /**
     * Creates a new ProductReceived model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate($id=null,$from=null)
    {
        Yii::$app->view->params['active']=100;
        $model = new ProductReceived();
        $vendor=Vendor::find()->all();
        $datalist= new ReceivedProductList();

        if($id)
        {
          if($from=='inventory')
          {
                $purchasedata= PurchaseOrder::find()->where(['id' =>$id])->one(); 
                $model->vendor_id = $purchasedata->vendor_name;
                $datalist = PurchaseOrderProduct::find()->select(['product_id','description','quantity As qty'])->where(['purchase_order_id'=>$id])->all();


                foreach ($datalist as $key => $value) {

                    $findreceived = ProductReceived::find()->where(['purchase_order_id'=>$purchasedata->id])->all();
                    $pdt_count =0;
                    foreach ($findreceived as $key => $val) {
                        $revdproduct = ReceivedProductList::find()->where(['received_id'=>$val->id])->andWhere(['product_id'=>$value->product_id])->one();
                        if($revdproduct)
                        {
                            $pdt_count = $pdt_count + $revdproduct->qty;
                        }
                        else
                        {
                            $pdt_count = $pdt_count;
                        }

                        # code...
                    }
                   // print_r($value->qty);
                    //print_r($pdt_count); die();
                    

                    $value->qty = $value->qty -  $pdt_count;

                    # code...
                }
          }
        }
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        $count=ProductReceived::find()->count();


        $selected_date=date('Y-m-d');
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if(strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=ProductReceived::find()->where(['<=', 'date', $dynamic_date])->count();
              
               
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=ProductReceived::find()->where(['>=', 'date', $dynamic_date])->count();
                
            }
            else{
                $last_entry=0;
            }
        if ($model->load(Yii::$app->request->post()) ) {
            
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$model->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['product-received/index']);
                }
            }
            // $model->partially=Yii::$app->request->post('partially');
            $model->shippment_no=$model->shippment_no;
            $count = 0;
            $product_count = 0;
            if($from=='inventory')
                {
                        $formpurchaseorder = PurchaseOrder::find()->where(['id'=>$purchasedata->id])->one();

                        $totalpurchaseproductqty = PurchaseOrderProduct::find()->where(['purchase_order_id'=>$formpurchaseorder->id])->all();
                        $findreceivedlist = ProductReceived::find()->where(['purchase_order_id'=>$formpurchaseorder->id])->count();
                        if($findreceivedlist)
                        {
                            $receivedlist = ProductReceived::find()->where(['purchase_order_id'=>$formpurchaseorder->id])->all();
                            $rec_count = 0;
                            foreach ($receivedlist as $key => $val) {
                                $receivedlistproduct = ReceivedProductList::find()->where(['received_id'=>$val->id])->all();
                                foreach ($receivedlistproduct as $key => $rvl) {
                                    $rec_count = $rec_count + $rvl->qty;
                                }
                            }

                            foreach ($totalpurchaseproductqty as $key => $value) {
                                $count = $count + $value->quantity;
                            }

                            for($i=1;$i<=(count(Yii::$app->request->post())-1)/3;$i++)
                            {       
                              $product_count = $product_count + Yii::$app->request->post('quantity'.($i));
                            }

                            $total_pro = $product_count + $rec_count;

                            if($total_pro<$count)
                            {
                                 $formpurchaseorder->gr_status = 3;
                                 $formpurchaseorder->status = "Partially Received";
                                 $formpurchaseorder->save(false);
                            }
                            else
                            {
                                 $formpurchaseorder->gr_status = 1;
                                 $formpurchaseorder->status = "Fully Received";
                                 $formpurchaseorder->save(false);
                            }

                        }
                        else
                        {
                            foreach ($totalpurchaseproductqty as $key => $value) {
                                $count = $count + $value->quantity;
                            }

                            for($i=1;$i<=(count(Yii::$app->request->post())-1)/3;$i++)
                            {       
                              $product_count = $product_count + Yii::$app->request->post('quantity'.($i));
                            }

                            if($product_count<$count)
                            {
                                 $formpurchaseorder->gr_status = 3;
                                 $formpurchaseorder->status = "Partially Received";
                                 $formpurchaseorder->save(false);
                            }
                            else
                            {
                                 $formpurchaseorder->gr_status = 1;
                                 $formpurchaseorder->status = "Fully Received";
                                 $formpurchaseorder->save(false);
                            }

                        }

                        $model->purchase_order_id = $formpurchaseorder->id;
                  
                }
                if($product_count<$count)
                {
                     $model->partially = 1;
                }
                else
                {
                     $model->partially = 0;
                }

            if($model->save(false)){
            for($i=1;$i<=(count(Yii::$app->request->post())-1)/3;$i++){
                $val=new ReceivedProductList();
                $val->received_id=$model->id;
                $val->product_id=Yii::$app->request->post('product'.($i));
                $val->description=Yii::$app->request->post('desc'.($i));
                $val->qty=Yii::$app->request->post('quantity'.($i));
                $val->save();



                //inventory
                if($model->purchase_order_id!='')
                {
                    $prv_inventory = ProductInventory::find()->where(['product_id'=>$val->product_id])->andWhere(['status'=>1])->one();
                    $product = PurchaseOrderProduct::find()->where(['product_id'=>$val->product_id])->andWhere(['purchase_order_id'=>$model->purchase_order_id])->one();
                    $unit_price = ($product->price/$product->quantity);
                    $new_inventory = New ProductInventory();
                    $new_inventory->product_id = $prv_inventory->product_id;
                    $new_inventory->quantity = $val->qty + $prv_inventory->quantity;
                    if($new_inventory->quantity != 0)
                        $new_inventory->price = (($prv_inventory->price*$prv_inventory->quantity) + ($val->qty*$unit_price))/($val->qty+$prv_inventory->quantity);
                    else
                    $new_inventory->price = (($prv_inventory->price*$prv_inventory->quantity) + ($val->qty*$unit_price));  
                    $new_inventory->status = 1;

                    $set_date = $model->date;
                    $date = new \DateTime($set_date);
                    $old_date = date('l, F d y H:i:s');
                    $time = date("H:i:s",strtotime($old_date));
                    $merge = new \DateTime($date->format('Y-m-d') .' ' .$time);
                    $final_date = $merge->format('Y-m-d H:i:s');

                    $new_inventory->date = $final_date;
                    $new_inventory->save();
                    $prv_inventory->status = 0;
                    $prv_inventory->save();
                }
                else
                {
                    $prv_inventory = ProductInventory::find()->where(['product_id'=>$val->product_id])->andWhere(['status'=>1])->one();
                    $product = Product::find()->where(['id'=>$val->product_id])->one();
                    $new_inventory = New ProductInventory();
                    $new_inventory->product_id = $prv_inventory->product_id;
                    $new_inventory->quantity = $val->qty + $prv_inventory->quantity;
                    if($new_inventory->quantity !=0)
                        $new_inventory->price = (($prv_inventory->price*$prv_inventory->quantity) + ($val->qty*$product->selling_price))/($val->qty+$prv_inventory->quantity);
                    else
                        $new_inventory->price = (($prv_inventory->price*$prv_inventory->quantity) + ($val->qty*$product->selling_price));
                    $new_inventory->status = 1;

                    $set_date = $model->date;
                    $date = new \DateTime($set_date);
                    $old_date = date('l, F d y H:i:s');
                    $time = date("H:i:s",strtotime($old_date));
                    $merge = new \DateTime($date->format('Y-m-d') .' ' .$time);
                    $final_date = $merge->format('Y-m-d H:i:s');

                    $new_inventory->date = $final_date;
                    $new_inventory->save();
                    $prv_inventory->status = 0;
                    $prv_inventory->save();
                }
            


                $avilableproduct = Product::find()->where(['id'=>$val->product_id])->one();
               // $avilableproduct->stock_unit = $avilableproduct->stock_unit + $val->qty;
                //$avilableproduct->save(false);
                }

                if($model->purchase_order_id!='')
                {
                    $newpurchase_against_gr = new PurchaseAgainstGr();
                    $newpurchase_against_gr->purchase_id = $formpurchaseorder->id;
                    $newpurchase_against_gr->gr_id = $model->id;
                    $newpurchase_against_gr->vendor_id = $model->vendor_id;
                    $newpurchase_against_gr->total_amount = $formpurchaseorder->total_amount;
                    // $newsaleorder_against_dc->delivered_amount = $model-> 
                    $newpurchase_against_gr->purchase_date = $formpurchaseorder->date;
                      // $newpurchase_against_gr->source = $formpurchaseorder->saleorder_source;
                    $newpurchase_against_gr->gr_status = $formpurchaseorder->gr_status;
                    $newpurchase_against_gr->convert_to = 'Bill';
                    $newpurchase_against_gr->save(false);
                }

                Yii::$app->getSession()->setFlash('success', 'Created Successfully');
                 return $this->redirect(['index']);
            }
        }

        return $this->render('_form', [
            'model' => $model,
            'update'=>0,
            'datalist'=>$datalist,
            'productlist'=>[],
            'vendorlist'=>$vendorlist,
            'count'=>$count+1,
            'from'=>$from,
            'last_entry'=>$last_entry+1,
        ]);
    }

    /**
     * Updates an existing ProductReceived model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=100;
        $model = $this->findModel($id);

        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = ProductReceived::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['product-received/index']);
            }
        }



        if($model->is_deleted)
        throw new NotFoundHttpException('The requested page does not exist.');
        $count=ProductReceived::find()->count();
        $data=ReceivedProductList::find()->where(['received_id'=>$id,'is_deleted'=>0])->all();
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        $datalist=[];
        foreach($data as $index=>$value){
            $data[$index]['id']=Product::find()->where(['id'=>$value->product_id])->one()->product_name;
        }
        $product=Product::find()->all();
        $productlist=ArrayHelper::map($product, 'id', 'product_name');



         $selected_date=date('Y-m-d');
        $year=date("Y"); 
       
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if(strtotime($selected_date) <= strtotime($dynamic_date))
            {
                // print_r('eee');
                $last_entry=ProductReceived::find()->where(['<=', 'date', $dynamic_date])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                // print_r('fff');
                $last_entry=ProductReceived::find()->where(['>=', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }
            // print_r($last_entry);
            // die();

        if ($model->load(Yii::$app->request->post()) ) {
            ReceivedProductList::deleteAll(['received_id'=>$id,'is_deleted'=>0]);
            $model->partially=Yii::$app->request->post('partially');
            // $model->shippment_no=((ProductReceived::find()->count())+1);


            $count = 0;
            $product_count = 0;
           
            $formpurchaseorder = PurchaseOrder::find()->where(['id'=>$model->purchase_order_id])->one();

            if($formpurchaseorder!='')
            {
                $totalpurchaseproductqty = PurchaseOrderProduct::find()->where(['purchase_order_id'=>$formpurchaseorder->id])->all();
                $findreceivedlist = ProductReceived::find()->where(['purchase_order_id'=>$formpurchaseorder->id])->count();
            }

            if($formpurchaseorder!='')
            {
                if($findreceivedlist)
                {
                    $receivedlist = ProductReceived::find()->where(['purchase_order_id'=>$formpurchaseorder->id])->all();
                    $rec_count = 0;
                    foreach ($receivedlist as $key => $val) {
                        $receivedlistproduct = ReceivedProductList::find()->where(['received_id'=>$val->id])->all();
                        foreach ($receivedlistproduct as $key => $rvl) {
                            $rec_count = $rec_count + $rvl->qty;
                        }
                    }

                    foreach ($totalpurchaseproductqty as $key => $value) {
                        $count = $count + $value->quantity;
                    }

                    for($i=1;$i<=(count(Yii::$app->request->post())-1)/3;$i++)
                    {       
                      $product_count = $product_count + Yii::$app->request->post('quantity'.($i));
                    }

                    $total_pro = $product_count + $rec_count;

                    if($total_pro<$count)
                    {
                         $formpurchaseorder->gr_status = 3;
                         $formpurchaseorder->status = "Partially Received";
                         $formpurchaseorder->save(false);
                    }
                    else
                    {
                         $formpurchaseorder->gr_status = 1;
                         $formpurchaseorder->status = "Fully Received";
                         $formpurchaseorder->save(false);
                    }

                }
                else
                {
                    foreach ($totalpurchaseproductqty as $key => $value) {
                        $count = $count + $value->quantity;
                    }

                    for($i=1;$i<=(count(Yii::$app->request->post())-1)/3;$i++)
                    {       
                      $product_count = $product_count + Yii::$app->request->post('quantity'.($i));
                    }

                    if($product_count<$count)
                    {
                         $formpurchaseorder->gr_status = 3;
                         $formpurchaseorder->status = "Partially Received";
                         $formpurchaseorder->save(false);
                    }
                    else
                    {
                         $formpurchaseorder->gr_status = 1;
                         $formpurchaseorder->status = "Fully Received";
                         $formpurchaseorder->save(false);
                    }

                }
                $model->purchase_order_id = $formpurchaseorder->id;
            }
            if($product_count<$count)
            {
                 $model->partially = 1;
            }
            else
            {
                 $model->partially = 0;
            }
            if($model->save(false)){
               
            for($i=1;$i<=((count(Yii::$app->request->post())-1)/3);$i++){

                $val=new ReceivedProductList();
                $val->received_id=$model->id;
                $val->product_id=Yii::$app->request->post('product'.($i));
                $val->description=Yii::$app->request->post('desc'.($i));
                $val->qty=Yii::$app->request->post('quantity'.($i));
                $val->save();   
                }
                Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
            return $this->redirect(['index']);
        }
    }
        return $this->render('_form', [
            'model' => $model,
            'datalist'=>$data,
            'update'=>1,
            'productlist'=>$productlist,
            'vendorlist'=>$vendorlist,
            'count'=>$count,
            'last_entry'=>$last_entry,
        ]);
    }

    /**
     * Deletes an existing ProductReceived model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=100;
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            if($ids){
            foreach($ids as $id){  
            $against = PurchaseAgainstGr::find()->where(['gr_id'=>$id])->one();
            if($against!='')
            {
               $against->is_deleted = 1;
               $against->save();
            }
            $document=ReceivedProductList::find()->where(['received_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
            $data=$this->findModel($id);
            $data->is_deleted=1;$data->save();
            }
            }
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        else
        {
            $data=$this->findModel($id);
            $against = PurchaseAgainstGr::find()->where(['gr_id'=>$data->id])->one();
            if($against!='')
            {
                $against->is_deleted = 1;
                $against->save();
            }
            $data->is_deleted=1;$data->save();
            $document=ReceivedProductList::find()->where(['received_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        
    }
    public function actionExcelExport($detail=null)
    {
        Yii::$app->view->params['active']=100;

        $check_id = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $query=ProductReceived::find()->where(['is_deleted'=>0])->orderBy(['id'=>SORT_DESC]);
        }
        else
        {
            $accessed = Yii::$app->user->identity->id;
            $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
            $customer_ids = [];
            $j = 0;
            foreach($customers as $data)
            {
                $customer_ids[$j] = $data->client_id;
                $j++;
            }
            $query = ProductReceived::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
        }


        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            [
                'label'=>'Received #',
               'attribute' => 'shippment_no',
               'value' => function ($data) {
                return 'GRN'.$data->shippment_no;
               }
            ],
            [
                'label'=>'Vendor/Source',
               'attribute' => 'vendor_id',
               'value' => function ($data) {
                $value=Vendor::find()->where(['id'=>$data->vendor_id])->one();
                return $value?$value->business_name:'';
               }
            ],
            'date:date',
            [
                'label'=>'Status',
               'attribute' => 'partially',
               'value' => function ($data) {
                return $data->partially?'Partially Received':'Fully Received';
               }
            ],
            'setting_id',
            'suffix',
            'notes:ntext',
            'income_terms:ntext',
            'product_code',
            'created_at:date',
            'updated_at:date',
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    [
                        'label'=>'Received #',
                       'attribute' => 'shippment_no',
                       'value' => function ($data) {
                        return 'GRN'.$data->shippment_no;
                       }
                    ],
                    [
                        'label'=>'Vendor/Source',
                       'attribute' => 'vendor_id',
                       'value' => function ($data) {
                        $value=Vendor::find()->where(['id'=>$data->vendor_id])->one();
                        return $value?$value->business_name:'';
                       }
                    ],
                    'date:date',
                    [
                        'label'=>'Status',
                       'attribute' => 'partially',
                       'value' => function ($data) {
                        return $data->partially?'Partially Received':'Fully Received';
                       }
                    ],
                    
                ],
            ]);
        }
    if($detail)
    return $exporter->send('product_received_details.xlsx');
    return $exporter->send('product_received.xlsx');
    }
    public function actionGoodsReceiptTemplate($id=null,$from=null)
    {

        $check_id = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $query=ProductReceived::find()->where(['is_deleted'=>0])->orderBy(['id'=>SORT_DESC]);
        }
        else
        {
            $accessed = Yii::$app->user->identity->id;
            $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
            $customer_ids = [];
            $j = 0;
            foreach($customers as $data)
            {
                $customer_ids[$j] = $data->client_id;
                $j++;
            }
            $query = ProductReceived::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
        }

        if($from=='all')
        {
            Yii::$app->view->params['active']=100;
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
            ]);
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->WriteHTML($this->renderPartial('exportpdf', [
                'dataProvider' => $dataProvider,

            ]));
            $mpdf->Output('product_received.pdf', 'I');
        }
        else
        {
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_for_purchase = ProductReceived::find()->where(['id'=>$id])->one();
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['product-received/index']);
                }
            }
              $model = $this->findModel($id);
              $data=ReceivedProductList::find()->where(['received_id'=>$id,'is_deleted'=>0])->all();
              $count=ReceivedProductList::find()->where(['received_id'=>$id,'is_deleted'=>0])->count();
              $mpdf = new mPDF;
              $mpdf->showImageErrors = true;
              $mpdf->WriteHTML($this->renderPartial('/product-received/goods-receipt-template', [
                    'model' => $model,
                    'products'=>$data,
                    'count' =>$count,
                ]));
             $mpdf->Output('goods-receipt-template.pdf', 'I');
        }
    }

    public function actionExportPdf()
    {
        Yii::$app->view->params['active']=100;
        $dataProvider = new ActiveDataProvider([
            'query' => ProductReceived::find(),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $mpdf->WriteHTML($this->renderPartial('exportpdf', [
            'dataProvider' => $dataProvider,

        ]));
        $mpdf->Output('product_received.pdf', 'D');
    }
    public function actionGetProduct()
    {
        $product=Product::find()->all();
        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        return json_encode($productlist);
    }
    public function actionGetVendor()
    {
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        return json_encode($vendorlist);
    }
    public function actionGetDescription()
    {
        $product=Product::find()->where(['id'=>Yii::$app->request->post('data')])->one()->category;
        return $product;
    }
    /**
     * Finds the ProductReceived model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ProductReceived the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = ProductReceived::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionVendordetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $customer=Vendor::find()->limit(10)->all();
        else
        $customer=Vendor::find()->where(["like","business_name",$searchTerm])->limit(10)->all();

        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    public function actionProductdetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $product=Product::find()->limit(10)->all();
        else
        $product=Product::find()->where(["like","product_name",$searchTerm])->limit(10)->all();

        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        return json_encode($productlist);
    }

    public function actionSelectdate($selected_date=null)
    {   $selected_date=$_GET['selected_date']; 
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
        $count_before_year=ProductReceived::find()->where(['<=', 'date', $dynamic_date])->count();
        $count_after_year=ProductReceived::find()->where(['>=', 'date', $dynamic_date])->count();

        if(strtotime($selected_date) <= strtotime($dynamic_date)) { 
            $last_entry=$count_before_year+1;
        }
        else if(strtotime($selected_date) > strtotime($dynamic_date)){
            $last_entry=$count_after_year+1;
        } 
        else{
            $last_entry=0;
        }
        echo $last_entry;
    }
}
