<?php

namespace backend\modules\users\controllers;

use Yii;
use backend\modules\users\models\ProformaInvoice;
use backend\modules\users\models\ProformaInvoiceSearch;
use backend\modules\users\models\ProformaInvoiceDocument;
use backend\modules\users\models\ProformaInvoiceProduct;
use backend\modules\master\models\Product;
use backend\modules\master\models\LeadExecutive;
use backend\modules\master\models\Customer;
use backend\modules\users\models\Invoice;
use backend\modules\users\models\InvoiceProduct;
use backend\modules\users\models\ReceiptDetails;
use backend\modules\users\models\LeadsQuotes;
use backend\modules\users\models\LeadsQuotesProduct;
use backend\modules\users\models\SaleOrderProduct;
use backend\modules\users\models\SaleOrder;
use backend\modules\master\models\AccountingTax;
use yii\web\Controller;
use backend\modules\master\models\salesTemplates;
use backend\modules\master\models\GenericTemplate;
use yii\web\NotFoundHttpException;
use backend\models\AuthAssignment;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use Mpdf\Mpdf;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use yii\web\UploadedFile;
use yii\helpers\Json;
use yii\helpers\Url;
use backend\models\UserClient;
use backend\modules\master\GenericTemplates;

/**
 * ProformaInvoiceController implements the CRUD actions for ProformaInvoice model.
 */
class ProformaInvoiceController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    // 'delete' => ['POST'],
                ],
            ],
        ];
    }
    public static $sales_types = [
            "B2B" => "B2B",
            "B2C" => "B2C",
            "EXPORT" => "EXPORT",
            "ADV PAYMENT" => "ADV PAYMENT",
        ];
    public static $supply_types = [
            "Inter-State" => "Inter-State",
            "Intra-State" => "Intra-State",
            "Export" => "Export",
            "Exempted" => "Exempted",
        ];
    public static $qty_units = [
            "NOS" => "NOS",
            "PCS" => "PCS",
            "PKT" => "PKT",
        ];
    /**
     * Lists all ProformaInvoice models.
     * @return mixed
     */
    public function actionIndex()
    {
        Yii::$app->view->params['active'] = 24;
        $model = new ProformaInvoice();
        $searchModel = new ProformaInvoiceSearch();
        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $check_id])->one();
        if ($user_type->item_name == 'Admin') {
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        } else {

            $accessed = LeadExecutive::find()->where(['user_id' => $check_id])->one();
            if ($accessed != '') {
                $accessed_id[] = $accessed['type'];
            } else {
                $loged_user = AuthAssignment::find()->where(['user_id' => $check_id])->one();
                $accessed_id[] = $loged_user['item_name'];
            }
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams, $accessed_id);
        }
        if ($model->load(Yii::$app->request->get())) {

            if ($user_type->item_name == 'Admin') {
                $query = ProformaInvoice::find()->where(['is_deleted' => 0])->orderBy(['id' => SORT_DESC]);
            } else {
                $all = ProformaInvoice::find()->where(['is_deleted' => 0])->orderBy(['id' => SORT_DESC])->all();
                $exe_array = [];
                $i = 0;
                foreach ($all as $key => $val) {
                    $executivedetails = LeadExecutive::find()->where(['id' => $val['executive']])->one();
                    if ($executivedetails != '') {
                        if ($executivedetails->type == $accessed_id[0])
                            $exe_array[$i] = $executivedetails->id;
                        $i++;
                    }
                }
                $exe_array = array_unique($exe_array);

                $customers = UserClient::find()->where(['user_id' => $check_id])->andWhere(['client_type' => 'Customer'])->all();
                $customer_ids = [];
                $j = 0;
                foreach ($customers as $data) {
                    $customer = Customer::find()->where(['id' => $data->client_id])->one();
                    if ($customer) {
                        $customer_ids[$j] = $customer->business_name;
                        $j++;
                    }
                }


                $query = ProformaInvoice::find()->where(['is_deleted' => 0])->andWhere(['in', 'customer_name', $customer_ids])->andWhere(['in', 'executive', $exe_array])->orderBy(['id' => SORT_DESC]);
            }
            if ($model->findid != '') {
                if ($model->send_email != '') {
                    $message = Yii::$app->mailer->compose()
                        ->setFrom([trim('ticket.system11@gmail.com') => "Mohapatra scientific"])
                        ->setTo(trim($model->send_email))
                        ->setSubject($model->email_subject)
                        ->setHtmlBody(nl2br($model->email_message));

                    if (!empty($model->cc_to_email != "")) {
                        $message->setBcc(trim($model->bcc_to_email));
                    }
                    if (!empty($model->bcc_to_email != "")) {
                        $message->setBcc(trim($model->bcc_to_email));
                    }
                    Yii::$app->mailer->send($message);
                    Yii::$app->getSession()->setFlash('success', 'Mail send Successfully.');
                    return $this->redirect(['proforma-invoice/index']);
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Mail To can not be blank.');
                    return $this->redirect(['proforma-invoice/index']);
                }
            } else {
                $query->andFilterWhere(['like', 'proforma_invoice_id', $model->proforma])
                    ->andFilterWhere(['between', 'date', $model->from_date, $model->to_date])
                    ->andFilterWhere(['like', 'customer_name', $model->customer])
                    ->andFilterWhere(['like', 'setting_id', $model->setting])
                    ->andFilterWhere(['like', 'total_price', $model->total_amount]);
                $dataProvider = new ActiveDataProvider(['query' => $query]);
            }
        }

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'model' => $model,
        ]);
    }

    /**
     * Displays a single ProformaInvoice model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id, $from = null)
    {
        Yii::$app->view->params['active'] = 24;

        $loged_by = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();

        if ($user_type->item_name != 'Admin') {
            $check_for_inv = ProformaInvoice::find()->where(['id' => $id])->one();
            $customer_dtl = Customer::find()->where(['business_name' => $check_for_inv->customer_name])->one();
            $check_customer = UserClient::find()->where(['user_id' => $loged_by])->andWhere(['client_id' => $customer_dtl->id])->one();
            if ($check_customer == '') {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['proforma-invoice/index']);
            }
        }

        $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
        return $this->render('view', [
            'model' => $this->findModel($id),
            'products' => $products,
            'from' => $from,
        ]);
    }

    /**
     * Creates a new ProformaInvoice model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate($receipt = 0, $id = null, $from = null)
    {
        Yii::$app->view->params['active'] = 24;
        $model = new ProformaInvoice();
        $document = new ProformaInvoiceDocument();
        $proforma_products = new ProformaInvoiceProduct();
        $proproducts = new ProformaInvoiceProduct();


        if ($id) {
            if ($from == 'leads-quotes') {
                $statuscheck = ProformaInvoice::find()->where(['lead_quote_id' => $id, 'is_deleted' => 0])->one();
                if ($statuscheck) {
                    Yii::$app->getSession()->setFlash('success', 'Proforma Invoice for this Lead Quote already exist.');
                    return $this->redirect(['leads-quotes/index']);
                } else {
                    $leadsdata = LeadsQuotes::find()->where(['id' => $id])->one();
                    // $model->setAttributes($data->attributes);
                    $name = Customer::find()->where(['id' => $leadsdata->customer])->one();
                    $model->customer_name = $name->business_name;
                    $model->executive = $leadsdata->executive;
                    $model->sales_type = $leadsdata->sales_type;
                    $model->notes = $leadsdata->notes;
                    $model->receiver_gstn = $leadsdata->receiver_gstn;
                    $model->supply_reverse_change = $leadsdata->supply_attracts_rcharge;
                    $model->sub_total_price = $leadsdata->sub_total_amount;
                    $model->total_price = $leadsdata->total_amount;
                    $model->terms = $leadsdata->terms;
                    $model->supply_type = $leadsdata->supply_type;
                    $model->reminder = $leadsdata->set_reminder;
                    $model->reminder_for = $leadsdata->reminder_for;
                    $model->reminder_through = $leadsdata->reminder_through;
                    $model->reminder_content = $leadsdata->reminder_content;
                    $model->reminder_from_date = $leadsdata->reminder_from_date;
                    $model->reminder_to_date = $leadsdata->reminder_to_date;
                    $model->add_delivery_details = $leadsdata->add_delivery_details;
                    $model->suppliers_reference = $leadsdata->suppliers_ref;
                    $model->reminder_emails = $leadsdata->reminder_email;
                    $model->reminder_repeat_on = $leadsdata->reminder_repeat;
                    $model->point_of_supply = $leadsdata->pointof_supply;
                    $model->delivery_note = $leadsdata->delivery_note;
                    $model->other_reference = $leadsdata->other_reference;
                    $model->is_invoice_paid_provisional = $leadsdata->Is_tax_paid;
                    $model->lead_quote_id = $id;
                    $payment_mode = Customer::find()->where(['id' => $leadsdata->customer])->one();
                    if ($payment_mode->credit_terms != '') {
                        $model->terms_of_payment = $payment_mode->credit_terms;
                    }
                    $proforma_products = LeadsQuotesProduct::find()->where(['leads_quotes_id' => $id])->all();
                }
            }
            if ($from == 'saleorder') {
                $check = ProformaInvoice::find()->where(['sale_order_id' => $id, 'is_deleted' => 0])->one();
                if ($check) {
                    Yii::$app->getSession()->setFlash('success', 'ProformaInvoice for this sale order already exist.');
                    return $this->redirect(['sale-order/index']);
                } else {
                    $saleorder_data = SaleOrder::find()->where(['id' => $id])->one();
                    // $model->setAttributes($saleorder_data->attributes);
                    $name = Customer::find()->where(['id' => $saleorder_data->customer_id])->one();
                    $model->customer_name = $name->business_name;
                    $model->executive = $saleorder_data->executive_id;
                    $model->sales_type = $saleorder_data->sale_type;
                    $model->notes = $saleorder_data->note;
                    $model->receiver_gstn = $saleorder_data->receiver;
                    $model->supply_reverse_change = $saleorder_data->reverse_charge;
                    $model->sub_total_price = $saleorder_data->sub_total_amount;
                    $model->total_price = $saleorder_data->total_amount;
                    $model->terms = $saleorder_data->terms;
                    $model->point_of_supply = $saleorder_data->point_of_supply;
                    $model->is_invoice_paid_provisional = $saleorder_data->is_invoice_paid_provisional;
                    $model->terms_of_payment = $saleorder_data->terms_of_payment;
                    $model->due_date = $saleorder_data->due_date;
                    $model->supply_type = $saleorder_data->supply_type;
                    $model->sale_order_id = $id;
                    $model->lead_quote_id = $saleorder_data->lead_quote_id;
                    $model->status = $saleorder_data->order_status;
                    $model->is_tcs = $saleorder_data->is_tcs;
                    $model->round_off = $saleorder_data->round_off;
                    $proforma_products = SaleOrderProduct::find()->where(['sale_order_no' => $id])->all();
                }
            }
            if ($from == 'proformainvoice') {
                $data = $this->findModel($id)->attributes;
                $model->setAttributes($data);
                $proforma_products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
            }
        }

        $selected_date = date('Y-m-d');
        $year = date("Y");
        $dynamic_date = $year . '-' . '03' . '-' . '31';
        if (strtotime($selected_date) <= strtotime($dynamic_date)) {
            $last_entry = ProformaInvoice::find()->where(['<=', 'date', $dynamic_date])->count();
        } elseif (strtotime($selected_date) > strtotime($dynamic_date)) {
            $last_entry = ProformaInvoice::find()->where(['>=', 'date', $dynamic_date])->count();
        } else {
            $last_entry = 0;
        }


        if (ProformaInvoice::find()->count() > 0) {
            $total = ProformaInvoice::find()->orderBy(['id' => SORT_DESC])->one();
            $count = $total->id;
        } else {
            $count = 0;
        }



        $data = Yii::$app->request->post();
        if ($model->load(Yii::$app->request->post()) && $document->load(Yii::$app->request->post())) {


            $loged_by = Yii::$app->user->identity->id;
            $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();

            if ($user_type->item_name != 'Admin') {
                $customer_dtl = Customer::find()->where(['business_name' => $model->customer_name])->one();
                if ($customer_dtl != '') {
                    $check_customer = UserClient::find()->where(['user_id' => $loged_by])->andWhere(['client_id' => $customer_dtl->id])->one();
                    if ($check_customer == '') {
                        Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                        return $this->redirect(['proforma-invoice/index']);
                    }
                } else {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['proforma-invoice/index']);
                }
            }

            $model->created_by = Yii::$app->user->identity->id;
            $model->round_off = Yii::$app->request->post('roundoffval');

            if ($from == 'saleorder') {
                $saleorder_data->converted_to = 'Proforma Invoice';
                if ($model->lead_quote_id != '') {
                    $leadsquote = LeadsQuotes::find()->where(['id' => $model->lead_quote_id])->one();
                    $leadsquote->converted_to = '';
                }
                $saleorder_data->save(false);
            }
            if ($from == 'leads-quotes') {
                $leadsdata->converted_to = 'Proforma Invoice';
                $leadsdata->status = 'Processing Quotes';
                $leadsdata->save(false);
            }
            // $proforma_id=$count+1;  
            $model->proforma_invoice_id = $model->prefix . '' . $model->proforma_invoice_no . '' . $model->suffix;

            if (Yii::$app->request->post('reminder'))
                $model->reminder = 'Yes';
            else
                $model->reminder = 'No';
            if (Yii::$app->request->post('delivery_details'))
                $model->add_delivery_details = 'Yes';
            else
                $model->add_delivery_details = 'No';
            if ($model->reminder_through == 1 || $model->reminder_through == 'Yes') {
                $model->reminder_through = 'Yes';
            } else {
                $model->reminder_through = 'No';
            }
            if ($model->reminder == 'No') {
                $model->reminder_for = '';
                $model->reminder_through = 'No';
                $model->reminder_content = '';
                $model->reminder_emails = '';
                $model->reminder_repeat_on = '';
                $model->reminder_from_date = '';
                $model->reminder_to_date = '';
            }
            if ($model->add_delivery_details == 'No') {
                $model->delivery_note = '';
                $model->suppliers_reference = '';
                $model->other_reference = '';
            }
            if ($model->sale_order_id == '') {
                $model->status = "Pending";
            }

            $model->converted_to = 'Proforma Invoice';
            $model->receipt_no = $receipt;

            if ($model->point_of_supply == 21) {
                $model->igst_amount = 0;
            }
            if ($model->point_of_supply != 21) {
                $model->cgst_amount = 0;
                $model->sgst_amount = 0;
            }
            if ($model->validate() && $model->save(false)) {
                if (isset($data['productdyn'])) {
                    foreach ($data['productdyn'] as $value) {
                        if ($value['price'] != '') {
                            $product = new ProformaInvoiceProduct();
                            $product->product_id = $value['product'];
                            $product->description = $value['desc'];
                            $product->quantity = $value['quantity'];
                            $product->units = $value['units'];
                            $product->price = $value['price'];
                            $product->unit_price = $value['unitprice'];
                            $product->discount = $value['discount'];
                            $product->tax = $value['tax'];
                            $product->discount_type = $value['discounttype'];
                            $product->tax_included = $value['tax_included'];
                            $product->amount = $value['amount'];
                            $product->proforma_invoice_no = $model->id;
                            $product->created_at = date('Y-m-d H:i:s');
                            $product->updated_at = date('Y-m-d H:i:s');
                            $product->save(false);
                        }
                    }
                }
                if (isset($data['productdynupdate'])) {

                    foreach ($data['productdynupdate'] as $value) {
                        if ($value['price'] != '') {
                            $productpinv = new ProformaInvoiceProduct();
                            $productpinv->product_id = $value['product'];
                            $productpinv->description = $value['desc'];
                            $productpinv->quantity = $value['quantity'];
                            $productpinv->units = $value['units'];
                            $productpinv->price = $value['price'];
                            $productpinv->unit_price = $value['unitprice'];
                            $productpinv->discount = $value['discount'];
                            $productpinv->tax = $value['tax'];
                            $productpinv->discount_type = $value['discounttype'];
                            $productpinv->tax_included = $value['tax_included'];
                            $productpinv->amount = $value['amount'];
                            $productpinv->proforma_invoice_no = $model->id;
                            $productpinv->created_at = date('Y-m-d H:i:s');
                            $productpinv->updated_at = date('Y-m-d H:i:s');
                            $productpinv->save(false);
                        }
                    }
                }
                if ($document->validate()) {
                    $names = UploadedFile::getInstances($document, 'document_name');
                    foreach ($names as $name) {
                        $path = 'invoicedocuments/proforma-documents/' . $name->baseName . '.' . $name->extension;
                        if ($name->saveAs($path)) {
                            $filename = $name->baseName . '.' . $name->extension;
                            $filepath = $path;
                            yii::$app->db->createCommand()->insert('proforma_invoice_document', ['document_name' => $filename, 'proforma_invoice_id' => $model->id])->execute();
                        }
                    }
                }
                if (Yii::$app->request->post('save') == 0) {
                    Yii::$app->getSession()->setFlash('success', 'Successfully created the ProformaInvoice');
                    return $this->redirect(['proforma-invoice/index']);
                }

                if (Yii::$app->request->post('save') == 1) {
                    Yii::$app->getSession()->setFlash('success', 'Successfully created the ProformaInvoice');
                    return $this->redirect(['proforma-invoice/create']);
                }
            }
        }
        return $this->render('create', [
            'model' => $model,
            'document' => $document,
            'count' => $count,
            'proforma_products' => $proforma_products,
            'sales_types' => ProformaInvoiceController::$sales_types,
            'supply_types' => ProformaInvoiceController::$supply_types,
            'qty_units' => ProformaInvoiceController::$qty_units,
            'last_entry' => $last_entry + 1,
            'from' => $from,
            'proproducts' => $proproducts,
        ]);
    }


    /**
     * Updates an existing ProformaInvoice model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active'] = 24;

        $loged_by = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();

        if ($user_type->item_name != 'Admin') {
            $check_for_inv = ProformaInvoice::find()->where(['id' => $id])->one();
            $customer_dtl = Customer::find()->where(['business_name' => $check_for_inv->customer_name])->one();
            $check_customer = UserClient::find()->where(['user_id' => $loged_by])->andWhere(['client_id' => $customer_dtl->id])->one();
            if ($check_customer == '') {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['proforma-invoice/index']);
            }
        }



        $model = ProformaInvoice::find()->where(['is_deleted' => 0, 'id' => $id])->one();
        if ($model == null) {
            return $this->redirect(['proforma-invoice/index']);
        }
        $data = Yii::$app->request->post();
        $last_entry = ProformaInvoice::find()->count();
        $document = new ProformaInvoiceDocument();
        $proforma_products = new ProformaInvoiceProduct();
        $proforma_invoice_products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();

        if ($model->load(Yii::$app->request->post()) && $document->load(Yii::$app->request->post())) {
            $model->round_off = Yii::$app->request->post('roundoffval');
            $model->updated_at = date('Y-m-d H:i:s');

            if (Yii::$app->request->post('reminder'))
                $model->reminder = 'Yes';
            else
                $model->reminder = 'No';
            if (Yii::$app->request->post('delivery_details'))
                $model->add_delivery_details = 'Yes';
            else
                $model->add_delivery_details = 'No';
            if ($model->reminder_through == 1 || $model->reminder_through == 'Yes') {
                $model->reminder_through = 'Yes';
            } else {
                $model->reminder_through = 'No';
            }
            if ($model->reminder == 'No') {
                $model->reminder_for = '';
                $model->reminder_through = 'No';
                $model->reminder_content = '';
                $model->reminder_emails = '';
                $model->reminder_repeat_on = '';
                $model->reminder_from_date = '';
                $model->reminder_to_date = '';
            }
            if ($model->reminder_through == 'No') {
                $model->reminder_content = '';
                $model->reminder_emails = '';
                $model->reminder_repeat_on = '';
                $model->reminder_from_date = '';
                $model->reminder_to_date = '';
            }
            if ($model->add_delivery_details == 'No') {
                $model->delivery_note = '';
                $model->suppliers_reference = '';
                $model->other_reference = '';
            }
            if ($model->point_of_supply == 21) {
                $model->igst_amount = 0;
            }
            if ($model->point_of_supply != 21) {
                $model->cgst_amount = 0;
                $model->sgst_amount = 0;
            }
            if ($model->sale_order_id == '') {
                $model->status = "Pending";
            }
            if ($model->validate() && $model->save(false)) {

                $purchase_order_product_delete = ProformaInvoiceProduct::deleteAll(['proforma_invoice_no' => $id]);
                if (isset($data['productdyn'])) {
                    foreach ($data['productdyn'] as $value) {
                        if ($value['price'] != '') {
                            $product = new ProformaInvoiceProduct();
                            $product->product_id = $value['product'];
                            $product->description = $value['desc'];
                            $product->quantity = $value['quantity'];
                            $product->units = $value['units'];
                            $product->price = $value['price'];
                            $product->unit_price = $value['unitprice'];
                            $product->discount = $value['discount'];
                            $product->tax = $value['tax'];
                            $product->discount_type = $value['discounttype'];
                            $product->tax_included = $value['tax_included'];
                            $product->amount = $value['amount'];
                            $product->proforma_invoice_no = $model->id;
                            $product->created_at = date('Y-m-d H:i:s');
                            $product->updated_at = date('Y-m-d H:i:s');
                            $product->save(false);
                        }
                    }
                }

                if (isset($data['productdynupdate'])) {

                    foreach ($data['productdynupdate'] as $value) {
                        if ($value['price'] != '') {
                            $productpinv = new ProformaInvoiceProduct();
                            $productpinv->product_id = $value['product'];
                            $productpinv->description = $value['desc'];
                            $productpinv->quantity = $value['quantity'];
                            $productpinv->units = $value['units'];
                            $productpinv->price = $value['price'];
                            $productpinv->unit_price = $value['unitprice'];
                            $productpinv->discount = $value['discount'];
                            $productpinv->tax = $value['tax'];
                            $productpinv->discount_type = $value['discounttype'];
                            $productpinv->tax_included = $value['tax_included'];
                            $productpinv->amount = $value['amount'];
                            $productpinv->proforma_invoice_no = $model->id;
                            $productpinv->created_at = date('Y-m-d H:i:s');
                            $productpinv->updated_at = date('Y-m-d H:i:s');
                            $productpinv->save(false);
                        }
                    }
                }

                if ($document->validate()) {
                    $names = UploadedFile::getInstances($document, 'document_name');
                    foreach ($names as $name) {
                        $path = 'invoicedocuments/proforma-documents/' . $name->baseName . '.' . $name->extension;
                        if ($name->saveAs($path)) {
                            $filename = $name->baseName . '.' . $name->extension;
                            $filepath = $path;
                            yii::$app->db->createCommand()->insert('proforma_invoice_document', ['document_name' => $filename, 'proforma_invoice_id' => $model->id])->execute();
                        }
                    }
                }
                if (Yii::$app->request->post('save') == 0) {
                    Yii::$app->getSession()->setFlash('success', 'Data Updated Successfully');
                    return $this->redirect(['proforma-invoice/index']);
                }

                if (Yii::$app->request->post('save') == 1) {
                    Yii::$app->getSession()->setFlash('success', 'Data Updated Successfully');
                    return $this->redirect(['proforma-invoice/create']);
                }
            }
        }

        return $this->render('update', [
            'model' => $model,
            'document' => $document,
            'proforma_products' => $proforma_products,
            'proforma_invoice_products' => $proforma_invoice_products,
            'sales_types' => ProformaInvoiceController::$sales_types,
            'supply_types' => ProformaInvoiceController::$supply_types,
            'qty_units' => ProformaInvoiceController::$qty_units,
            'last_entry' => $last_entry + 1,
        ]);
    }

    public function actionFindattachments($id)
    {
        $find = ProformaInvoiceDocument::find()->select(['document_name', 'id'])->where(['proforma_invoice_id' => $id])->asArray()->all();
        echo Json::encode($find);
    }
    public function actionDownload($id)
    {
        $model = ProformaInvoiceDocument::find()->where(['id' => $id])->one();
        if ($model->document_name != null) {
            $local = yii::getAlias('@webroot/invoicedocuments/proforma-documents/');
            $name = $model->document_name;
            $find = $local . $name;
            if (file_exists($find)) {
                return yii::$app->response->sendFile($find);
            }
        }
    }
    public function actionFindproduct($productid)
    {
        $productdetails = Product::find()->where(['id' => $productid])->one();
        echo Json::encode($productdetails);
    }
    public function actionProductdetails($searchTerm = null)
    {
        if (!$searchTerm)
            $product = Product::find()->limit(10)->all();
        else
            $product = Product::find()->where(["like", "product_name", $searchTerm])->limit(10)->all();

        $productlist = ArrayHelper::map($product, 'id', 'product_name');
        return json_encode($productlist);
    }


    /**
     * Deletes an existing ProformaInvoice model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id = null)
    {
        $loged_by = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();
        if ($id == null) {
            $ids = Yii::$app->request->post('data');
            if ($ids) {
                foreach ($ids as $id) {
                    $document = ProformaInvoiceDocument::find()->where(['proforma_invoice_id' => $id])->all();
                    foreach ($document as $value) {
                        $value->is_deleted = 1;
                        $value->save(false);
                    }
                    $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
                    foreach ($products as $products) {
                        $products->is_deleted = 1;
                        $products->save(false);
                    }
                    $data = $this->findModel($id);
                    $data->is_deleted = 1;
                    $data->save(false);
                }
            }
            Yii::$app->getSession()->setFlash('success', 'Successfully deleted selected items');
            return $this->redirect(['proforma-invoice/index']);
        } else {
            $loged_by = Yii::$app->user->identity->id;
            $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();

            if ($user_type->item_name != 'Admin') {
                $check_for_inv = ProformaInvoice::find()->where(['id' => $id])->one();
                $customer_dtl = Customer::find()->where(['business_name' => $check_for_inv->customer_name])->one();
                $check_customer = UserClient::find()->where(['user_id' => $loged_by])->andWhere(['client_id' => $customer_dtl->id])->one();
                if ($check_customer == '') {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['proforma-invoice/index']);
                }
            }
            $data = $this->findModel($id);
            $data->is_deleted = 1;
            $data->save(false);
            $document = ProformaInvoiceDocument::find()->where(['proforma_invoice_id' => $id])->all();
            foreach ($document as $value) {
                $value->is_deleted = 1;
                $value->save(false);
            }
            $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
            foreach ($products as $products) {
                $products->is_deleted = 1;
                $products->save(false);
            }

            Yii::$app->getSession()->setFlash('success', 'Successfully deleted');
            return $this->redirect(['proforma-invoice/index']);
        }
        // $this->findModel($id)->delete();

    }


    public function actionExcelExport($detail = null)
    {
    }

    public function actionProformaInvoicePdf($id = null, $from = null)
    {
        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $check_id])->one();
        $accessed_id[] = $user_type->item_name;
        if ($user_type->item_name == 'Admin') {
            $query = ProformaInvoice::find()->where(['is_deleted' => 0])->orderBy(['id' => SORT_DESC]);
        } else {
            $all = ProformaInvoice::find()->where(['is_deleted' => 0])->orderBy(['id' => SORT_DESC])->all();
            $exe_array = [];
            $i = 0;
            foreach ($all as $key => $val) {
                $executivedetails = LeadExecutive::find()->where(['id' => $val['executive']])->one();
                if ($executivedetails != '') {
                    if ($executivedetails->type == $accessed_id[0])
                        $exe_array[$i] = $executivedetails->id;
                    $i++;
                }
            }
            $exe_array = array_unique($exe_array);

            $customers = UserClient::find()->where(['user_id' => $check_id])->andWhere(['client_type' => 'Customer'])->all();
            $customer_ids = [];
            $j = 0;
            foreach ($customers as $data) {
                $customer = Customer::find()->where(['id' => $data->client_id])->one();
                if ($customer) {
                    $customer_ids[$j] = $customer->business_name;
                    $j++;
                }
            }


            $query = ProformaInvoice::find()->where(['is_deleted' => 0])->andWhere(['in', 'customer_name', $customer_ids])->andWhere(['in', 'executive', $exe_array])->orderBy(['id' => SORT_DESC]);
        }
        if ($from == 'all') {
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
            ]);
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/proforma-invoice/export-pdf', [
                'dataProvider' => $dataProvider,

            ]));
            $mpdf->Output('Proforma-invoice.pdf', 'I');
        } else {

            $loged_by = Yii::$app->user->identity->id;
            $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();

            if ($user_type->item_name != 'Admin') {
                $check_for_inv = ProformaInvoice::find()->where(['id' => $id])->one();
                $customer_dtl = Customer::find()->where(['business_name' => $check_for_inv->customer_name])->one();
                $check_customer = UserClient::find()->where(['user_id' => $loged_by])->andWhere(['client_id' => $customer_dtl->id])->one();
                if ($check_customer == '') {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['proforma-invoice/index']);
                }
            }
            $dataProvider = new ActiveDataProvider([
                'query' => ProformaInvoice::find()->where(['id' => $id]),
            ]);
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $model = $this->findModel($id);
            $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
            $mpdf->showImageErrors = true;
            $mpdf->WriteHTML($this->renderPartial('/proforma-invoice/proforma-invoice-pdf', [
                'dataProvider' => $dataProvider,
                'model' => $model,
                'products' => $products,
            ]));
            $find = ProformaInvoice::findOne($id);
            $proforma_invoice_id = $find->proforma_invoice_id;
            $mpdf->Output($proforma_invoice_id . '-ProformaInvoice.pdf', 'I');
        }
    }
    public function actionProformaInvoicePrint($id)
    {
        // print_r($id);
        // exit();
        $dataProvider = new ActiveDataProvider([
            'query' => ProformaInvoice::find()->where(['id' => $id]),
        ]);
        $dataProvider->pagination = false;
        $mpdf = new mPDF;
        $model = $this->findModel($id);
        $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
        $mpdf->showImageErrors = true;
        $mpdf->WriteHTML($this->renderPartial('/proforma-invoice/proforma-invoice-pdf', [
            'dataProvider' => $dataProvider,
            'model' => $model,
            'products' => $products,
        ]));
        $find = ProformaInvoice::findOne($id);
        $proforma_invoice_id = $find->proforma_invoice_id;
        $mpdf->Output($proforma_invoice_id . '-ProformaInvoice.pdf', 'I');
    }
    public function actionExcelProforma($id = null, $detail = null)
    {
        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $check_id])->one();
        $accessed_id[] = $user_type->item_name;
        if ($user_type->item_name == 'Admin') {
            $query = ProformaInvoice::find()->where(['is_deleted' => 0])->orderBy(['id' => SORT_DESC]);
        } else {
            $all = ProformaInvoice::find()->where(['is_deleted' => 0])->orderBy(['id' => SORT_DESC])->all();
            $exe_array = [];
            $i = 0;
            foreach ($all as $key => $val) {
                $executivedetails = LeadExecutive::find()->where(['id' => $val['executive']])->one();
                if ($executivedetails != '') {
                    if ($executivedetails->type == $accessed_id[0])
                        $exe_array[$i] = $executivedetails->id;
                    $i++;
                }
            }
            $exe_array = array_unique($exe_array);

            $customers = UserClient::find()->where(['user_id' => $check_id])->andWhere(['client_type' => 'Customer'])->all();
            $customer_ids = [];
            $j = 0;
            foreach ($customers as $data) {
                $customer = Customer::find()->where(['id' => $data->client_id])->one();
                if ($customer) {
                    $customer_ids[$j] = $customer->business_name;
                    $j++;
                }
            }


            $query = ProformaInvoice::find()->where(['is_deleted' => 0])->andWhere(['in', 'customer_name', $customer_ids])->andWhere(['in', 'executive', $exe_array])->orderBy(['id' => SORT_DESC]);
        }


        if ($id == null) {
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
            ]);

            if ($detail) {
                $exporter = new Spreadsheet([
                    'dataProvider' => $dataProvider,
                    'columns' => [
                        'setting_id',
                        'customer_name',
                        'notes',
                        [
                            'label' => 'Executive',
                            'format'    => 'raw',
                            // 'attribute' => 'customer_name',
                            'value' => function ($data) {
                                $user = LeadExecutive::find()->where(['id' => $data['executive']])->one();
                                return $user['name'];
                            },
                        ],
                        'receiver_gstn',
                        'sales_type',
                        'supply_type',
                        'supply_reverse_change',
                        'point_of_supply',
                        'is_invoice_paid_provisional',
                        'prefix',
                        'proforma_invoice_no',
                        'suffix',
                        'proforma_invoice_id',
                        'date',
                        'due_date',
                        'terms_of_payment',
                        'sub_total_price',
                        'total_price',
                        'reminder',
                        'reminder_for',
                        'reminder_through',
                        'reminder_emails',
                        'reminder_content',
                        'reminder_repeat_on',
                        'reminder_from_date',
                        'reminder_to_date',
                        'add_delivery_details',
                        'delivery_note',
                        'suppliers_reference',
                        'other_reference',
                        'converted_to',
                        'terms',
                        [
                            'label' => 'Status',
                            'headerOptions' => ['style' => 'text-align: center;'],

                            'value' => function ($data) {
                                if ($data['status']) {
                                    // return $data['status'];
                                    if ($data['status'] == 'Pending') {
                                        return 'DRAFT';
                                    } else if ($data['status'] == 'Paid') {
                                        return 'PAID';
                                    } else if ($data['status'] == 'Draft') {
                                        return 'DRAFT';
                                    }
                                }
                            }
                        ],
                        [
                            'label' => 'Documents',
                            'format'    => 'raw',
                            // 'attribute' => 'customer_name',
                            'value' => function ($data) {
                                $user = ProformaInvoiceDocument::find()->select(['document_name'])->where(['proforma_invoice_id' => $data['id']])->asArray()->column();
                                $documents = implode(" , ", $user);
                                return $documents;
                            },
                        ],
                        'created_at',
                        'updated_at',
                    ],
                ]);
            } else {
                $exporter = new Spreadsheet([
                    'dataProvider' => $dataProvider,
                    'columns' => [
                        'proforma_invoice_id',
                        'customer_name',
                        'date',
                        'total_price',
                        'converted_to',
                        [
                            'label' => 'Paid',
                            'value' => function ($data) {
                                $amount = ReceiptDetails::find()->select(['total_amount'])->where(['id' => $data->receipt_no])->asArray()->one();
                                $amt = $data->total_price - $amount;
                                $paid_amount = $data->total_price - $amt;
                                return $paid_amount;
                            }
                        ],
                        [
                            'label' => 'Pending',
                            'value' => function ($data) {
                                $amount = ReceiptDetails::find()->select(['total_amount'])->where(['id' => $data->receipt_no])->asArray()->one();
                                $amt = $data->total_price - $amount;
                                return $amt;
                            }
                        ],

                    ],
                ]);
            }

            return $exporter->send('Proforma-invoice.xlsx');
        } else {
            $loged_by = Yii::$app->user->identity->id;
            $user_type = AuthAssignment::find()->where(['user_id' => $loged_by])->one();

            if ($user_type->item_name != 'Admin') {
                $check_for_inv = ProformaInvoice::find()->where(['id' => $id])->one();
                $customer_dtl = Customer::find()->where(['business_name' => $check_for_inv->customer_name])->one();
                $check_customer = UserClient::find()->where(['user_id' => $loged_by])->andWhere(['client_id' => $customer_dtl->id])->one();
                if ($check_customer == '') {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['proforma-invoice/index']);
                }
            }
            $dataProvider = new ActiveDataProvider([
                'query' => ProformaInvoice::find()->where(['id' => $id]),
            ]);
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    'setting_id',
                    'customer_name',
                    'notes',
                    [
                        'label' => 'Executive',
                        'format'    => 'raw',
                        // 'attribute' => 'customer_name',
                        'value' => function ($data) {
                            $user = LeadExecutive::find()->where(['id' => $data['executive']])->one();
                            if ($user)
                                return $user['name'];
                        },
                    ],
                    'receiver_gstn',
                    'sales_type',
                    'supply_type',
                    'supply_reverse_change',
                    'point_of_supply',
                    'is_invoice_paid_provisional',
                    'prefix',
                    'proforma_invoice_no',
                    'suffix',
                    'proforma_invoice_id',
                    'date',
                    'due_date',
                    'terms_of_payment',
                    'sub_total_price',
                    'total_price',
                    'reminder',
                    'reminder_for',
                    'reminder_through',
                    'reminder_emails',
                    'reminder_content',
                    'reminder_repeat_on',
                    'reminder_from_date',
                    'reminder_to_date',
                    'add_delivery_details',
                    'delivery_note',
                    'suppliers_reference',
                    'other_reference',
                    'converted_to',
                    'terms',
                    'status',
                    [
                        'label' => 'Documents',
                        'format'    => 'raw',
                        // 'attribute' => 'customer_name',
                        'value' => function ($data) {
                            $user = ProformaInvoiceDocument::find()->select(['document_name'])->where(['proforma_invoice_id' => $data['id']])->asArray()->column();
                            $documents = implode(" , ", $user);
                            return $documents;
                        },
                    ],
                    'created_at',
                    'updated_at',
                ],
            ]);
            $find = ProformaInvoice::findOne($id);
            $proforma_invoice_id = $find->proforma_invoice_id;
            return $exporter->send($proforma_invoice_id . '-ProformaInvoice.xlsx');
        }
    }

    public function actionDuplicate($id)
    {
        $model = new ProformaInvoice();
        $data = $this->findModel($id)->attributes;
        $model->setAttributes($data);
        $last_entry = ProformaInvoice::find()->count();
        $model->proforma_invoice_no = $last_entry + 1;
        $total = ProformaInvoice::find()->orderBy(['id' => SORT_DESC])->one();
        $no = $total->id;
        $count = $no + 1;
        $model->proforma_invoice_id = $model->prefix . '' . $count . '' . $model->suffix;
        $model->save(false);
        $document = ProformaInvoiceDocument::find()->where(['proforma_invoice_id' => $id])->all();
        foreach ($document as $value) {
            $doc = new ProformaInvoiceDocument();
            $data = $value->attributes;
            $doc->setAttributes($data);
            $doc->proforma_invoice_id = $model->id;
            $doc->save(false);
        }
        $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();

        foreach ($products as $product) {
            $pdt = new ProformaInvoiceProduct();
            $data = $product->attributes;
            $pdt->setAttributes($data);
            $pdt->proforma_invoice_no = $model->id;
            $pdt->save(false);
        }
        Yii::$app->getSession()->setFlash('success', 'Duplicate Created Successfully');
        return $this->redirect(['proforma-invoice/index']);
    }

    public function actionConvertInvoice($id)
    {
        $model = $this->findModel($id);
        $invoice = new Invoice();
        $find = Invoice::find()->where(['proforma_invoice_id' => $id])->one();
        if ($find) {
            $model->converted_to = 'Invoice';
            $model->save(false);
            Yii::$app->getSession()->setFlash('success', 'Invoice already exist for this Proforma Invoice');
            return $this->redirect(['proforma-invoice/index']);
        } else {
            if (Invoice::find()->count() > 0) {
                $total = Invoice::find()->orderBy(['id' => SORT_DESC])->one();
                $count = $total->id;
            } else {
                $count = 0;
            }
            $invoice->invoice_no = $count + 1;
            $invoice->prefix = 'INV';
            $invoice->suffix = '';
            $invoice->invoice_id = $invoice->prefix . '' . $invoice->invoice_no . '' . $invoice->suffix;
            $invoice->customer_name = $model->customer_name;
            $name = LeadExecutive::find()->where(['id' => $model->executive])->one();
            $invoice->executive = $name->name;
            $invoice->receiver_gstn = $model->receiver_gstn;
            $invoice->sales_type = $model->sales_type;
            $invoice->sypply_type = $model->supply_type;
            $invoice->supply_reverse_change = $model->supply_reverse_change;
            $invoice->point_of_supply = $model->point_of_supply;
            $invoice->due_date = $model->due_date;
            $invoice->terms_of_payment = $model->terms_of_payment;
            $invoice->tax_status = $model->is_invoice_paid_provisional;
            $invoice->sub_total_price = $model->sub_total_price;
            $invoice->total_price = $model->total_price;
            $invoice->notes = $model->notes;
            $invoice->proforma_invoice_id = $id;
            $invoice->date = date('Y-m-d H:i:s');
            $invoice->created_at = date('Y-m-d H:i:s');
            $invoice->updated_at = date('Y-m-d H:i:s');
            $invoice->save(false);
            $products = ProformaInvoiceProduct::find()->where(['proforma_invoice_no' => $id])->all();
            foreach ($products as $products) {
                $invoiceproduct = new InvoiceProduct();
                $invoiceproduct->invoice_id = $invoice->id;
                $invoiceproduct->product_id = $products->product_id;
                $invoiceproduct->description = $products->description;
                $invoiceproduct->quantity = $products->quantity;
                $invoiceproduct->units = $products->units;
                $invoiceproduct->price = $products->price;
                $invoiceproduct->discount = $products->discount;
                $invoiceproduct->tax = $products->tax;
                $invoiceproduct->discount_type = $products->discount_type;
                $invoiceproduct->tax_included = $products->tax_included;
                $invoiceproduct->amount = $products->amount;
                $invoiceproduct->created_at = date('Y-m-d H:i:s');
                $invoiceproduct->updated_at = date('Y-m-d H:i:s');
                $invoiceproduct->save(false);
            }
            $model->converted_to = 'Invoice';
            $model->save(false);
            Yii::$app->getSession()->setFlash('success', 'Successfully Generated Invoice');
            return $this->redirect(['proforma-invoice/index']);
        }
    }
    public function actionDelattachment($id)
    {
        $file = ProformaInvoiceDocument::find()->where(['id' => $id])->one();
        $file->delete();
        // return $exporter->send($find);
        // return $find;
    }

    public function actionFindcustomerdetails($customerid)
    {
        $customerdetails = Customer::find()->where(['business_name' => $customerid])->one();
        echo Json::encode($customerdetails);
    }
    public function actionFindexecutivedetails($executivedata)
    {
        $executivedetails = LeadExecutive::find()->where(['name' => $executivedata])->one();
        echo Json::encode($executivedetails);
    }
    public function actionCustomer($searchTerm = null)
    {
        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id' => $check_id])->one();

        if ($user_type->item_name == 'Admin') {
            if (!$searchTerm)
                $customer = Customer::find()->limit(10)->all();
            else
                $customer = Customer::find()->where(["like", "business_name", $searchTerm])->limit(10)->all();

            $customerlist = ArrayHelper::map($customer, 'business_name', 'business_name');
            return json_encode($customerlist);
        } else {
            $customer_ids = [];
            $i = 0;

            $customers = UserClient::find()->where(['user_id' => $check_id])->andWhere(['client_type' => 'Customer'])->all();
            foreach ($customers as $data) {
                $customer_ids[$i] = $data->client_id;
                $i++;
            }
            if (!$searchTerm)
                $customer = Customer::find()->where(['in', 'id', $customer_ids])->limit(10)->all();
            else
                $customer = Customer::find()->where(["like", "business_name", $searchTerm])->andWhere(['in', 'id', $customer_ids])->limit(10)->all();

            $customerlist = ArrayHelper::map($customer, 'business_name', 'business_name');
            return json_encode($customerlist);
        }
    }

    /**
     * Finds the ProformaInvoice model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ProformaInvoice the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = ProformaInvoice::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionFinddetails()
    {
        $taxdetails = Accountingtax::find()->select(['tax_name', 'accounting_tax_id'])->where(['is_deleted' => 0])
            ->andWhere(['tax_sub_type' => 'Output'])
            ->andWhere(['status' => 1])
            ->andWhere(['or', ['tax_class' => 'IGST'], ['tax_class' => 'TCS']])->all();
        $taxlist = ArrayHelper::map($taxdetails, 'accounting_tax_id', 'tax_name');
        return Json::encode($taxlist);
    }
    public function actionTaxdetails($item)
    {
        $taxdetails = AccountingTax::find()->where(['accounting_tax_id' => $item])->one();
        echo Json::encode($taxdetails);
    }

    public function actionFindproinvoicealldetails($id)
    {
        $all = ProformaInvoice::find()->where(['id' => $id])->one();
        echo Json::encode($all);
    }
    public function actionCreateUrl($id)
    {
        // $ids=Yii::$app->request->post('data');
        $url = Url::to(['proforma-invoice-pdf', 'id' => $id], 'https');
        echo Json::encode($url);
    }
    public function actionFinduseralldetails($id)
    {
        $all = ProformaInvoice::find()->where(['id' => $id])->one();
        $userdetails = Customer::find()->where(['business_name' => $all->customer_name])->one();
        echo Json::encode($userdetails);
    }

    public function actionProformainvoicesettingid($setting = null, $template = null)
    {
        //print_r(1);die();
        $find = GenericTemplate::find()->where(['setting_type' => 'Proforma Invoice'])->andWhere(['gt_id' => $setting])->count();
        echo Json::encode($find);
    }

    public function actionSelectdate($selected_date = null)
    {
        $selected_date = $_GET['selected_date'];
        $year = date("Y");
        $dynamic_date = $year . '-' . '03' . '-' . '31';
        $count_before_year = ProformaInvoice::find()->where(['<=', 'date', $dynamic_date])->count();
        $count_after_year = ProformaInvoice::find()->where(['>=', 'date', $dynamic_date])->count();
        if (strtotime($selected_date) <= strtotime($dynamic_date)) {
            $last_entry = $count_before_year + 1;
        } else if (strtotime($selected_date) > strtotime($dynamic_date)) {
            $last_entry = $count_after_year + 1;
        } else {
            $last_entry = 0;
        }
        echo $last_entry;
    }
}
