<?php

namespace backend\modules\users\controllers;

use Yii;
use backend\modules\users\models\PurchaseBill;
use backend\modules\users\models\BillDocuments;
use backend\modules\users\models\BillProducts;
use backend\modules\master\models\PurchaseOrderProduct;
use backend\modules\users\models\PurchasePayment;
use backend\modules\master\models\PurchaseOrder;
use backend\modules\users\models\AccountingDetails;
use backend\modules\master\models\Vendor;
use backend\modules\master\models\Product;
use backend\modules\users\models\PurchaseBillSearch;
use backend\modules\master\models\AccountingTax;
use backend\modules\users\models\ProductReceived;
use backend\modules\users\models\ReceivedProductList;
use backend\modules\users\models\AdjustedBills;
use backend\modules\users\models\DebitNote;
use backend\modules\users\models\BillsAdjustedAmounts;
use backend\modules\master\models\AccountingLedger;
use backend\modules\master\models\PurchaseAgainstGr;
use backend\models\AuthAssignment;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use Mpdf\Mpdf;
use yii\helpers\Json;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use yii\web\UploadedFile;
use yii\helpers\Url;
use backend\models\UserClient;
use backend\modules\master\models\PurchaseTemplates;
use backend\modules\master\models\GenericTemplate;

/**
 * PurchaseBillController implements the CRUD actions for PurchaseBill model.
 */
class PurchaseBillController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all PurchaseBill models.
     * @return mixed
     */
    public function actionIndex($totalexcel=null,$excel=null,$print=null)
    {
        Yii::$app->view->params['active']=106;
        $searchModel = new PurchaseBillSearch();

        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        }
        else
        {
            $accessed[] = Yii::$app->user->identity->id;
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams,$accessed);
        }
        $model=new PurchaseBill();
        $product=Product::find()->all();
        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        if ($model->load(Yii::$app->request->get()) ) {

            if($user_type->item_name == 'Admin')
            {
               $query=PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['is_deleted' =>0]);
            }
            else
            {
                $accessed = Yii::$app->user->identity->id;

                $vendors = UserClient::find()->where(['user_id'=>$accessed])->andWhere(['client_type'=>'Vendor'])->all();
                $vendor_ids= [];
                $i = 0;
                foreach($vendors as $data)
                {
                    $vendor_ids[$i] = $data->client_id;
                    $i++;
                }
                $query = PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['in', 'vendor_id', $vendor_ids])->andWhere(['is_deleted' =>0]);
            }
            //$query=PurchaseBill::find()->where(['is_deleted'=>0]);
            $query->andFilterWhere(['between', 'date', $model->from_date,$model->to_date])
            ->andFilterWhere(['=', 'vendor_id', $model->vendor])
            ->andFilterWhere(['like', 'setting_type', $model->setting])
	        ->andFilterWhere(['like', 'vendor_bill_no', $model->supplier_bill])
            ->andFilterWhere(['like', 'total', $model->amount]);  
            if($model->product){
                $productlists=BillProducts::find()->where(['product_id'=>$model->product,'is_deleted'=>0])->groupBy(['bill_id'])->all();
                $productid=ArrayHelper::map($productlists, 'bill_id','bill_id');
                $query->andFilterWhere(['in', 'id', $productid]) ; 
            }
            if($model->po)
            {
                $po_details = PurchaseOrder::find()->where(['po_no_final'=>$model->po])->one();
                $query->andFilterWhere(['=', 'purchase_order_id', $po_details->id]) ; 
            }  
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
                'sort'=> ['defaultOrder' => ['id' => SORT_DESC]],
            ]);
        }
	if($totalexcel!=null)
          { 
                $details = true;
                $this->actionExcelExport($dataProvider,$details);
          }
          if($excel!=null)
          { 
                $this->actionExcelExport($dataProvider);
          }
          if($print!=null)
          { 
                $form = "all";
                $this->actionPurchasebillTemplate($dataProvider,$form);
          }
        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'model'=>$model,
            'product'=>$productlist,
            'vendorlist'=>$vendorlist,
        ]);
    }

    /**
     * Displays a single PurchaseBill model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id=null,$from=null)
    {
        Yii::$app->view->params['active']=106;


        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = PurchaseBill::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['purchase-bill/index']);
            }
        }

        $adjusted = new BillsAdjustedAmounts();
        $modl = new PurchaseBill();

        $data=BillProducts::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();
        $data1=BillDocuments::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();

        $bill = PurchaseBill::find()->where(['id'=>$id])->one();

        $qurey1 = PurchasePayment::find()->where(['against_bill'=> $id])->all();
        $findadjustedbill = AdjustedBills::find()->where(['bill_id'=>$id])->all();

        $finddebitnotes = DebitNote::find()->where(['bill_no'=>$bill->bill_no])->all();


        // print_r(Yii::$app->request->post());
        // print_r($mod->getErrors());

        //  die();
        if($modl->load(Yii::$app->request->post())){
                // print_r('xx');
                // die();
           $adj_amount_check = BillsAdjustedAmounts::find()->where(['bill_id'=>$id])->count();
           if($adj_amount_check)
           {
               $adj_check = BillsAdjustedAmounts::find()->where(['bill_id'=>$id])->all();

               foreach ($adj_check as $key => $value) {

                $bill->pending_amount = $bill->pending_amount+$value->amount;
                    // $sale->converted_to ='Receipt';
                $bill->save(false);

                  $ledgerdetails=AccountingLedger::find()->where(['accounting_ledger_id'=>$value->account])->one();

                  $ledger = new AccountingDetails();
                  $ledger->ledgerhead_id = $ledgerdetails->ledger_id;
                  $ledger->transaction_type = "Payment";
                  $ledger->transaction_id =$value->id ;
                  $ledger->inc_exp = "Expense";
                  $ledger->accounting_type = "Credit";
                  $ledger->amount = $value->amount;
                  $ledger->date = $bill->date;
                  $ledger->save(false);

                  $findcustomer=Vendor::find()->where(['id'=>$bill->vendor_id])->one();
                  $ledger_details=AccountingLedger::find()->where(['ledger_id'=>$findcustomer->business_name])->one();
                  $ledger = new AccountingDetails();
                  $ledger->ledgerhead_id =$ledger_details->ledger_id ;
                  $ledger->transaction_type = "Payment";
                  $ledger->transaction_id =$value->id ;
                  $ledger->inc_exp = "Expense";
                  $ledger->accounting_type = "Debit";
                  $ledger->amount = $value->amount;
                  $ledger->date = $bill->date;
                  $ledger->save(false);
               }

                $adj_delete=BillsAdjustedAmounts::deleteAll(['bill_id'=>$id]);
           }
          $adj=Yii::$app->request->post()['Adj'];
          foreach($adj as $val){
          $adjust=new BillsAdjustedAmounts();
          $adjust->account=$val['accountDD'];
          $adjust->bill_id=$id;
          $adjust->amount=$val['item_price'];
          $adjust->notes=$val['notes'];
          $adjust->save(false);

          $bill->pending_amount = $bill->pending_amount-$adjust->amount;
                    // $sale->converted_to ='Receipt';
          $bill->save(false);

          $ledgerdetails=AccountingLedger::find()->where(['accounting_ledger_id'=>$adjust->account])->one();

          $ledger = new AccountingDetails();
          $ledger->ledgerhead_id = $ledgerdetails->ledger_id;
          $ledger->transaction_type = "Payment";
          $ledger->transaction_id =$adjust->id ;
          $ledger->inc_exp = "Expense";
          $ledger->accounting_type = "Debit";
          $ledger->amount = $adjust->amount;
          $ledger->date = $bill->date;
          $ledger->save(false);

          $findcustomer=Vendor::find()->where(['id'=>$bill->vendor_id])->one();
          $ledger_details=AccountingLedger::find()->where(['ledger_id'=>$findcustomer->business_name])->one();
          $ledger = new AccountingDetails();
          $ledger->ledgerhead_id =$ledger_details->ledger_id ;
          $ledger->transaction_type = "Payment";
          $ledger->transaction_id =$adjust->id ;
          $ledger->inc_exp = "Expense";
          $ledger->accounting_type = "Credit";
          $ledger->amount = $adjust->amount;
          $ledger->date = $bill->date;
          $ledger->save(false);



        }  
      }


        $adj_amount_of_bill = BillsAdjustedAmounts::find()->where(['bill_id'=>$id])->all();
        return $this->render('view', [
            'model' => $this->findModel($id),
            'modl'=>$modl,
            'products'=>$data,
            'documents'=>$data1,
            'qurey1'=>$qurey1,
            'findadjustedbill'=>$findadjustedbill,
            'adj_amount_of_bill'=>$adj_amount_of_bill,
            'finddebitnotes'=>$finddebitnotes,
            'from'=>$from
        ]);
    }
   
    

    /**
     * Creates a new PurchaseBill model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate($id=null,$po=null,$from=null,array $gr=null)
    {
        Yii::$app->view->params['active']=106;
        $model = new PurchaseBill();
        $datalist= new BillProducts();

        $selected_date=date('Y-m-d');
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
        $dynamic_date_last = ($year - 1) . '-' . '03' . '-' . '31';
            if( strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=PurchaseBill::find()->where(['<=', 'date', $dynamic_date])->andWhere(['>', 'date', $dynamic_date_last])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=PurchaseBill::find()->where(['>', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }

        
        if($id){
            $data = $this->findModel($id)->attributes;
            $model->setAttributes($data);
            $datalist=BillProducts::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();
            foreach($datalist as $index=>$value){
                $datalist[$index]['id']=Product::find()->where(['id'=>$value->product_id])->one()->product_name;
            }
            $from = 'bill';
        }else if($po){

            if($from=='purchaseorder')
            {
                $po_order=PurchaseOrder::find()->where(['id'=>$po])->one();
                $model->vendor_id=$po_order->vendor_name;
                $model->sender_gst_id=$po_order->sendor_gstin;
                $model->point_of_supply=$po_order->point_of_supply;
                $model->supply_type=$po_order->supply_type;
                $model->purchase_order_id = $po_order->id;
                $datalist = PurchaseOrderProduct::find()->select(['product_id','description','quantity As qty','unit_price As price','tax as tax','units As unit','tax_included As incl_tax','discount','discount_type','amount'])->where(['purchase_order_id'=>$po])->all();



                // foreach ($datalist as $key => $value) {

                //     $findreceived = ProductReceived::find()->where(['purchase_order_id'=>$po_order->id])->andWhere(['is_billed'=>0])->all();
                //     $pdt_count =0;
                //     foreach ($findreceived as $key => $val) {
                //         $recvproduct = ReceivedProductList::find()->where(['received_id'=>$val->id])->andWhere(['product_id'=>$value->product_id])->one();
                //         if($recvproduct)
                //         {
                //             $pdt_count = $pdt_count + $recvproduct->qty;
                //         }
                //         else
                //         {
                //             $pdt_count = $pdt_count;
                //         }
                //     }
                //     $value->qty = $pdt_count;
                // }
            }
            if($from=='against_gr')
            {
              // print_r('ok');
              // die();
              
                $po_order=PurchaseOrder::find()->where(['id'=>$po])->one();
                $model->vendor_id=$po_order->vendor_name;
                $model->sender_gst_id=$po_order->sendor_gstin;
                $model->point_of_supply=$po_order->point_of_supply;
                $model->supply_type=$po_order->supply_type;
                $model->purchase_order_id = $po_order->id;
               
                // print_r($model->customer_name);
                //     die();
                  
                    // $model->receipt_setting_type = $saledata->receipt_setting_type;
                    // $model->receipt_total_amount = $saledata->receipt_total_amount;
                    // $model->receipt_bank = $saledata->receipt_bank;
                    // $model->receipt_cheque_no = $saledata->receipt_cheque_no;
                    // $model->receipt_cheque_date = $saledata->receipt_cheque_date;
                    // $model->advance_note = $saledata->advance_note;
                    // $model->sale_order_id = $id;
                    // $model->receipt_id = $saledata->receipt_id;

                    $datalist = PurchaseOrderProduct::find()->select(['product_id','description','quantity As qty','unit_price As price','tax as tax','units As unit','tax_included As incl_tax','discount','discount_type'])->where(['purchase_order_id'=>$po])->all();
                    // $invoiceproducts = SaleOrderProduct::find()->where(['sale_order_no'=>$id])->all();
                    foreach ($datalist as $key => $value) {
                      $pdt_count =0;
                        for ($i=0; $i <count($gr) ; $i++) { 

                            $purchaseagainsgr = PurchaseAgainstGr::find()->where(['id'=>$gr[$i]])->andWhere(['purchase_id'=>$po])->one();
                            // print_r($po);
                            // die();
                            
                               $findreceived = ProductReceived::find()->where(['id'=>$purchaseagainsgr->gr_id])->andWhere(['is_billed'=>0])->all();
                                 

                                foreach ($findreceived as $key => $val) {
                                $recdproduct = ReceivedProductList::find()->where(['received_id'=>$val->id])->andWhere(['product_id'=>$value->product_id])->one();
                                if($recdproduct)
                                {
                                    $pdt_count = $pdt_count + $recdproduct->qty;
                                }
                                else
                                {
                                    $pdt_count = $pdt_count;
                                }
                            }
                            $value->qty = $pdt_count;
                            // $value->price = $value->price * $value->qty; 
                            if($value->discount!='')
                            {
                              if($value->discount_type=='FLAT')
                              {
                                $value->amount = ($value->price - $value->discount)*$value->qty;
                              }
                              else if($value->discount_type=='%')
                              {
                                $value->amount = ($value->price - ($value->price * ($value->discount/100)))*$value->qty;
                              }
                              else
                              {
                                $value->amount = $value->price * $value->qty;
                              }
                            }
                            if($value->incl_tax==1)
                            {
                               $tax_percent=AccountingTax::find()->select(['tax_rate'])->where(['accounting_tax_id' => $value->tax])->one()->tax_rate;
                               $value->amount = $value->amount + ($value->amount * ($tax_percent/100));
                              
                            }
                            
                            
                           
                         }
                    }
            }

            
        }
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
     
        $count=PurchaseBill::find()->count();
        if ($model->load(Yii::$app->request->post())) {

            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$model->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['purchase-bill/index']);
                }
            }
            if($from=='purchaseorder')
            {
                $poorder=PurchaseOrder::find()->where(['id'=>$po])->one();
                $findreceived = ProductReceived::find()->where(['purchase_order_id'=>$poorder->id])->andWhere(['is_billed'=>0])->all();
                foreach ($findreceived as $key => $value) {
                $value->is_billed = 1;
                $value->bill_order_id = $model->id;
                $value->save(false);
                    }
            }
            

            
            $model->pending_amount = $model->total;
            $model->subtotal=Yii::$app->request->post('subtotalamount');
            $model->cgst=Yii::$app->request->post('cgst');
            $model->sgst=Yii::$app->request->post('sgst');
            $model->igst=Yii::$app->request->post('igst');
            // $model->total=Yii::$app->request->post('totalamount');
            $model->updated_by=Yii::$app->user->identity->id;
            //$model->bill_no=PurchaseBill::find()->orderby(['id' => SORT_DESC])->one()?PurchaseBill::find()->orderby(['id' => SORT_DESC])->one()->id+1:1;
            $model->round_off=Yii::$app->request->post('roundoffval');
            // $model->tcs=$model->tcs;
            // print_r($model->tcs);
            // die();
            if($model->save(false)){ 
                if($model->purchase_order_id!='' ){
               $poorder=PurchaseBill::find()->where(['purchase_order_id'=>$po])->one();
               $findbilled = PurchaseOrder::find()->where(['id'=>$poorder->purchase_order_id])->andWhere(['billed'=>0])->all(); 
               foreach ($findbilled as $key => $value) {
                    $value->billed = 1;
                   // $value->gr_status=2;
                    $value->save(false);
               
                    }
                }
            $data=Yii::$app->request->post();
            for($i=1;$i<=$data['imagecount'];$i++){
                $document = new BillDocuments();
                $document->document_name = UploadedFile::getInstanceByName('file'.$i);
                if($document->document_name){
                $nameimage=(new \DateTime())->format('u');
                $document->document_name->saveAs('usersdocument/purchase_bill_document/' .$nameimage.'.'.$document->document_name->extension);
                $document->document_name=$nameimage. '.' . $document->document_name->extension; 
                $document->bill_id=$model->id;
                $document->save();
                }
                } 
            for($i=1;$i<=$data['productcount'];$i++){
                $val=new BillProducts();
                $val->bill_id=$model->id;
                $val->product_id=Yii::$app->request->post('product'.($i));
                $val->description=Yii::$app->request->post('desc'.($i));
                $val->qty=Yii::$app->request->post('quantity'.($i));
                $val->unit=Yii::$app->request->post('unit'.($i));
                $val->price=Yii::$app->request->post('price'.($i));
                $val->discount=Yii::$app->request->post('discount'.($i));
                $val->discount_type=Yii::$app->request->post('discounttype'.($i));
                // print_r(Yii::$app->request->post('tax'.($i)));
            // exit();
                $val->tax = Yii::$app->request->post('tax'.($i));
                
                $val->incl_tax=Yii::$app->request->post('includetax'.($i));
                $val->amount=Yii::$app->request->post('total'.($i));
                $val->save(false);
                }
            
                $ledger = new AccountingDetails();
                 $cgst = $model->cgst;
                  $igst = $model->igst;
                  $sgst = $model->sgst;
                  $roundoff = $model->round_off;
                  $totalamt = $model->total;
                  $amt = $model->subtotal;
                  $gsttype = $model->point_of_supply;
                  $tan_id = $model->id;
                  $paid_to = Vendor::find()->where(['id'=>$model->vendor_id])->one();
                  $amtto = $paid_to->business_name;
                  $tcs = $model->tcs;
                  $tan_date = $model->date;
                  // $is_tcs = $model->is_tcs;
                  $ledger->leadgerentrybill($cgst,$sgst,$igst,$roundoff,$totalamt,$gsttype,$tan_id,$amtto,$amt,$tcs,$tan_date);

                if($from=='against_gr')
                {
                    $poorder=PurchaseOrder::find()->where(['id'=>$po])->one();
                   for ($i=0; $i < count($gr); $i++) {
                      $purchase_gc = PurchaseAgainstGr::find()->where(['id'=>$gr[$i]])->andWhere(['purchase_id'=>$po])->one();
                      $purchase_gc->bill_no = $model->id;
                      $purchase_gc->convert_to = "Payment";
                      $purchase_gc->save(false);

                      $findreceived = ProductReceived::find()->where(['id'=>$purchase_gc->gr_id])->andWhere(['is_billed'=>0])->one();
                      $findreceived->is_billed = 1;
                      $findreceived->bill_order_id = $model->id;
                      $findreceived->save(false);
                    }
                } 
                if($from == 'purchaseorder')
                {
                    $purchaseagainsgr = PurchaseAgainstGr::find()->where(['purchase_id'=>$poorder->id])->all();
                    foreach ($purchaseagainsgr as $key => $val) {
                        $val->convert_to = 'Payment';
                        $val->bill_no = $model->id;
                        $val->save(false);
                    }
                    
                }
                
            if($from == ''|| $from=='bill')
              {
                $auto_po_order = new PurchaseOrder();
                $auto_po_order->vendor_name=$model->vendor_id;
                $auto_po_order->setting_type=$model->setting_type;
                $auto_po_order->sendor_gstin=$model->sender_gst_id;
                $auto_po_order->point_of_supply=$model->point_of_supply;
                $auto_po_order->supply_type=$model->supply_type;
                $auto_po_order->id=$model->purchase_order_id;
                $auto_po_order->date=$model->date;
                $auto_po_order->total_amount=$model->total;
                $auto_po_order->sub_total_amount=$model->subtotal;
                $auto_po_order->is_round_off=$model->round_off;
                $auto_po_order->igst=$model->igst;
                $auto_po_order->cgst=$model->cgst;
                $auto_po_order->sgst=$model->sgst;
                $auto_po_order->tcs=$model->tcs;
                $auto_po_order->is_tcs=$model->is_tcs;
                $auto_po_order->terms_of_payment=$model->terms_of_payment;
                $auto_po_order->pending_amount=$model->pending_amount;
                $auto_po_order->notes=$model->notes;
                $auto_po_order->billed=1;
               
                // $last_purchase=PurchaseOrder::find()->orderby(['id' => SORT_DESC])->one();
                // if($last_purchase)
                // {
                //     $auto_po_order->po_last=$last_purchase->id+1;
                // }
                // else
                // {
                //     $auto_po_order->po_last=1;
                // }
                // $year = "MOHASYN".date("Y"); 
                // $auto_po_order->prefix=$year;
                
                // $auto_po_order->po_no_final=$auto_po_order->prefix."-". $auto_po_order->po_last;
                    $selected_date=$model->date;
                    $year=date("Y"); 
                    $dynamic_date=$year.'-'.'03'.'-'.'31';
                    $dynamic_date_last = ($year - 1) . '-' . '03' . '-' . '31';
                    if( strtotime($selected_date) <= strtotime($dynamic_date))
                    {
                        $last_entry=PurchaseOrder::find()->where(['<=', 'date', $dynamic_date])->andWhere(['>=', 'date', $dynamic_date_last])->count();
                        $auto_po_order->po_last=$last_entry+1;
                    }
                    elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                        $last_entry=PurchaseOrder::find()->where(['>=', 'date', $dynamic_date])->count();
                        $auto_po_order->po_last=$last_entry+1;
                    }
                    else{
                        $auto_po_order->po_last=1;
                    }

                    if(date("m",strtotime($model->date))>=4)
                        {
                            $dt = (date('Y', strtotime('+0 year'))).'-'.(date('y', strtotime('+1 year')));
                        }
                        else
                        {
                            $dt = (date('Y', strtotime('-1 year'))).'-'.(date('y', strtotime('+0 year')));
                        }
                    $year = "MOHASYN".$dt; 
                    $auto_po_order->prefix=$year;
                    $auto_po_order->po_no_final= $year.'-'.$auto_po_order->po_last;

                    $auto_po_order->save(false);
              
                $datalist = BillProducts::find()->select(['product_id','description','qty','price','tax','unit','incl_tax','discount','discount_type'])->where(['bill_id'=>$model->id])->all();
                foreach ($datalist as $key => $value) {
                    $addproduct = new PurchaseOrderProduct();
                    $addproduct->product_id=$value->product_id;
                    $addproduct->description=$value->description;
                    $addproduct->quantity=$value->qty;
                    $addproduct->units=$value->unit;
                    $addproduct->price=$value->price*$value->qty;
                    $addproduct->unit_price=$value->price;
                    $addproduct->discount=$value->discount;
                    if($value->discount=='')
                    {
                        $value->discount = 0;
                    }
                    $addproduct->tax=$value->tax;
                    $addproduct->discount_type=$value->discount_type;
                    $addproduct->tax_included=$value->incl_tax;
                    $addproduct->amount=$value->amount;
                    $addproduct->purchase_order_id= $auto_po_order->id;
                    $addproduct->created_at=date('Y-m-d H:i:s');
                    $addproduct->updated_at=date('Y-m-d H:i:s');
                    $addproduct->save(false);
                   // print_r($datalist);die();
                }
                $model->purchase_order_id=$auto_po_order->id;
                }
		
		$pre_name="Bill";
                if(date("m",strtotime($model->date))>=4)
                {
                    $dt = (date('Y', strtotime('+0 year'))).'-'.(date('y', strtotime('+1 year')));
                }
                else
                {
                $dt = (date('Y', strtotime('-1 year'))).'-'.(date('y', strtotime('+0 year')));
                }
                $model->full_bill_name=$pre_name.'/'.$model->bill_no.'/'.$dt;

                $model->save(false);
                Yii::$app->getSession()->setFlash('success', 'Created Successfully');
                if(Yii::$app->request->post('createnew'))
                   return $this->redirect(['create']);
                return $this->redirect(['index']);

        }
    }
        return $this->render('_form', [
            'model' => $model,
            'update'=>0,
            'vendorlist'=>$vendorlist,
            'count'=>$count,
            'datalist'=>$datalist,
            'from'=>$from,
            'last_entry'=>$last_entry+1,
        ]);
    }

    /**
     * Updates an existing PurchaseBill model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=106;


        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();


        $selected_date="2022-03-21";
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if( strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=PurchaseBill::find()->where(['<=', 'date', $dynamic_date])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=PurchaseBill::find()->where(['>', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }



        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = PurchaseBill::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                return $this->redirect(['purchase-bill/index']);
            }
        }


        $model = $this->findModel($id);
        $prv_amount = $model->total;

        $from = '';
        if($model->is_deleted)
        throw new NotFoundHttpException('The requested page does not exist.');
        $count=PurchaseBill::find()->count();
        $data=BillProducts::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        $datalist=[];
        foreach($data as $index=>$value){
            $data[$index]['id']=Product::find()->where(['id'=>$value->product_id])->one()->product_name;
        }
        $product=Product::find()->all();
        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        $doc=BillDocuments::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();
        if ($model->load(Yii::$app->request->post()) ) {
            // print_r(Yii::$app->request->post());die();
            $data=Yii::$app->request->post();
            $model->pending_amount = $model->total;
            
            $pmts = PurchasePayment::find()->where(['against_bill'=>$id])->all();
            if($pmts)
            {
                foreach ($pmts as $key => $value) {
                    $model->pending_amount = $model->pending_amount - $value->total_amount;
                }
            }
            $adjpmts = AdjustedBills::find()->where(['bill_id'=>$id])->all();
            if($adjpmts)
            {
                foreach ($adjpmts as $key => $adjp) {
                    $model->pending_amount = $model->pending_amount - $adjp->amount_adjusted;
                }
            }

            $newids=[];
            for($i=1;$i<=$data['imagecount'];$i++){ 
                $newids[]=Yii::$app->request->post('file'.$i);
            }
            $oldids=ArrayHelper::map($doc, 'id', 'id');
            $diffids=array_diff($oldids,$newids);
            BillProducts::deleteAll(['bill_id'=>$id,'is_deleted'=>0]);
            // $model->po=((Bill::find()->count())+1);
            $model->subtotal=Yii::$app->request->post('subtotalamount');
            // $model->total=Yii::$app->request->post('totalamount');
            $model->cgst=Yii::$app->request->post('cgst');
            $model->sgst=Yii::$app->request->post('sgst');
            $model->igst=Yii::$app->request->post('igst');
            $model->updated_by=Yii::$app->user->identity->id;
            $model->round_off=Yii::$app->request->post('roundoffval');
            // $model->tcs=$model->tcs;


	    $pre_name="Bill";
            if(date("m",strtotime($model->date))>=4)
            {
                $dt = (date('Y', strtotime('+0 year'))).'-'.(date('y', strtotime('+1 year')));
            }
            else
            {
            $dt = (date('Y', strtotime('-1 year'))).'-'.(date('y', strtotime('+0 year')));
            }
            $model->full_bill_name=$pre_name.'/'.$model->bill_no.'/'.$dt;

            $model->save(false);


            if($model->save(false)){
                BillDocuments::deleteAll(['in','id',$diffids]);
                for($i=2;$i<=$data['imagecount']+1;$i++){ 
                if(!Yii::$app->request->post('file'.$i)){
                $document = new BillDocuments();
                $document->document_name = UploadedFile::getInstanceByName('file'.$i);
                if($document->document_name){
                $nameimage=(new \DateTime())->format('u');
                $document->document_name->saveAs('usersdocument/purchase_bill_document/' .$nameimage.'.'.$document->document_name->extension);
                $document->document_name=$nameimage. '.' . $document->document_name->extension; 
                $document->bill_id=$model->id;
                $document->save();
                }
                }
                }
            for($i=1;$i<=$data['productcount'];$i++){
                $val=new BillProducts();
                $val->bill_id=$model->id;
                $val->product_id=Yii::$app->request->post('product'.($i));
                // /print_r($val->product_id);
                $val->description=Yii::$app->request->post('desc'.($i));
                $val->qty=Yii::$app->request->post('quantity'.($i));
                $val->unit=Yii::$app->request->post('unit'.($i));
                $val->price=Yii::$app->request->post('price'.($i));
                $val->discount=Yii::$app->request->post('discount'.($i));
                $val->discount_type=Yii::$app->request->post('discounttype'.($i));

                
                // $tax = Yii::$app->request->post('tax'.($i));
                // // // print_r(Yii::$app->request->post('tax'.($i)));
                //  if(Yii::$app->request->post('tax'.($i))){
                // if(!is_string(Yii::$app->request->post('tax'.($i))))
                // $val->tax=implode(',',Yii::$app->request->post('tax'.($i)));
                // }
                //  print_r($val->tax);

                $val->tax=Yii::$app->request->post('tax'.($i));
                $val->incl_tax=Yii::$app->request->post('includetax'.($i));
                $val->amount=Yii::$app->request->post('total'.($i));
               // print_r($val->amount);
                $val->save(false);
                }
                // exit();
//                if($model->total!=$prv_amount)
//                 {
                    $find_doubleentries = AccountingDetails::find()->where(['transaction_type'=>'Bill'])->andWhere(['transaction_id'=>$model->id])->all();
                    foreach ($find_doubleentries as $key => $value) {

                          $ledger = new AccountingDetails();
                          $ledger->ledgerhead_id = $value->ledgerhead_id;
                          $ledger->transaction_type = $value->transaction_type;
                          $ledger->transaction_id =$value->transaction_id ;
                          $ledger->inc_exp = $value->inc_exp;
                          if($value->accounting_type=='Credit')
                          {
                            $ledger->accounting_type = "Debit";
                          }
                          elseif($value->accounting_type=='Debit') {
                            $ledger->accounting_type = "Credit";
                          }
                          $ledger->amount = $value->amount;
                          $ledger->date =$value->date ;
                          $ledger->edit_status = 1 ;
                          $ledger->save(false);
                          $value->edit_status = 1;
                          $value->save(false);
                          
                    }

                     $ledger = new AccountingDetails();
                     $cgst = $model->cgst;
                      $igst = $model->igst;
                      $sgst = $model->sgst;
                      $roundoff = $model->round_off;
                      $totalamt = $model->total;
                      $amt = $model->subtotal;
                      $gsttype = $model->point_of_supply;
                      $tan_id = $model->id;
                      $paid_to = Vendor::find()->where(['id'=>$model->vendor_id])->one();
                      $amtto = $paid_to->business_name;
                      $tcs = $model->tcs;
                      $tan_date = $model->date;
                      // $is_tcs = $model->is_tcs;
                      $ledger->leadgerentrybill($cgst,$sgst,$igst,$roundoff,$totalamt,$gsttype,$tan_id,$amtto,$amt,$tcs,$tan_date);
 //                 }



                Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
                if(Yii::$app->request->post('createnew'))
                return $this->redirect(['create']);
                return $this->redirect(['index']);
            }
        }
    
            return $this->render('_form', [
                'model' => $model,
                'datalist'=>$data,
                'update'=>1,
                'productlist'=>$productlist,
                'vendorlist'=>$vendorlist,
                'count'=>$count,
                'doc'=>$doc,
                'from'=>$from,
                'last_entry'=>$last_entry+1,
            ]);
        }
    /**
     * Deletes an existing PurchaseBill model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=106;
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            if($ids){
            foreach($ids as $id){
            $document=BillProducts::find()->where(['bill_id'=>$id])->all();
            $document1=BillDocuments::find()->where(['bill_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
            foreach($document1 as $value){$value->is_deleted=1;$value->save();}
            $data=$this->findModel($id);
            $data->is_deleted=1;$data->updated_by=Yii::$app->user->identity->id;$data->save();
	    $accounting = AccountingDetails::find()->where(['transaction_id'=>$id])->andWhere(['transaction_type'=>"Bill"])->all();
                    foreach ($accounting as $account) {
                        $account->edit_status = 1;
                        $account->save(false);
                    }
            }
            }
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        else
        {
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_for_purchase = PurchaseBill::find()->where(['id'=>$id])->one();
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['purchase-bill/index']);
                }
            }

            $data=$this->findModel($id);
            $data->is_deleted=1;$data->updated_by=Yii::$app->user->identity->id;$data->save();
            $document=BillProducts::find()->where(['bill_id'=>$id])->all();
            $document1=BillDocuments::find()->where(['bill_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
            foreach($document1 as $value){$value->is_deleted=1;$value->save();}

	    $accounting = AccountingDetails::find()->where(['transaction_id'=>$id])->andWhere(['transaction_type'=>"Bill"])->all();
            foreach ($accounting as $account) {
                 $account->edit_status = 1;
                 $account->save(false);
            }
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        } 
    }
    public function actionPurchasebillTemplate($dataProvider=null,$form=null,$id=null,$for=null)
    {


        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
            {
               $query=PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['is_deleted' =>0]);
            }
            else
            {
                $accessed = Yii::$app->user->identity->id;

                $vendors = UserClient::find()->where(['user_id'=>$accessed])->andWhere(['client_type'=>'Vendor'])->all();
                $vendor_ids= [];
                $i = 0;
                foreach($vendors as $data)
                {
                    $vendor_ids[$i] = $data->client_id;
                    $i++;
                }
                $query = PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['in', 'vendor_id', $vendor_ids])->andWhere(['is_deleted' =>0]);
            }
        if($form=='all')
        {
            if($dataProvider==null)
             {
                $dataProvider = new ActiveDataProvider([
                'query' => $query,
                ]);
             }
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->WriteHTML($this->renderPartial('exportpdf', [
                'dataProvider' => $dataProvider,
            ]));
            $mpdf->Output('purchase_bill.pdf', 'I');
        }
        else
        {
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_for_purchase = PurchaseBill::find()->where(['id'=>$id])->one();
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['purchase-bill/index']);
                }
            }
              $model = $this->findModel($id);
              $products=BillProducts::find()->all();
              $mpdf = new mPDF;
              $mpdf->showImageErrors = true;

              // if($for=='pdf')
              // {
                  //   $mpdf->WriteHTML($this->renderPartial('/purchase-bill/purchasebill-template', [
                  //   'model' => $model,
                  //   'products'=>$products,
                  // ]));
                  //   $mpdf->Output('purchase-bill.pdf', 'D');
              // } 
              // else
              // {
              //       return $this->renderPartial('/purchase-bill/purchasebill-template', [
              //            'model' => $model,
              //               'products'=>$products,
              //           ]);
              // }
              $mpdf->WriteHTML($this->renderPartial('/purchase-bill/purchasebill-template', [
                    'model' => $model,
                    'products'=>$products,
                ]));

              
              if($form=='print')
              {
                 $mpdf->Output('purchase-bill.pdf', 'I');
              }
              else if($form==null)
              {
                 $mpdf->Output('purchase-bill.pdf', 'D');
              }
        }
    }
    public function actionExcelExport($dataProvider=null,$detail=null)
    {
        Yii::$app->view->params['active']=106;
        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
            {
               $query=PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['is_deleted' =>0]);
            }
            else
            {
                $accessed = Yii::$app->user->identity->id;

                $vendors = UserClient::find()->where(['user_id'=>$accessed])->andWhere(['client_type'=>'Vendor'])->all();
                $vendor_ids= [];
                $i = 0;
                foreach($vendors as $data)
                {
                    $vendor_ids[$i] = $data->client_id;
                    $i++;
                }
                $query = PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['in', 'vendor_id', $vendor_ids])->andWhere(['is_deleted' =>0]);
            }



       if($dataProvider==null)
        {
            $dataProvider = new ActiveDataProvider([
            'query' => $query,
            ]);
        }
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            [
                'label'=>'Bill #',
               'attribute' => 'bill_no',
               'value' => function ($data) {
                return $data->bill_no;
               }
            ],
            [
                'label'=>'Vendor / Category',
               'attribute' => 'vendor_id',
               'value' => function ($data) {
                $val=Vendor::find()->where(['id'=>$data->vendor_id])->one();
                return $val->business_name;
               }
            ],
            'date:date',
            [
                'label'=>'Amount',
               'attribute' => 'total',
               'value' => function ($data) {
                return $data->total;
               }
            ],
            'setting_type',
            'vendor_bill_no',
            'bill_date:date',
            'sender_gst_id',
            'notes',
            'bill_type',
            'supply_type',
            'supply_attracts',
            'is_ineligible_itc',
            'point_of_supply',
            'pricelist',
            'prefix',
            'suffix',
            'due_date:date',
            'bill_notes',
            'terms_of_payment',
            'product_code',
            'subtotal',
            'total',
            [
                'label'=>'Payment',
               'attribute' => 'payment',
               'value' => function ($data) {
                switch($data->payment){
                    case '1':  return "Add Payment";
                    case '2':  return "Match Payment";
                    case '3':  return "Match Debit Note";
                    case '3':  return "Match Debit Note";
        }
                return $data->payment?'YES':'NO';
               }
            ],
            [
                'label'=>'Set Reminder',
               'attribute' => 'set_reminder',
               'value' => function ($data) {
                return $data->set_reminder?'YES':'NO';
               }
            ],
            [
                'label'=>'Show to store keeper',
               'attribute' => 'show_to_store_keeper',
               'value' => function ($data) {
                return $data->show_to_store_keeper?'YES':'NO';
               }
            ],
            'order_type',
            'terms',
            'created_at:date',
            'updated_at:date',
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    [
                        'label'=>'Bill #',
                       'attribute' => 'bill_no',
                       'value' => function ($data) {
                        return $data->bill_no;
                       }
                    ],
                    [
                        'label'=>'Vendor / Category',
                       'attribute' => 'vendor_id',
                       'value' => function ($data) {
                        $val=Vendor::find()->where(['id'=>$data->vendor_id])->one();
                        return $val->business_name;
                       }
                    ],
                    'date:date',
                    [
                        'label'=>'Amount',
                       'attribute' => 'total',
                       'value' => function ($data) {
                        return $data->total;
                       }
                    ],
                ],
            ]);
        }
    if($detail)
    return $exporter->send('purchase_bill_details.xlsx');
    return $exporter->send('purchase_bill.xlsx');
    }
    Public function actionDownloadAttachment($id)
    {
    Yii::$app->view->params['active']=106;
    $data=BillDocuments::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();
    if(!$data){
    Yii::$app->getSession()->setFlash('success', 'No Attachment Found');
    return $this->redirect(['index']);
    }
    $file = Yii::$app->basePath.'/web/usersdocument/purchase_bill_document/product_'.$id.'.zip';
    $zip = new \ZipArchive();
    $zip->open($file, \ZipArchive::CREATE | \ZipArchive::OVERWRITE) ;
    foreach($data as $value){
       $files[]=Yii::$app->basePath . '/web/usersdocument/purchase_bill_document/'.$value->document_name;
    }
    foreach($files as $val){
        $zip->addFromString(basename($val), file_get_contents($val));
    }
    $zip->close();
    return Yii::$app->response->sendFile($file);
    }
    public function actionGetVendor($id)
    {
        return Vendor::findOne($id)->gstin_no;
    }
    /**
     * Finds the PurchaseBill model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return PurchaseBill the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = PurchaseBill::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionProductdetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $product=Product::find()->limit(10)->all();
        else
        $product=Product::find()->where(["like","product_name",$searchTerm])->limit(10)->all();

        $productlist=ArrayHelper::map($product, 'id', 'product_name');
        return json_encode($productlist);
    }
     public function actionVendordetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $customer=Vendor::find()->limit(10)->all();
        else
        $customer=Vendor::find()->where(["like","business_name",$searchTerm])->limit(10)->all();

        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    public function actionFinddetails()
    { 
        $taxdetails= Accountingtax::find()->select(['tax_name','accounting_tax_id'])->where(['is_deleted'=>0])
              ->andWhere(['tax_sub_type'=>'Input'])
              ->andWhere(['status'=>1])
              ->andWhere(['or',['tax_class'=>'IGST'],['tax_class'=>'TCS']])->all();      
        $taxlist=ArrayHelper::map($taxdetails, 'accounting_tax_id','tax_name');
        return Json::encode($taxlist);
    }
    public function actionTaxdetails($item)
    {
       $taxdetails= AccountingTax::find()->where(['accounting_tax_id' =>$item])->one();
        echo Json::encode($taxdetails);
    }


    public function actionDeleteadjust($id)
    {

          $bill = PurchaseBill::find()->where(['id'=>$id])->one();
          $adj_amount_check = BillsAdjustedAmounts::find()->where(['bill_id'=>$id])->count();
           if($adj_amount_check)
           {
               $adj_check = BillsAdjustedAmounts::find()->where(['bill_id'=>$id])->all();

               foreach ($adj_check as $key => $value) {

                $bill->pending_amount = $bill->pending_amount+$value->amount;
                $bill->save(false);

                   $ledgerdetails=AccountingLedger::find()->where(['accounting_ledger_id'=>$value->account])->one();

                  $ledger = new AccountingDetails();
                  $ledger->ledgerhead_id = $ledgerdetails->ledger_id;
                  $ledger->transaction_type = "Payment";
                  $ledger->transaction_id =$value->id ;
                  $ledger->inc_exp = "Expense";
                  $ledger->accounting_type = "Credit";
                  $ledger->amount = $value->amount;
                  $ledger->date = $bill->date;
                  $ledger->save(false);

                  $findcustomer=Vendor::find()->where(['id'=>$bill->vendor_id])->one();
                  $ledger_details=AccountingLedger::find()->where(['ledger_id'=>$findcustomer->business_name])->one();
                  $ledger = new AccountingDetails();
                  $ledger->ledgerhead_id =$ledger_details->ledger_id ;
                  $ledger->transaction_type = "Payment";
                  $ledger->transaction_id =$value->id ;
                  $ledger->inc_exp = "Expense";
                  $ledger->accounting_type = "Debit";
                  $ledger->amount = $value->amount;
                  $ledger->date = $bill->date;
                  $ledger->save(false);
               }

                $adj_delete=BillsAdjustedAmounts::deleteAll(['bill_id'=>$id]);
           }


          return $this->redirect(['purchase-bill/index']);
    }


     public function actionFindbillalldetails($id)
    {
        $all= PurchaseBill::find()->where(['id' =>$id])->one();
        echo Json::encode($all);
    } 
    public function actionCreateUrl($id)
    {
       // $ids=Yii::$app->request->post('data');
        $products=BillProducts::find()->all();
        $url = Url::to(['purchasebill-template', 'id' => $id,'products'=>$products],'https');
        echo Json::encode($url);
            // print_r($url);

    }
    public function actionFinduseralldetails($id)
    {
        $all= PurchaseBill::find()->where(['id' =>$id])->one();
        $userdetails = Vendor::find()->where(['id'=>$all->vendor_id])->one();
        echo Json::encode($userdetails);
    }
    public function actionFieldpurchasebillsettingtype($setting=null,$template=null)
    {   
        //print_r(1);die();
        $find = GenericTemplate::find()->where(['template_for'=>'Expense'])->andWhere(['gt_id'=>$setting])->count();
        echo Json::encode($find);
    }
    public function actionSelectdate($selected_date=null)
    {   $selected_date=$_GET['selected_date']; 
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
        $dynamic_date_last = ($year - 1) . '-' . '03' . '-' . '31';
        $count_before_year=PurchaseBill::find()->where(['<=', 'date', $dynamic_date])->andWhere(['>', 'date', $dynamic_date_last])->count();
        $count_after_year=PurchaseBill::find()->where(['>', 'date', $dynamic_date])->count();

        if(strtotime($selected_date) <= strtotime($dynamic_date)) { 
            $last_entry=$count_before_year+1;
        }
        else if(strtotime($selected_date) > strtotime($dynamic_date)){
            $last_entry=$count_after_year+1;
        } 
        else{
            $last_entry=0;
        }
        echo $last_entry;
    }


    public function actionPrintall($id=null,$from=null)
    {

         $adjusted = new BillsAdjustedAmounts();
        $modl = new PurchaseBill();

        $data=BillProducts::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();
        $data1=BillDocuments::find()->where(['bill_id'=>$id,'is_deleted'=>0])->all();

        $bill = PurchaseBill::find()->where(['id'=>$id])->one();

        $qurey1 = PurchasePayment::find()->where(['against_bill'=> $id])->all();
        $findadjustedbill = AdjustedBills::find()->where(['bill_id'=>$id])->all();

        $finddebitnotes = DebitNote::find()->where(['bill_no'=>$bill->bill_no])->all();

        if($from=='pdf')
        {



    //           $mPDF1 = new mPDF;
    // $mPDF1->WriteHTML($this->render('printall',['model' => $this->findModel($id),
    //                     'products'=>$data,
    //                     'from'=>$from]true));

              $mpdf = new mPDF;
              $mpdf->showImageErrors = true;

                    $mpdf->WriteHTML($this->renderPartial('printall', [
                     'model' => $this->findModel($id),
                        'products'=>$data,
                        'from'=>$from,
                  ]));
                    $mpdf->Output('purchase-bill.pdf', 'D');


         
        }
        else
        {
             return $this->renderPartial('printall', [
            'model' => $this->findModel($id),
            'products'=>$data,
            'from'=>$from,
           ]);
        }
       
        
    }



   

}
