<?php

namespace backend\modules\users\controllers;

use Yii;
use backend\modules\users\models\PurchasePayment;
use backend\modules\users\models\PaymentMode;
use backend\modules\users\models\PurchaseBill;
use backend\modules\users\models\PurchasePaymentDocument;
use backend\modules\users\models\PurchasePaymentSearch;
use backend\modules\users\models\AccountingDetails;
use backend\modules\users\models\AdjustedBills;
use backend\modules\master\models\Vendor;
use backend\modules\users\models\DirectPayment;
use backend\modules\master\models\AccountingLedger;
use backend\modules\master\models\PurchaseOrder;
use backend\modules\master\models\PurchaseAgainstGr;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use Mpdf\Mpdf;
use yii\data\ActiveDataProvider;
use yii2tech\spreadsheet\Spreadsheet;
use yii\web\UploadedFile;
use yii\helpers\Url;
use yii\helpers\Json;
use backend\models\UserClient;
use backend\models\AuthAssignment;
use backend\modules\master\models\PurchaseTemplates;
use backend\modules\master\models\GenericTemplate;
/**
 * PurchasePaymentController implements the CRUD actions for PurchasePayment model.
 */
class PurchasePaymentController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all PurchasePayment models.
     * @return mixed
     */
    public function actionIndex($totalexcel=null,$excel=null,$print=null)
    {
        Yii::$app->view->params['active']=109;
        $searchModel = new PurchasePaymentSearch();

        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        }
        else
        {
            $accessed[] = Yii::$app->user->identity->id;
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams,$accessed);
        }

        // $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $model=new PurchasePayment();
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        $paymentmode=PaymentMode::find()->all();
        $paymentmodelist=ArrayHelper::map($paymentmode, 'payment_mode_name', 'payment_mode_name');
        if ($model->load(Yii::$app->request->get()) ) {

            if($user_type->item_name == 'Admin')
            {
                $query=PurchasePayment::find()->where(['is_deleted'=>0]);
            }
            else
            {
                $accessed = Yii::$app->user->identity->id;
                $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
                $customer_ids = [];
                $j = 0;
                foreach($customers as $data)
                {
                    $customer_ids[$j] = $data->client_id;
                    $j++;
                }
                $query = PurchasePayment::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
            }

            
            $query->andFilterWhere(['between', 'date', $model->from_date,$model->to_date]);
            $query->andFilterWhere(['=', 'vendor_id', $model->vendor])
            ->andFilterWhere(['like', 'prefix', $model->prefix])
            ->andFilterWhere(['like', 'suffix', $model->suffix])
            ->andFilterWhere(['like', 'payment_no', $model->payment_no])
            ->andFilterWhere(['like', 'payment_mode', $model->bank])
            ->andFilterWhere(['like', 'notes', $model->notes])
            ->andFilterWhere(['like', 'cheque_no', $model->cheque_no])
            ->andFilterWhere(['like', 'payment_type', $model->account_category])
            ->andFilterWhere(['like', 'centre_category', $model->centre_category])
            ->andFilterWhere(['like', 'payment_type', $model->payment_type])
            ->andFilterWhere(['like', 'setting_type', $model->settingtype]);
            if($model->amountfrom||$model->amount_from||$model->amount_to||$model->amountto){
            $query->andFilterWhere([
                'and',[$model->amount_from, 'amount', $model->amountfrom],
                [$model->amount_to, 'amount', $model->amountto]
            ]);
            }
            $dataProvider = new ActiveDataProvider([
                'query' => $query,
                'sort'=> ['defaultOrder' => ['id' => SORT_DESC]],
            ]);
        }
        if($totalexcel!=null)
        { 
            $details = true;
            $this->actionExcelExport($dataProvider,$details);
        }
        if($excel!=null)
        { 
            $this->actionExcelExport($dataProvider);
        }
        if($print!=null)
        { 
            $from = "all";
            // print_r($from); die();
            $this->actionPurchasepaymentTemplate($dataProvider,$from);
        }


        return $this->render('index', [
            'model'=>$model,
            'vendorlist'=>$vendorlist,
            'dataProvider' => $dataProvider,
            'searchModel'=>$searchModel,
            'bank'=>$paymentmodelist,
        ]);
    }

    /**
     * Displays a single PurchasePayment model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id,$from=null)
    {
        Yii::$app->view->params['active']=109;

        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = PurchasePayment::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'You are not allowed to perform this action.');
                return $this->redirect(['purchase-payment/index']);
            }
        }
        $data1=PurchasePaymentDocument::find()->where(['payment_id'=>$id,'is_deleted'=>0])->all();
        return $this->render('view', [
            'model' => $this->findModel($id),
            'documents'=>$data1,
            'from'=>$from,
        ]);
    }

    /**
     * Creates a new PurchasePayment model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate($id=null,$from=null)
    {
        Yii::$app->view->params['active']=109;
        $model = new PurchasePayment();
        $total_amount_adjusted = 0;
        $amount_added_to_opening = 0;

        $selected_date=date('Y-m-d');
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if( strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=PurchasePayment::find()->where(['<=', 'date', $dynamic_date])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=PurchasePayment::find()->where(['>=', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }

       if($from=="from-bill")
       {
            if($id){
            $bill=PurchaseBill::find()->where(['id'=>$id])->one();
            if($bill->pending_amount>0)
            {
                $paymentagenstbill = PurchasePayment::find()->where(['against_bill'=>$bill->bill_no])->all();
                $amountagenstbill = 0;
                foreach ($paymentagenstbill as $key => $value) {
                  $amountagenstbill = $amountagenstbill + $value->amount;
                }
                $model->payment_type="Payment against bill";
                $model->total_bill_amount = $bill->total;
                $model->balance_amount = $bill->pending_amount;
                $model->vendor_id=$bill->vendor_id;
                $model->against_bill = $bill->id;
                $from = "from-bill";
                // print_r($model->vendor_id);
            }
            else
            {
                Yii::$app->getSession()->setFlash('success', 'Full amount for this bill is already paid.');
                             return $this->redirect(['purchase-bill/index']);
            }
          }
        }
        else if($from == "against_gr")
        {   
                if($id){
                $against = PurchaseAgainstGr::find()->where(['id'=>$id])->one();    
                $bill=PurchaseBill::find()->where(['id'=>$against->bill_no])->one();
                if($bill->pending_amount>0)
                {
                    $paymentagenstbill = PurchasePayment::find()->where(['against_bill'=>$bill->bill_no])->all();
                    $amountagenstbill = 0;
                    foreach ($paymentagenstbill as $key => $value) {
                      $amountagenstbill = $amountagenstbill + $value->amount;
                    }
                    $model->payment_type="Payment against bill";
                    $model->total_bill_amount = $bill->total;
                    $model->balance_amount = $bill->pending_amount;
                    $model->vendor_id=$bill->vendor_id;
                    $model->against_bill = $bill->id;
                    $from = "against_gr";
                    // print_r($model->vendor_id);
                }
                else
                {
                    // print_r('w');
                    // die();
                    Yii::$app->getSession()->setFlash('success', 'Full amount for this is already paid.');
                                 return $this->redirect(['../master/purchase-order/purchase-pending']);
                }
              }
        }
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        $paymentmode=PaymentMode::find()->all();
        $paymentmodelist=ArrayHelper::map($paymentmode, 'payment_mode_name', 'payment_mode_name');
       // $count=PurchasePayment::find()->count();
      
        if ($model->load(Yii::$app->request->post())) {


        if($model->vendor_id!='')
        {
            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$model->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'Access Denied.');
                    return $this->redirect(['purchase-payment/index']);
                }
            }
        }    


        $model->updated_by=Yii::$app->user->identity->id;
        if($id)
        {
                $model->against_bill = $id;
        }
            // if(PurchasePayment::find()->orderby(['id' => SORT_DESC])->one())
            // $model->payment_no=PurchasePayment::find()->orderby(['id' => SORT_DESC])->one()->id+1;
            // else
            // $model->payment_no=1;

            if($model->save(false)){
                if(Yii::$app->request->post('Bill')){
                    $billdata=Yii::$app->request->post()['Bill'];
                    foreach($billdata as $value){
                        if($value['ledger_os_id']){
                        $bill=PurchaseBill::find()->where(['id'=>$value['ledger_os_id']])->one();

                        if($bill->pending_amount-($value['totamount'] + $value['tds_amount'])>=0)
                        {
                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            $purchase_order->pending_amount = $purchase_order->pending_amount - ($value['totamount'] + $value['tds_amount']);
                            $purchase_order->save(false);
                        }
                             
                        $bill->pending_amount=$bill->pending_amount-($value['totamount'] + $value['tds_amount']);
                        
                    }
                        else{
                            $bill->extra_paid_amount=($value['totamount'] + $value['tds_amount'])-$bill->pending_amount;
                            $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                            if($purchase_order)
                            {
                                $purchase_order->pending_amount = $purchase_order->pending_amount - $bill->pending_amount;
                                $purchase_order->save(false);
                            }
                            
                            $bill->pending_amount=0;
                            
                            
                            
                        }
                        $bill->save(false);
                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            if($purchase_order->pending_amount==0)
                            {
                                $purchase_order->gr_status=2;
                                 $purchase_order->save(false);
                            }
                            if($bill->pending_amount<=0)
                            {
                                 $all_grs = PurchaseAgainstGr::find()->where(['bill_no'=>$value['ledger_os_id']])->all(); 
                                 foreach ($all_grs as $key => $grs) {
                                   $grs->convert_to = "Completed";
                                   $grs->save(false);
                                 }
                            }
                        }
                        
                    }else{
                       $vendorind=Vendor::findOne($model->vendor_id);
                       $vendorind->opening_as_on=$vendorind->opening_as_on-($value['totamount']+ $value['tds_amount']);
                       $vendorind->save(false);
                    }
                    
                        if($value['totamount']){
                        $adjust=new AdjustedBills();
                        $adjust->payment_id=$model->id;
                        $adjust->vendor_id=$model->vendor_id;
                        if($value['ledger_os_id'])
                        {

                            $adjust->type="bill";
                            if($value['ledger_os_id'])
                            {
                                $find_inv_data = PurchaseBill::find()->where(['id'=>$value['ledger_os_id']])->one();
                                $req_invoice_amount = $find_inv_data->total;
                                if($find_inv_data->point_of_supply == '21')
                                {
                                    $total_tax = $find_inv_data->cgst + $find_inv_data->sgst;
                                }
                                else
                                {
                                    $total_tax = $find_inv_data->igst;
                                }
                                $gst_percentage = ((($total_tax/$req_invoice_amount))*100);
                                $receipt_gst_amount = $value['totamount']*($gst_percentage/100);
                                if($find_inv_data->point_of_supply == '21')
                                {
                                    $cgst_tax = $receipt_gst_amount/2;
                                    $sgst_tax = $receipt_gst_amount/2;
                                    $adjust->cgst_amount=$cgst_tax;
                                    $adjust->sgst_amount=$sgst_tax;
                                }
                                else
                                {
                                    $adjust->igst_amount=$receipt_gst_amount;
                                }

                                $receipt_taxable_amount =  $value['totamount'] - $receipt_gst_amount;
                                if($find_inv_data->is_tcs==1)
                                {
                                    $receipt_tds_amount = $value['totamount']*(1/100);
                                    $adjust->tcs_amount=$receipt_tds_amount;
                                    $receipt_taxable_amount =  $receipt_taxable_amount - $receipt_tds_amount;
                                }
                                $adjust->taxable_amount = $receipt_taxable_amount;
                            }
                        }
                        else
                        $adjust->type="opening";
                        $adjust->bill_id=$value['ledger_os_id'];
                        $adjust->tds_amount=$value['tds_amount'];
                        $adjust->notes=$value['notes'];
                        $adjust->amount_adjusted=$value['totamount'];

                        $total_amount_adjusted = $total_amount_adjusted + $adjust->amount_adjusted;

                        $adjust->save(false);
                    }
                    }


                    if($total_amount_adjusted < $model->amount)
                    {
                        $vendorinds=Vendor::findOne($model->vendor_id);
                        // print_r($vendorinds->opening_as_on);
                        // die();
                        if($vendorinds->opening_as_on=='')
                        {
                            $vendorinds->opening_as_on = 0;
                        }
                        // print_r($vendorinds->opening_as_on);
                        // die();
                        // $vendorinds->opening_as_on = $vendorinds->opening_as_on - ($prv_model_amount - $prv_adjusted_bill_amt);

                        $vendorinds->opening_as_on=$vendorinds->opening_as_on+($model->amount-$total_amount_adjusted);
                        $vendorinds->save(false);
                    }   

                }

                if($from=='against_gr')
                {
                    $against = PurchaseAgainstGr::find()->where(['id'=>$id])->one(); 
                    $all_grs = PurchaseAgainstGr::find()->where(['bill_no'=>$against->bill_no])->all();   
                    $bill=PurchaseBill::find()->where(['id'=>$against->bill_no])->one();

                    if(($bill->pending_amount-($model->amount+$model->tds_amount))>=0)
                    {
                        $bill->pending_amount = $bill->pending_amount-($model->amount+$model->tds_amount);
                        $bill->save(false);

                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            $purchase_order->pending_amount = $purchase_order->pending_amount - ($model->amount+$model->tds_amount);
                            $purchase_order->save(false);   
                        }
                        
                    }
                    else{

                        
                            $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                            if($purchase_order)
                            {
                                $purchase_order->pending_amount = $purchase_order->pending_amount - $bill->pending_amount;
                                $purchase_order->save(false);
                            }
                            
                            $bill->extra_paid_amount=($model->amount+$model->tds_amount)-$bill->pending_amount;
                            $bill->pending_amount=0;
                            
                            $bill->save(false);

                    }
                    $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                    if($purchase_order)
                    {
                        if($purchase_order->pending_amount==0)
                        {
                            $purchase_order->gr_status=2;
                             $purchase_order->save(false);
                        }
                    }

                    if($bill->pending_amount<=0)
                    {
                         foreach ($all_grs as $key => $value) {
                           $value->convert_to = "Completed";
                           $value->save(false);
                         }
                    }



                    $find_inv_data = PurchaseBill::find()->where(['id'=>$against->bill_no])->one();
                    $req_invoice_amount = $find_inv_data->total;
                    if($find_inv_data->point_of_supply == '21')
                    {
                        $total_tax = $find_inv_data->cgst + $find_inv_data->sgst;
                    }
                    else
                    {
                        $total_tax = $find_inv_data->igst;
                    }
                    $gst_percentage = ((($total_tax/$req_invoice_amount))*100);
                    $receipt_gst_amount = $model->total_amount*($gst_percentage/100);
                    if($find_inv_data->point_of_supply == '21')
                    {
                        $cgst_tax = $receipt_gst_amount/2;
                        $sgst_tax = $receipt_gst_amount/2;
                        $model->cgst_amount=$cgst_tax;
                        $model->sgst_amount=$sgst_tax;
                    }
                    else
                    {
                        $model->igst_amount=$receipt_gst_amount;
                    }

                    $receipt_taxable_amount =  $model->total_amount - $receipt_gst_amount;
                    if($find_inv_data->is_tcs==1)
                    {
                        $receipt_tds_amount = $model->total_amount*(1/100);
                        $model->tcs_amount=$receipt_tds_amount;
                        $receipt_taxable_amount =  $receipt_taxable_amount - $receipt_tds_amount;
                    }
                    $model->taxable_amount = $receipt_taxable_amount;
                    $model->save(false);



                    

                }


                 if($from=='from-bill')
                {
                    $bill=PurchaseBill::find()->where(['id'=>$id])->one();

                    if(($bill->pending_amount-($model->amount+$model->tds_amount))>=0)
                    {
                        $bill->pending_amount = $bill->pending_amount-($model->amount+$model->tds_amount);
                        $bill->save(false);

                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            $purchase_order->pending_amount = $purchase_order->pending_amount - ($model->amount+$model->tds_amount);
                            $purchase_order->save(false);   
                        }
                        
                    }
                    else{

                        
                            $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                            if($purchase_order)
                            {
                                $purchase_order->pending_amount = $purchase_order->pending_amount - $bill->pending_amount;
                                $purchase_order->save(false);
                            }
                            
                            $bill->extra_paid_amount=($model->amount+$model->tds_amount)-$bill->pending_amount;
                            $bill->pending_amount=0;
                            
                            $bill->save(false);

                    }
                    $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                    if($purchase_order)
                    {
                        if($purchase_order->pending_amount==0)
                        {
                            $purchase_order->gr_status=2;
                             $purchase_order->save(false);
                        }
                    }
                    if($bill->pending_amount<=0)
                    {
                        $all_grs = PurchaseAgainstGr::find()->where(['bill_no'=>$id])->all();
                        foreach ($all_grs as $key => $value) {
                            $value->convert_to = "Completed";
                            $value->save(false);
                            # code...
                        }
                    }
                    


                    $find_inv_data = PurchaseBill::find()->where(['id'=>$id])->one();
                    $req_invoice_amount = $find_inv_data->total;
                    if($find_inv_data->point_of_supply == '21')
                    {
                        $total_tax = $find_inv_data->cgst + $find_inv_data->sgst;
                    }
                    else
                    {
                        $total_tax = $find_inv_data->igst;
                    }
                    $gst_percentage = ((($total_tax/$req_invoice_amount))*100);
                    $receipt_gst_amount = $model->total_amount*($gst_percentage/100);
                    if($find_inv_data->point_of_supply == '21')
                    {
                        $cgst_tax = $receipt_gst_amount/2;
                        $sgst_tax = $receipt_gst_amount/2;
                        $model->cgst_amount=$cgst_tax;
                        $model->sgst_amount=$sgst_tax;
                    }
                    else
                    {
                        $model->igst_amount=$receipt_gst_amount;
                    }

                    $receipt_taxable_amount =  $model->total_amount - $receipt_gst_amount;
                    if($find_inv_data->is_tcs==1)
                    {
                        $receipt_tds_amount = $model->total_amount*(1/100);
                        $model->tcs_amount=$receipt_tds_amount;
                        $receipt_taxable_amount =  $receipt_taxable_amount - $receipt_tds_amount;
                    }
                    $model->taxable_amount = $receipt_taxable_amount;
                    $model->save(false);
                    

                }
               
                   
                 if($model->payment_type=="Direct payment"){
                    $model->tds_amount=Yii::$app->request->post('Invoice')['taxVal']; 
                    $model->amount= Yii::$app->request->post('Invoice')['sumTotal']; 
                    $model->total_amount= Yii::$app->request->post('Invoice')['sumTotal'];
                    $model->save(false);
                    $ledger=Yii::$app->request->post()['Products'];
                    foreach($ledger as $val){
                    $adjust=new DirectPayment();
                    $adjust->ledger_id=$val['accountDD'];
                    $adjust->payment_id=$model->id;
                   
                    if($val['item_price']=='')
                    {
                        $adjust->price='0.00';
                    }
                    else
                    {
                        $adjust->price=$val['item_price'];
                    }


                    if($val['tax_amount']=='')
                    {
                        $adjust->tds_amount='0.00';
                    }
                    else
                    {
                        $adjust->tds_amount=$val['tax_amount'];
                    }
                    
                    $adjust->total=$val['total'];
                    $adjust->save(false);
                    }

                }

               
            $data=Yii::$app->request->post();
            for($i=1;$i<=$data['imagecount'];$i++){
                $document = new PurchasePaymentDocument();
                $document->document_name = UploadedFile::getInstanceByName('file'.$i);
                if($document->document_name){
                $nameimage=(new \DateTime())->format('u');
                $document->document_name->saveAs('usersdocument/purchase_payment/' .$nameimage.'.'.$document->document_name->extension);
                $document->document_name=$nameimage. '.' . $document->document_name->extension; 
                $document->payment_id=$model->id;
                $document->save();
                }
                } 


                
                if($model->payment_type=="Direct payment"){
                    $ledgerind=Yii::$app->request->post()['Products'];
                    foreach($ledgerind as $val){
                    $accounting=AccountingLedger::findOne($val['accountDD']);
                    $ledger = new AccountingDetails();
                    $ledger->ledgerhead_id = $accounting->ledger_id;
                    $ledger->transaction_type = "Payment";
                    $ledger->transaction_id =$model->id ;
                    $ledger->inc_exp = "Expense";
                    $ledger->accounting_type = "Debit";
                    if($val['item_price']=='')
                    {
                        $ledger->amount = '0.00';
                    }
                    else
                    {
                        $ledger->amount = $val['item_price'];   
                    } 
                    $ledger->date =$model->date;
                    $ledger->save(false);
                    if($val['tax_amount']){
                    $ledger = new AccountingDetails();
                    $ledger->ledgerhead_id = "TDS Amount";
                    $ledger->transaction_type = "Receipt";
                    $ledger->transaction_id =$model->id;
                    $ledger->inc_exp = "Expense";
                    $ledger->accounting_type = "Debit";
                    $ledger->amount = $val['tax_amount'];
                    $ledger->date =$model->date;
                    $ledger->save(false);
                    }
                    }
                    $ledger = new AccountingDetails();
                    $ledger->ledgerhead_id = $model->payment_mode;
                    $ledger->transaction_type = "Payment";
                    $ledger->transaction_id =$model->id ;
                    $ledger->inc_exp = "Expense";
                    $ledger->accounting_type = "Credit";
                    $ledger->amount = $model->total_amount;
                    $ledger->date =$model->date;
                    $ledger->save(false);
                }else
                {
                    $ledger = new AccountingDetails();
                    $amt = $model->amount;
                    $tds_amount = $model->tds_amount;
                    $final_amt = $model->total_amount;
                    $tan_id = $model->id;
                    $paid_to = Vendor::find()->where(['id'=>$model->vendor_id])->one();
                    $amtto = $paid_to->business_name;
                    $payment_mode = $model->payment_mode;
                    $tan_date =$model->date;
                   $ledger->leadgerentrypayment($amt,$tds_amount,$final_amt,$tan_id,$amtto,$payment_mode,$tan_date);
                }

                Yii::$app->getSession()->setFlash('success', 'Created Successfully');
                if(Yii::$app->request->post('createnew'))
                return $this->redirect(['create']);
                return $this->redirect(['index']);
        }
    }
        return $this->render('_form', [
            'model' => $model,
            'vendorlist'=>$vendorlist,
            'paymentmode'=>$paymentmodelist,
            'update'=>0,
            'from'=>$from,
            'last_entry'=>$last_entry+1,
        ]);
    }
    public function actionPurchasepaymentTemplate($dataProvider=null,$from=null)
    {

        // $check_id = Yii::$app->user->identity->id;
        // $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        // if($user_type->item_name == 'Admin')
        // {
        //     $query=PurchasePayment::find()->where(['is_deleted'=>0]);
        // }
        // else
        // {
        //     $accessed = Yii::$app->user->identity->id;
        //     $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
        //     $customer_ids = [];
        //     $j = 0;
        //     foreach($customers as $data)
        //     {
        //         $customer_ids[$j] = $data->client_id;
        //         $j++;
        //     }
        //     $query = PurchasePayment::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
        // }

        if($from=='all')
        {
            // $dataProvider = new ActiveDataProvider([
            // 'query' => $query,
            // ]);
            $dataProvider->pagination = false;
            $mpdf = new mPDF;
            $mpdf->WriteHTML($this->renderPartial('exportpdf', [
                'dataProvider' => $dataProvider,

            ]));
            $mpdf->Output('purchase_payment.pdf', 'I');
        }
        else
        {

            $loged_by = Yii::$app->user->identity->id; 
            $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();

            if($user_type->item_name != 'Admin')
            {
                $check_for_purchase = PurchasePayment::find()->where(['id'=>$id])->one();
                $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
                if($check_vendor == '')
                {
                    Yii::$app->getSession()->setFlash('error', 'You are not allowed to perform this action.');
                    return $this->redirect(['purchase-payment/index']);
                }
            }
              $model = $this->findModel($id);
            //$accountids=JournalDebitCredit::find()->all();
              $mpdf = new mPDF;
              $mpdf->showImageErrors = true;
              $mpdf->WriteHTML($this->renderPartial('/purchase-payment/purchasepayment-template', [
                    'model' => $model,
                ]));
              $mpdf->Output('purchase-payment.pdf', 'I');
        } 
    }

    /**
     * Updates an existing PurchasePayment model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        Yii::$app->view->params['active']=109;
        $model = $this->findModel($id);
        $selected_date=$model->date;
        $prv_model_amount = $model->amount;
        $extra_amount_paid_by_customer = 0;
        $prv_adjusted_bill_amt = 0;
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
            if( strtotime($selected_date) <= strtotime($dynamic_date))
            {
                $last_entry=PurchasePayment::find()->where(['<=', 'date', $dynamic_date])->count();
            }
            elseif(strtotime($selected_date) > strtotime($dynamic_date)){
                $last_entry=PurchasePayment::find()->where(['>=', 'date', $dynamic_date])->count();
            }
            else{
                $last_entry=0;
            }

        $loged_by = Yii::$app->user->identity->id; 
        $user_type = AuthAssignment::find()->where(['user_id'=>$loged_by])->one();




    //       $vendor_id=Yii::$app->request->post('data');
    //     $bill="SELECT * FROM `purchase_bill` WHERE pending_amount>0 and vendor_id=$vendor_id";
    //     $bills=Yii::$app->db->createCommand($bill)->queryAll();
    //     $vendor=Vendor::find()->where(['id'=>$vendor_id])->asArray()->one();
    //     $other =json_encode(['bills'=>$bills,
    //     'vendor'=>$vendor]
    // );




        if($user_type->item_name != 'Admin')
        {
            $check_for_purchase = PurchasePayment::find()->where(['id'=>$id])->one();
            $check_vendor = UserClient::find()->where(['user_id'=>$loged_by])->andWhere(['client_id'=>$check_for_purchase->vendor_id])->andWhere(['client_type'=>'Vendor'])->one(); 
            if($check_vendor == '')
            {
                Yii::$app->getSession()->setFlash('error', 'You are not allowed to perform this action.');
                return $this->redirect(['purchase-payment/index']);
            }
        }


        $from='';
        if($model->is_deleted)
        throw new NotFoundHttpException('The requested page does not exist.');
        $count=PurchasePayment::find()->count();
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        $paymentmode=PaymentMode::find()->all();
        $paymentmodelist=ArrayHelper::map($paymentmode, 'payment_mode_name', 'payment_mode_name');
        $doc=PurchasePaymentDocument::find()->where(['payment_id'=>$id,'is_deleted'=>0])->all();


        if ($model->load(Yii::$app->request->post()) ) {
            if($model->payment_type=="Direct payment"){
                DirectPayment::deleteAll(['payment_id'=>$model->id]);
                AdjustedBills::deleteAll(['payment_id'=>$model->id]);
                $model->tds_amount=Yii::$app->request->post('Invoice')['taxVal']; 
                $model->amount= Yii::$app->request->post('Invoice')['sumTotal']; 
                $model->total_amount= Yii::$app->request->post('Invoice')['sumTotal'];
                $model->save(false);
                $ledger=Yii::$app->request->post()['Products'];
                foreach($ledger as $val){
                $adjust=new DirectPayment();
                $adjust->ledger_id=$val['accountDD'];
                $adjust->payment_id=$model->id;
                
                if($val['item_price']=='')
                {
                    $adjust->price='0.00';
                }
                else
                {
                    $adjust->price=$val['item_price'];
                }

                if($val['tax_amount']=='')
                {
                    $adjust->tds_amount='0.00';
                }
                else
                {
                    $adjust->tds_amount=$val['tax_amount'];
                }
                $adjust->total=$val['total'];
                $adjust->save(false);
                }

            }
            if($model->against_bill !='' && $model->total_bill_amount!= '')
              {
                  $findbill = PurchaseBill::find()->where(['id'=>$model->against_bill])->one();
                  $findpayment = PurchasePayment::find()->where(['id'=>$model->id])->one();
                      $findbill->pending_amount = $findbill->pending_amount + $findpayment->amount;
                      $findbill->save(false);
                      $findpurchase = PurchaseOrder::find()->where(['id'=>$findbill->purchase_order_id])->one();
                      if($findpurchase)
                      {
                        $findpurchase->pending_amount = $findpurchase->pending_amount + $findpayment->amount;
                        $findpurchase->save(false);
                      }
                      
              }
            $alltransaction=AccountingDetails::find()->where(['transaction_id'=>$model->id,'edit_status'=>0])->all();
            foreach($alltransaction as $value){
                $accountind=new AccountingDetails();
                // $accountind->setAttributes($value);
                if($value->accounting_type=='Debit')
                $accountind->accounting_type='Credit';
                else
                $accountind->accounting_type='Debit';
                $accountind->amount=$value->amount;
                $accountind->ledgerhead_id = $value->ledgerhead_id;
                $accountind->transaction_type = $value->transaction_type;
                $accountind->transaction_id =$value->transaction_id ;
                $accountind->inc_exp = $value->inc_exp;
                $accountind->edit_status=1;
                $accountind->date = $value->date;
                $accountind->save(false);
                $value->edit_status=1;
                $value->save();
            }
            if($model->payment_type=="Direct payment"){
                $ledgerind=Yii::$app->request->post()['Products'];
                foreach($ledgerind as $val){
                $accounting=AccountingLedger::findOne($val['accountDD']);
                $ledger = new AccountingDetails();
                $ledger->ledgerhead_id = $accounting->ledger_id;
                $ledger->transaction_type = "Payment";
                $ledger->transaction_id =$model->id ;
                $ledger->inc_exp = "Income";
                $ledger->accounting_type = "Debit";
                
                if($val['item_price']=='')
                {
                    $ledger->amount = '0.00';
                }
                else
                {
                    $ledger->amount = $val['item_price'];
                }
                $ledger->date = $model->date;
                $ledger->save(false);
                if($val['tax_amount']){
                $ledger = new AccountingDetails();
                $ledger->ledgerhead_id = "TDS Amount";
                $ledger->transaction_type = "Receipt";
                $ledger->transaction_id =$model->id ;
                $ledger->inc_exp = "Income";
                $ledger->accounting_type = "Debit";
                $ledger->amount = $val['tax_amount'];
                $ledger->date = $model->date;
                $ledger->save(false);
                }
                }
                $ledger = new AccountingDetails();
                $ledger->ledgerhead_id = $model->payment_mode;
                $ledger->transaction_type = "Payment";
                $ledger->transaction_id =$model->id ;
                $ledger->inc_exp = "Income";
                $ledger->accounting_type = "Credit";
                $ledger->amount = $model->total_amount;
                $ledger->date = $model->date;
                $ledger->save(false);
            }
            if(Yii::$app->request->post('Bill')){
                DirectPayment::deleteAll(['payment_id'=>$model->id]);
                $adjusted=AdjustedBills::find()->where(['payment_id'=>$model->id,'type'=>'bill'])->all();
                foreach($adjusted as $value){
                    $adjbill=PurchaseBill::find()->where(['id'=>$value->bill_id])->one();
                    $purchaseodr = PurchaseOrder::find()->where(['id'=>$adjbill->purchase_order_id])->one();
                    if($purchaseodr) 
                    {
                        $purchaseodr->pending_amount =$purchaseodr->pending_amount + $value->amount_adjusted;
                        $purchaseodr->save(false);
                    }
                    
                    $adjbill->pending_amount=(($adjbill->pending_amount)+($value->amount_adjusted));

                    $adjbill->tds_amount=(($adjbill->tds_amount)+($value->tds_amount));
                    $adjbill->save(false);
                    $prv_adjusted_bill_amt = $prv_adjusted_bill_amt + $value->amount_adjusted;
                    $value->delete();
                }
                $billdata=Yii::$app->request->post()['Bill'];

                foreach($billdata as $value){
                    if($value['ledger_os_id']){
                    $bill=PurchaseBill::find()->where(['id'=>$value['ledger_os_id']])->one();
                    if($bill->pending_amount-($value['totamount'])>=0)
                    {
                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            $purchase_order->pending_amount = $purchase_order->pending_amount - $value['totamount'];
                            $purchase_order->save(false); 
                        }
                        
                        $bill->pending_amount=($bill->pending_amount-$value['totamount']); 
                        
                    }
                    else{
                        $bill->extra_paid_amount=(($value['totamount'])-($bill->pending_amount));
                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            $purchase_order->pending_amount = $purchase_order->pending_amount - $bill->pending_amount;
                            $purchase_order->save(false); 
                               
                        }
                        
                        $bill->pending_amount=0;     
                    }
                    $bill->save(false);
                }else{
                    $adjustedopening=AdjustedBills::find()->where(['payment_id'=>$model->id,'type'=>'opening'])->one();
                    if($adjustedopening)
                    {
                        $vendorind=Vendor::findOne($model->vendor_id);
                        $vendorind->opening_as_on=($vendorind->opening_as_on+$adjustedopening->amount_adjusted)-($value['totamount']);
                        $vendorind->save(false);
                        $adjustedopening->delete();
                    }
                 }
                    if($value['totamount']||$value['tds_amount']){
                    $adjust=new AdjustedBills();
                    $adjust->vendor_id=$model->vendor_id;
                    $adjust->payment_id=$model->id;
                    $adjust->bill_id=$value['ledger_os_id'];
                    if($value['ledger_os_id'])
                    {
                         $adjust->type="bill";
                         if($value['ledger_os_id'])
                            {
                                $find_inv_data = PurchaseBill::find()->where(['id'=>$value['ledger_os_id']])->one();
                                $req_invoice_amount = $find_inv_data->total;
                                if($find_inv_data->point_of_supply == '21')
                                {
                                    $total_tax = $find_inv_data->cgst + $find_inv_data->sgst;
                                }
                                else
                                {
                                    $total_tax = $find_inv_data->igst;
                                }
                                $gst_percentage = ((($total_tax/$req_invoice_amount))*100);
                                $receipt_gst_amount = $value['totamount']*($gst_percentage/100);
                                if($find_inv_data->point_of_supply == '21')
                                {
                                    $cgst_tax = $receipt_gst_amount/2;
                                    $sgst_tax = $receipt_gst_amount/2;
                                    $adjust->cgst_amount=$cgst_tax;
                                    $adjust->sgst_amount=$sgst_tax;
                                }
                                else
                                {
                                    $adjust->igst_amount=$receipt_gst_amount;
                                }

                                $receipt_taxable_amount =  $value['totamount'] - $receipt_gst_amount;
                                if($find_inv_data->is_tcs==1)
                                {
                                    $receipt_tds_amount = $value['totamount']*(1/100);
                                    $adjust->tcs_amount=$receipt_tds_amount;
                                    $receipt_taxable_amount =  $receipt_taxable_amount - $receipt_tds_amount;
                                }
                                $adjust->taxable_amount = $receipt_taxable_amount;
                            }
                    }
                    
                    else
                    $adjust->type="opening";
                    $adjust->tds_amount=$value['tds_amount'];
                    $adjust->notes=$value['notes'];
                    $adjust->amount_adjusted=$value['totamount'];
                    $extra_amount_paid_by_customer = $extra_amount_paid_by_customer + $adjust->amount_adjusted;
                    $adjust->save(false);
                }
                }
                if($extra_amount_paid_by_customer < $model->amount)
                {
                    $vendorinds=Vendor::findOne($model->vendor_id);
                    // print_r($vendorinds->opening_as_on);
                    // die();
                    if($vendorinds->opening_as_on=='')
                    {
                        $vendorinds->opening_as_on = 0;
                    }
                    // print_r($vendorinds->opening_as_on);
                    // die();
                    $vendorinds->opening_as_on = $vendorinds->opening_as_on - ($prv_model_amount - $prv_adjusted_bill_amt);

                    $vendorinds->opening_as_on=$vendorinds->opening_as_on+($model->amount-$extra_amount_paid_by_customer);
                    $vendorinds->save(false);
                }   

            }
            
            if($model->payment_type!="Direct payment"){
                $amt = $model->amount;
                $tds_amount = $model->tds_amount;
                $final_amt = $model->total_amount;
                $tan_id = $model->id;
                $paid_to = Vendor::find()->where(['id'=>$model->vendor_id])->one();
                $amtto = $paid_to->business_name;
                $payment_mode = $model->payment_mode;
                $tan_date = $model->date;
                $accountingdetails=AccountingDetails::find()->where(['transaction_id'=>'P'.$model->payment_no])->all();
                $ledger = new AccountingDetails();
                $ledger->leadgerentrypayment($amt,$tds_amount,$final_amt,$tan_id,$amtto,$payment_mode,$tan_date);
            }

            

            $data=Yii::$app->request->post();
            $newids=[];
            for($i=1;$i<=$data['imagecount'];$i++){ 
                $newids[]=Yii::$app->request->post('file'.$i);
            }
            $oldids=ArrayHelper::map($doc, 'id', 'id');
            $diffids=array_diff($oldids,$newids);
            $model->updated_by=Yii::$app->user->identity->id;
            if($model->save(false)){
                PurchasePaymentDocument::deleteAll(['in','id',$diffids]);
                for($i=2;$i<=$data['imagecount']+1;$i++){ 
                if(!Yii::$app->request->post('file'.$i)){
                $document = new PurchasePaymentDocument();
                $document->document_name = UploadedFile::getInstanceByName('file'.$i);
                if($document->document_name){
                $nameimage=(new \DateTime())->format('u');
                $document->document_name->saveAs('usersdocument/purchase_payment/' .$nameimage.'.'.$document->document_name->extension);
                $document->document_name=$nameimage. '.' . $document->document_name->extension; 
                $document->payment_id=$model->id;
                $document->save();
                }
                }
                }

                if($model->against_bill !='' && $model->total_bill_amount!= '')
                {
                    $bill=PurchaseBill::find()->where(['id'=>$model->against_bill])->one();

                    if(($bill->pending_amount-$model->amount)>=0)
                    {
                        $bill->pending_amount = $bill->pending_amount-$model->amount;
                        $bill->save(false);

                        $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                        if($purchase_order)
                        {
                            $purchase_order->pending_amount = $purchase_order->pending_amount - $model->amount;
                            $purchase_order->save(false);
                        }
                        
                    }
                    else{

                        
                            $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                            if($purchase_order)
                            {
                                $purchase_order->pending_amount = $purchase_order->pending_amount - $bill->pending_amount;
                                 $purchase_order->save(false);
                            }
                            
                            $bill->extra_paid_amount=$model->amount-$bill->pending_amount;
                            $bill->pending_amount=0;
                           
                            $bill->save(false);

                    }
                    $purchase_order = PurchaseOrder::find()->where(['id'=>$bill->purchase_order_id])->one();
                    if($purchase_order)
                    {
                        if($purchase_order->pending_amount==0)
                        {
                            $purchase_order->gr_status=2;
                             $purchase_order->save(false);
                        }
                    }

                    $find_inv_data = PurchaseBill::find()->where(['id'=>$model->against_bill])->one();
                    $req_invoice_amount = $find_inv_data->total;
                    if($find_inv_data->point_of_supply == '21')
                    {
                        $total_tax = $find_inv_data->cgst + $find_inv_data->sgst;
                    }
                    else
                    {
                        $total_tax = $find_inv_data->igst;
                    }
                    $gst_percentage = ((($total_tax/$req_invoice_amount))*100);
                    $receipt_gst_amount = $model->total_amount*($gst_percentage/100);
                    if($find_inv_data->point_of_supply == '21')
                    {
                        $cgst_tax = $receipt_gst_amount/2;
                        $sgst_tax = $receipt_gst_amount/2;
                        $model->cgst_amount=$cgst_tax;
                        $model->sgst_amount=$sgst_tax;
                    }
                    else
                    {
                        $model->igst_amount=$receipt_gst_amount;
                    }

                    $receipt_taxable_amount =  $model->total_amount - $receipt_gst_amount;
                    if($find_inv_data->is_tcs==1)
                    {
                        $receipt_tds_amount = $model->total_amount*(1/100);
                        $model->tcs_amount=$receipt_tds_amount;
                        $receipt_taxable_amount =  $receipt_taxable_amount - $receipt_tds_amount;
                    }
                    $model->taxable_amount = $receipt_taxable_amount;
                    $model->save(false);
                    

                }
          
                Yii::$app->getSession()->setFlash('success', 'Updated Successfully');
                if(Yii::$app->request->post('createnew'))
                return $this->redirect(['create']);
                return $this->redirect(['index']);
        }
    }

        return $this->render('_form', [
            'model' => $model,
            'update'=>1,
            'vendorlist'=>$vendorlist,
            'paymentmode'=>$paymentmodelist,
            'count'=>$count,
            'doc'=>$doc,
            'from'=>$from,
            'last_entry'=>$last_entry+1,
        ]);
    }
    public function actionOptionData()
    {
        if(!Yii::$app->request->post('data')['payment_mode'])
        return json_encode([
            'required'=>"required",
        ]);
     
        if(Yii::$app->request->post('data')['payment_mode']){
        $model=new PaymentMode();
        $model->payment_mode_name=Yii::$app->request->post('data')['payment_mode'];
        if(!$model->save()){
            return json_encode([
                'error'=>"payment-mode",
            ]);
        }
        }
    $payment=PaymentMode::find()->asArray()->all();
      return json_encode([
          'payment_mode'=>$payment,
          'error'=>false
      ]);
    }
    /**
     * Deletes an existing PurchasePayment model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id=null)
    {
        Yii::$app->view->params['active']=109;
        if($id==null)
        {
            $ids=Yii::$app->request->post('data');
            if($ids){
            foreach($ids as $id){
            $document1=PurchasePaymentDocument::find()->where(['payment_id'=>$id])->all();
            foreach($document1 as $value){$value->is_deleted=1;$value->save();}
            $data=$this->findModel($id);
            $data->is_deleted=1;$data->updated_by=Yii::$app->user->identity->id;$data->save(false);

		  $accounting = AccountingDetails::find()->where(['transaction_id'=>$id])->andWhere(['transaction_type'=>"Payment"])->all();
                foreach ($accounting as $account) {
                    $account->edit_status = 1;
                    $account->save(false);
                }
            if($data->against_bill!='')
            {
                $bill = PurchaseBill::find()->where(['id'=>$data->against_bill])->one();
                $bill->pending_amount = $bill->pending_amount+$data->total_amount;
                $bill->save(false);
            }   
            
            if($data->payment_type!='Payment against bill' && $data->against_bill!='')
            {
                $adj_bills = AdjustedBills::find()->where(['payment_id'=>$data->id])->all();
                foreach ($adj_bills as $key => $value) {
                    $bill = PurchaseBill::find()->where(['id'=>$value->bill_id])->one();
                    $bill->pending_amount = $bill->pending_amount+$data->total_amount;
                    $bill->save(false);
                }
            }



            }
            }
            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }
        else
        {
            $data=$this->findModel($id);
            $data->is_deleted=1;$data->updated_by=Yii::$app->user->identity->id;$data->save(false);
            $document=PurchasePaymentDocument::find()->where(['payment_id'=>$id])->all();
            foreach($document as $value){$value->is_deleted=1;$value->save();}
		   
            if($data->against_bill!='')
            {
                $bill = PurchaseBill::find()->where(['id'=>$data->against_bill])->one();
                $bill->pending_amount = $bill->pending_amount+$data->total_amount;
                $bill->save(false);
            }  

            $accounting = AccountingDetails::find()->where(['transaction_id'=>$id])->andWhere(['transaction_type'=>"Payment"])->all();
              foreach ($accounting as $account) {
                  $account->edit_status = 1;
                  $account->save(false);
              }

            if($data->payment_type!='Payment against bill' && $data->against_bill!='')
            {
                $adj_bills = AdjustedBills::find()->where(['payment_id'=>$data->id])->all();
                foreach ($adj_bills as $key => $value) {
                    $bill = PurchaseBill::find()->where(['id'=>$value->bill_id])->one();
                    $bill->pending_amount = $bill->pending_amount+$data->total_amount;
                    $bill->save(false);
                }
            } 



            Yii::$app->getSession()->setFlash('success', 'Deleted Successfully');
            return $this->redirect(['index']);
        }  
    }
    
    public function actionExcelExport($dataProvider=null,$detail=null)
    {
        Yii::$app->view->params['active']=109;

        $check_id = Yii::$app->user->identity->id;
        $user_type = AuthAssignment::find()->where(['user_id'=>$check_id])->one();
        if($user_type->item_name == 'Admin')
        {
            $query=PurchasePayment::find()->where(['is_deleted'=>0]);
        }
        else
        {
            $accessed = Yii::$app->user->identity->id;
            $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
            $customer_ids = [];
            $j = 0;
            foreach($customers as $data)
            {
                $customer_ids[$j] = $data->client_id;
                $j++;
            }
            $query = PurchasePayment::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
        }

        if($dataProvider==null)
        {

                $dataProvider = new ActiveDataProvider([
                    'query' => $query,
                ]);
        }
        if($detail){
    $exporter = new Spreadsheet([
        'dataProvider' => $dataProvider,
        'columns' => [
            [
                'label'=>'Payment #',
               'attribute' => 'payment_no',
               'value' => function ($data) {
                return 'P '.$data->payment_no;
               }
            ],
            'date:date',
            'amount',
            [
                'label'=>'Bank',
               'attribute' => 'payment_mode',
               'value' => function ($data) {
                return $data->payment_mode;
               }
            ],
            [
               'label'=>'Vendor/Category',
               'value' => function ($data) {
                $against_ledger =  array();
                $all_ledgers = array();
                $vendor=Vendor::find()->where(['id'=>$data->vendor_id])->one();
                if($vendor!='')
                {
                   return $vendor->business_name;
                }
                else
                {
                    $against = DirectPayment::find()->where(['payment_id'=>$data->id])->all();
                    foreach ($against as $key => $value) {
                        $against_ledger[$key] = $value->ledger_id;
                    }
                    foreach ($against_ledger as $key => $val) {
                        $ledger = AccountingLedger::find()->where(['accounting_ledger_id'=>$val])->one();
                        if($ledger)
                          $all_ledgers[$key] = $ledger->ledger_id;
                    }
                    return implode(",",$all_ledgers);
                    
                }
               }
            ],
            'notes',
            'setting_type',
            'payment_type',
            'cheque_no',
            'cheque_date:date',
            'tds_amount',
            'total_amount',
            'created_at:date',
            'updated_at:date',
        ],
    ]);
        }else{
            $exporter = new Spreadsheet([
                'dataProvider' => $dataProvider,
                'columns' => [
                    [
                        'label'=>'Payment #',
                       'attribute' => 'payment_no',
                       'value' => function ($data) {
                        return 'P '.$data->payment_no;
                       }
                    ],
                    'date:date',
                    'amount',
                    [
                        'label'=>'Bank',
                       'attribute' => 'payment_mode',
                       'value' => function ($data) {
                        return $data->payment_mode;
                       }
                    ],
                    [
                       'label'=>'Vendor/Category',
                       'attribute' => 'vendor_id',
                       'value' => function ($data) {
                        $vendor=Vendor::find()->where(['id'=>$data->vendor_id])->one();
                        if($vendor!='')
                        {
                           return $vendor->business_name;
                        }
                       }
                    ],
                    'notes',
                 
                ],
            ]);
        }
    if($detail)
    return $exporter->send('purchase_payment_details.xlsx');
    return $exporter->send('purchase_payment.xlsx');
    }
   
    public function actionGetVendor()
    {
        $vendor=Vendor::find()->all();
        $vendorlist=ArrayHelper::map($vendor, 'id', 'business_name');
        return json_encode($vendorlist);
    }
    public function actionVendordetails($searchTerm=null)
    {   
        if(!$searchTerm)
        $customer=Vendor::find()->limit(10)->all();
        else
        $customer=Vendor::find()->where(["like","business_name",$searchTerm])->limit(10)->all();

        $customerlist=ArrayHelper::map($customer, 'id', 'business_name');
        return json_encode($customerlist);
    }
    Public function actionDownloadAttachment($id)
    {
    Yii::$app->view->params['active']=109;
    $data=PurchasePaymentDocument::find()->where(['payment_id'=>$id,'is_deleted'=>0])->all();
    if(!$data){
    Yii::$app->getSession()->setFlash('success', 'No Attachment Found');
    return $this->redirect(['index']);
    }
    $file = Yii::$app->basePath.'/web/usersdocument/purchase_payment/product_'.$id.'.zip';
    $zip = new \ZipArchive();
    $zip->open($file, \ZipArchive::CREATE | \ZipArchive::OVERWRITE) ;
    foreach($data as $value){
       $files[]=Yii::$app->basePath . '/web/usersdocument/purchase_payment/'.$value->document_name;
    }
    foreach($files as $val){
        $zip->addFromString(basename($val), file_get_contents($val));
    }
    $zip->close();
    return Yii::$app->response->sendFile($file);
    }
    public function actionPendingBill()
    {
        $vendor_id=Yii::$app->request->post('data');
        $bill="SELECT * FROM `purchase_bill` WHERE pending_amount>0 and vendor_id=$vendor_id";
        $bills=Yii::$app->db->createCommand($bill)->queryAll();
        $vendor=Vendor::find()->where(['id'=>$vendor_id])->asArray()->one();
        return json_encode(['bills'=>$bills,
        'vendor'=>$vendor]
    );
    }
    public function actionPendingOpening()
    {
        $vendor_id=Yii::$app->request->post('data');
        $vendor=Vendor::find()->where(['id'=>$vendor_id])->one();
        $openingtotal=AdjustedBills::find()->where(['type'=>"opening",'vendor_id'=>$vendor_id])->sum('`amount_adjusted`')+$vendor->opening_as_on;
        return $openingtotal;
    }


     public function actionFindpaymentalldetails($id)
    {
        $all= PurchasePayment::find()->where(['id' =>$id])->one();
        echo Json::encode($all);
    } 
    public function actionCreateUrl($id)
    {
       // $ids=Yii::$app->request->post('data');
        //$model = $this->findModel($id);
        $url = Url::to(['purchasepayment-template', 'id' => $id],'https');
        echo Json::encode($url);
            // print_r($url);

    }
    public function actionFinduseralldetails($id)
    {
        $all= PurchasePayment::find()->where(['id' =>$id])->one();
        $userdetails = Vendor::find()->where(['id'=>$all->vendor_id])->one();
        echo Json::encode($userdetails);
    }



    /**
     * Finds the PurchasePayment model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return PurchasePayment the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = PurchasePayment::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
    public function actionFieldpurchasepaymentsettingtype($setting=null,$template=null)
    {   
        //print_r(1);die();
        $find = GenericTemplate::find()->where(['template_for'=>'Payment'])->andWhere(['gt_id'=>$setting])->count();
        echo Json::encode($find);
    }

    public function actionSelectdate($selected_date=null)
    {   
        //$selected_date=$_GET['selected_date']; 
        $year=date("Y"); 
        $dynamic_date=$year.'-'.'03'.'-'.'31';
        $count_before_year=PurchasePayment::find()->where(['<=', 'date', $dynamic_date])->count();
        $count_after_year=PurchasePayment::find()->where(['>=', 'date', $dynamic_date])->count();
        if(strtotime($selected_date) <= strtotime($dynamic_date)) { 
            $last_entry=$count_before_year+1;
        }
        else if(strtotime($selected_date) > strtotime($dynamic_date)){
            $last_entry=$count_after_year+1;
        } 
        else{
            $last_entry=0;
        }
        echo $last_entry;
    }



}
