<?php

namespace backend\modules\users\models;

use Yii;

/**
 * This is the model class for table "credit_note".
 *
 * @property int $id
 * @property string $setting_type
 * @property string|null $customer_name
 * @property string|null $notes
 * @property string|null $receiver_gstn
 * @property string|null $sales_type
 * @property string|null $supply_type
 * @property string|null $supply_reverse_charge
 * @property string|null $pricelist
 * @property string|null $point_of_supply
 * @property string|null $is_invoice_paid_provisional
 * @property string|null $perfix
 * @property string|null $credit_note_no
 * @property string|null $suffix
 * @property string|null $credit_note_id
 * @property string|null $date
 * @property string|null $invoice_no
 * @property string|null $invoice_date
 * @property float $sub_total_amount
 * @property float $total_amount
 * @property int|null $match_invoice
 * @property int $show_to_store_keeper
 * @property string|null $full_order
 * @property int $product_wise
 * @property string|null $credit_note_terms
 * @property string|null $credit_notes
 * @property string $created_date
 * @property string $updated_date
 * @property int $id_deleted
 */
class CreditNote extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'credit_note';
    }
     public $from_date;
     public $to_date;
     public $customer;
     public $type_of_customer;
     public $setting;
     public $amount;
     public $user;
     public $product;

     public $c_code;
     public $c_mobile;
     public $findid;
     public $text_send;
     public $send_email;
     public $bcc_to_email;
     public $cc_to_email;
     public $email_subject;
     public $email_message;

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['customer_name','date','invoice_no','invoice_date','customer_type','setting_type'], 'required'],
            [['notes', 'credit_note_terms', 'credit_notes','text_send','email_subject','email_message','findid'], 'string'],
            [['date', 'invoice_date', 'created_at', 'updated_at','from_date','to_date','is_tcs'], 'safe'],
            [['sub_total_amount', 'total_amount','amount','sgst_amount','cgst_amount','igst_amount','tcs_amount','round_off_amount'], 'number'],
            [['match_invoice', 'show_to_store_keeper', 'is_deleted'], 'integer'],
            [['setting_type', 'customer_name', 'receiver_gstn', 'sales_type', 'supply_type', 'supply_reverse_charge', 'pricelist', 'point_of_supply', 'is_invoice_paid_provisional', 'credit_note_id', 'invoice_no','customer','setting','product','customer_type','type_of_customer'], 'string', 'max' => 254],
            ['send_email','email','message'=> 'Enter a valid Email'],
            ['bcc_to_email','email','message'=> 'Enter a valid Email'],
            ['cc_to_email','email','message'=> 'Enter a valid Email'],
            [['prefix', 'credit_note_no', 'suffix', 'order_type','c_code','c_mobile'], 'string', 'max' => 32],
            [['receiver_gstn'], 'string','min'=>15, 'max' => 15],
            [['receiver_gstn'], 'match' ,'pattern'=>'/^([0-9]){2}([A-Z]){4}([0-9A-Z]){1}([0-9]){4}([A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}?$/', 'message'=> 'Receiver GSTN Format should be like  12ABCDE1234Z1Z1.'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'setting_type' => 'Setting Type',
            'customer_name' => 'Customer Name',
            'notes' => 'Notes',
            'receiver_gstn' => 'Receiver Gstn',
            'sales_type' => 'Sales Type',
            'supply_type' => 'Supply Type',
            'supply_reverse_charge' => 'Supply Reverse Charge',
            'pricelist' => 'Pricelist',
            'point_of_supply' => 'Point Of Supply',
            'is_invoice_paid_provisional' => 'Is Invoice Paid Provisional',
            'perfix' => 'Perfix',
            'credit_note_no' => 'Credit Note No',
            'suffix' => 'Suffix',
            'credit_note_id' => 'Credit Note ID',
            'date' => 'Date',
            'invoice_no' => 'Invoice No',
            'invoice_date' => 'Invoice Date',
            'sub_total_amount' => 'Sub Total Amount',
            'total_amount' => 'Total Amount',
            'match_invoice' => 'Match Invoice',
            'show_to_store_keeper' => 'Show To Store Keeper',
            'full_order' => 'Full Order',
            'credit_note_terms' => 'Credit Note Terms',
            'credit_notes' => 'Credit Notes',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'id_deleted' => 'Id Deleted',
        ];
    }
}
