<?php

namespace backend\modules\users\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use backend\modules\users\models\Invoice;
use backend\modules\master\models\LeadExecutive;
use backend\models\UserClient;
use backend\modules\master\models\Customer;

/**
 * InvoiceSearch represents the model behind the search form of `backend\modules\users\models\Invoice`.
 */
class InvoiceSearch extends Invoice
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id'], 'integer'],
            [['settings_id', 'customer_name', 'executive', 'receiver_gstn', 'customer_referance', 'sales_type', 'notes', 'sypply_type', 'supply_reverse_change', 'po_number', 'po_date', 'price_list', 'point_of_supply', 'income_terms', 'tax_status', 'invoice_no', 'date', 'due_date', 'terms_of_payment', 'sub_total_price', 'round_up_price', 'total_price', 'time_of_preparation', 'time_o_removal', 'reminder', 'add_delivery_details', 'buyer_details', 'dispatch_details', 'deliver_to', 'recurring', 'packing_details', 'invoice_documents', 'created_at', 'updated_at','status','shipment_status','invoice_id'], 'safe'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params,array $accessed=null)
    {
        if($accessed == null)
        {
           $query = Invoice::find()->where(['is_deleted'=>0])->orderBy(['id' => SORT_DESC]);
        }
        else
        {
            $check_id = Yii::$app->user->identity->id;
            $all = Invoice::find()->where(['is_deleted'=>0])->orderBy(['id' => SORT_DESC])->all();
            
            $exe_array= [];
             $i=0;
            foreach ($all as $key => $val) {
                $executivedetails = LeadExecutive::find()->where(['name'=>$val['executive']])->one();
                if($executivedetails!='')
                {
                   if($executivedetails->type == $accessed[0]) 
                      $exe_array[$i] = $executivedetails->name;
                    $i++;

                }
            }

            $exe_array = array_unique($exe_array);

            $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Customer'])->all();
            $customer_ids = [];
            $j = 0;
            foreach($customers as $data)
            {
                $customer_ids[$j] = $data->client_id;
                $j++;
            }
            $query = Invoice::find()->where(['is_deleted'=>0])->andWhere(['in', 'customer_name', $customer_ids])->andWhere(['in', 'executive', $exe_array])->orderBy(['id' => SORT_DESC]);
        }
        

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);
        // print_r($dataProvider->models);
        // die();

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'invoice_id' => $this->invoice_id,
            'po_date' => $this->po_date,
            'date' => $this->date,
            'due_date' => $this->due_date,
            'time_of_preparation' => $this->time_of_preparation,
            'time_o_removal' => $this->time_o_removal,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'settings_id', $this->settings_id])
            ->andFilterWhere(['like', 'customer_name', $this->customer_name])
            ->andFilterWhere(['like', 'executive', $this->executive])
            ->andFilterWhere(['like', 'receiver_gstn', $this->receiver_gstn])
            ->andFilterWhere(['like', 'customer_referance', $this->customer_referance])
            ->andFilterWhere(['like', 'sales_type', $this->sales_type])
            ->andFilterWhere(['like', 'notes', $this->notes])
            ->andFilterWhere(['like', 'sypply_type', $this->sypply_type])
            ->andFilterWhere(['like', 'supply_reverse_change', $this->supply_reverse_change])
            ->andFilterWhere(['like', 'po_number', $this->po_number])
            ->andFilterWhere(['like', 'price_list', $this->price_list])
            ->andFilterWhere(['like', 'point_of_supply', $this->point_of_supply])
            ->andFilterWhere(['like', 'income_terms', $this->income_terms])
            ->andFilterWhere(['like', 'tax_status', $this->tax_status])
            ->andFilterWhere(['like', 'invoice_no', $this->invoice_no])
            ->andFilterWhere(['like', 'terms_of_payment', $this->terms_of_payment])
            ->andFilterWhere(['like', 'sub_total_price', $this->sub_total_price])
            ->andFilterWhere(['like', 'round_up_price', $this->round_up_price])
            ->andFilterWhere(['like', 'total_price', $this->total_price])
            ->andFilterWhere(['like', 'reminder', $this->reminder])
            ->andFilterWhere(['like', 'add_delivery_details', $this->add_delivery_details])
            ->andFilterWhere(['like', 'buyer_details', $this->buyer_details])
            ->andFilterWhere(['like', 'dispatch_details', $this->dispatch_details])
            ->andFilterWhere(['like', 'deliver_to', $this->deliver_to])
            ->andFilterWhere(['like', 'recurring', $this->recurring])
            ->andFilterWhere(['like', 'packing_details', $this->packing_details])
            ->andFilterWhere(['like', 'status', $this->status])
            ->andFilterWhere(['like', 'shipment_status', $this->shipment_status])
            ->andFilterWhere(['like', 'invoice_documents', $this->invoice_documents]);

        return $dataProvider;
    }
}
