<?php

namespace backend\modules\users\models;

use Yii;

/**
 * This is the model class for table "leads_quotes".
 *
 * @property int $id
 * @property string|null $supply_attracts
 * @property string|null $customer
 * @property string|null $executive
 * @property string|null $notes
 * @property string|null $receiver_gstn
 * @property string|null $sales_type
 * @property string|null $supply_type
 * @property string|null $supply_attracts_rcharge
 * @property string|null $pointof_supply
 * @property string|null $pricelist
 * @property string|null $terms
 * @property string $Is_tax_paid
 * @property string $prefix
 * @property string $quotes_no
 * @property string $suffix
 * @property string $quotes_id
 * @property string $closure_property
 * @property float $sub_total_amount
 * @property float $total_amount
 * @property string $set_reminder
 * @property string $add_delivery_details
 * @property string $packing_details
 * @property string $date
 * @property int $is_deleted
 * @property string $created_date
 * @property string $updated_date
 */
class LeadsQuotes extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'leads_quotes';
    }
    public $from_date;
    public $to_date;
    public $customer_name;
    public $setting;
    public $executive_name, $quote_status, $convert_status, $amount, $quote,$type_of_customer,$amount_compare,$find_product,$lead_quote_id,$closure_from,$closure_to;
    public $c_code;
    public $c_mobile;
    public $text_send;
    public $send_email;
    public $bcc_to_email;
    public $cc_to_email;
    public $email_subject;
    public $email_message;
    public $findid;

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['notes', 'terms','text_send','email_message'], 'string'],
            [['customer', 'date'], 'required'],
            [['sub_total_amount', 'total_amount','igst_amount','cgst_amount','sgst_amount','tcs_amount'], 'number'],
            [['date', 'created_date', 'updated_date','from_date','to_date','reminder_from_date','reminder_to_date'], 'safe'],
            [['is_deleted'], 'integer'],
            [['supply_attracts', 'customer', 'executive', 'receiver_gstn', 'sales_type', 'supply_type', 'supply_attracts_rcharge', 'pointof_supply', 'pricelist','findid'], 'string', 'max' => 255],
            [['Is_tax_paid','is_tcs','c_code','c_mobile'], 'string', 'max' => 30],
            [['prefix', 'quotes_no', 'suffix', 'quotes_id', 'closure_property', 'set_reminder', 'add_delivery_details', 'packing_details','converted_to','status','dimention_of_primary','dimention_of_master','dimention_of_pallet','no_of_pallets','net_weight','gross_weight','delivery_note','suppliers_ref','other_reference','executive_name','customer_name','setting','quote_status','convert_status','amount','type_of_customer','amount_compare','quote','find_product','lead_quote_id','closure_from','closure_to','reminder_for','reminder_through','reminder_email','reminder_content','reminder_repeat','email_subject'], 'string', 'max' => 254],
            [['receiver_gstn'], 'string','min'=>15, 'max' => 15],
             [['receiver_gstn'], 'match' ,'pattern'=>'/^([0-9]){2}([A-Z]){4}([0-9A-Z]){1}([0-9]){4}([A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}?$/', 'message'=> 'Receiver GSTN Format should be like  12ABCDE1234Z1Z1.'],
            ['reminder_email','email'],
            ['send_email','email','message'=> 'Enter a valid Email'],
            ['bcc_to_email','email','message'=> 'Enter a valid Email'],
            ['cc_to_email','email','message'=> 'Enter a valid Email'],
            [['reminder_from_date'], 'compare' ,'compareValue'=>date("Y-m-d"),'operator'=>'>=', 'message'=> 'Due Date must be equal/greater than todays date'],
            ['reminder_to_date', 'compare', 'compareAttribute' => 'reminder_from_date', 'operator' => '>'],
             [['to_date'], 'compare' ,'compareAttribute' => 'from_date','operator'=>'>=', 'message'=> 'To Date must be equal/greater than From date'],
             ['to_date', 'required', 'when' => function($model) {
                return $model->from_date;
            },'whenClient' => "function (attribute, value) {
                return $('#leadsquotes-from_date').val();
            }"],
            ['from_date', 'required', 'when' => function($model) {
                return $model->to_date;
            },'whenClient' => "function (attribute, value) {
                return $('#leadsquotes-to_date').val();
            }"],
            [['closure_to'], 'compare' ,'compareAttribute' => 'closure_from','operator'=>'>', 'message'=> 'Closure To value  must be greater than Closure From'],

        ];
    }
    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'supply_attracts' => 'Supply Attracts',
            'customer' => 'Customer',
            'executive' => 'Executive',
            'notes' => 'Notes',
            'receiver_gstn' => 'Receiver Gstn',
            'sales_type' => 'Sales Type',
            'supply_type' => 'Supply Type',
            'supply_attracts_rcharge' => 'Supply Attracts Rcharge',
            'pointof_supply' => 'Pointof Supply',
            'pricelist' => 'Pricelist',
            'terms' => 'Terms',
            'Is_tax_paid' => 'Is Tax Paid',
            'prefix' => 'Prefix',
            'quotes_no' => 'Quotes No',
            'suffix' => 'Suffix',
            'quotes_id' => 'Quotes ID',
            'closure_property' => 'Closure Property',
            'sub_total_amount' => 'Sub Total Amount',
            'total_amount' => 'Total Amount',
            'set_reminder' => 'Set Reminder',
            'add_delivery_details' => 'Add Delivery Details',
            'packing_details' => 'Packing Details',
            'date' => 'Date',
            'is_deleted' => 'Is Deleted',
            'created_date' => 'Created Date',
            'updated_date' => 'Updated Date',
        ];
    }
}
