<?php

namespace backend\modules\users\models;
use yii\validators\EmailValidator;

use Yii;

/**
 * This is the model class for table "proforma_invoice".
 *
 * @property int $id
 * @property string $setting_id
 * @property string $customer_name
 * @property string|null $notes
 * @property string $executive
 * @property string|null $receiver_gstn
 * @property string|null $sales_type
 * @property string|null $supply_type
 * @property string|null $supply_reverse_change
 * @property string|null $point_of_supply
 * @property string|null $is_invoice_paid_provisional
 * @property string|null $prefix
 * @property string|null $proforma_invoice_no
 * @property string|null $suffix
 * @property string|null $proforma_invoice_id
 * @property string|null $date
 * @property string|null $due_date
 * @property string|null $terms_of_payment
 * @property float|null $sub_total_price
 * @property float|null $total_price
 * @property string|null $reminder
 * @property string|null $reminder_for
 * @property string|null $reminder_through
 * @property string|null $reminder_emails
 * @property string|null $reminder_content
 * @property string|null $reminder_repeat_on
 * @property string|null $reminder_from_date
 * @property string|null $reminder_to_date
 * @property string|null $add_delivery_details
 * @property string|null $delivery_note
 * @property string|null $suppliers_reference
 * @property string|null $other_reference
 * @property string|null $converted_to
 * @property string|null $receipt_id
 * @property string|null $terms
 * @property string|null $status
 * @property string $created_at
 * @property string $updated_at
 */
class ProformaInvoice extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'proforma_invoice';
    }
public $invoice_prifix;
    public $invoice_number;
    public $invoice_sufix;
    public $proforma;
    public $from_date;
    public $to_date;
    public $customer;
    public $setting;
    public $total_amount;


    public $c_code;
    public $c_mobile;
    public $findid;
    public $text_send;
    public $send_email;
    public $bcc_to_email;
    public $cc_to_email;
    public $email_subject;
    public $email_message;

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [[ 'customer_name', 'date','setting_id'], 'required'],
            [['notes', 'reminder_content', 'delivery_note', 'suppliers_reference', 'other_reference', 'terms','findid','text_send','email_subject','email_message'], 'string'],
            ['send_email','email','message'=> 'Enter a valid Email'],
            ['bcc_to_email','email','message'=> 'Enter a valid Email'],
            ['cc_to_email','email','message'=> 'Enter a valid Email'],
            [['date', 'due_date', 'reminder_from_date','from_date','to_date', 'reminder_to_date', 'created_at', 'updated_at'], 'safe'],
            [['sub_total_price', 'total_price','tcs_amount','igst_amount','sgst_amount','cgst_amount'], 'number'],
            [['setting_id', 'customer_name', 'executive', 'receiver_gstn', 'sales_type', 'supply_type', 'supply_reverse_change', 'point_of_supply', 'prefix', 'suffix', 'proforma_invoice_id', 'terms_of_payment', 'reminder_for', 'reminder_through', 'reminder_repeat_on', 'converted_to', 'status','proforma','customer','setting','total_amount', 'reminder_emails','lead_quote_id','sale_order_id'], 'string', 'max' => 254],
            [['is_invoice_paid_provisional', 'proforma_invoice_no', 'reminder', 'add_delivery_details','is_tcs','c_code','c_mobile'], 'string', 'max' => 30],
            [['receiver_gstn'], 'string','min'=>15, 'max' => 15],
            [['receiver_gstn'], 'match' ,'pattern'=>'/^([0-9]){2}([A-Z]){4}([0-9A-Z]){1}([0-9]){4}([A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}([0-9A-Z]){1}?$/', 'message'=> 'GSTN Format should be like  12ABCDE1234Z1Z1.'],
            ['due_date', 'compare', 'compareAttribute' => 'date', 'operator' => '>='],
            ['reminder_emails','email'],
            [['reminder_from_date'], 'compare' ,'compareValue'=>date("Y-m-d"),'operator'=>'>=', 'message'=> 'Due Date must be equal/greater than todays date'],
            ['reminder_to_date', 'compare', 'compareAttribute' => 'reminder_from_date', 'operator' => '>'],

            [['to_date'], 'compare' ,'compareAttribute' => 'from_date','operator'=>'>=', 'message'=> 'To Date must be equal/greater than From date'],
             ['to_date', 'required', 'when' => function($model) {
                return $model->from_date;
            },'whenClient' => "function (attribute, value) {
                return $('#proformainvoice-from_date').val();
            }"],
            ['from_date', 'required', 'when' => function($model) {
                return $model->to_date;
            },'whenClient' => "function (attribute, value) {
                return $('#proformainvoice-to_date').val();
            }"],

        ];
    }
    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'setting_id' => 'Setting ID',
            'customer_name' => 'Customer Name',
            'notes' => 'Notes',
            'executive' => 'Executive',
            'receiver_gstn' => 'Receiver Gstn',
            'sales_type' => 'Sales Type',
            'supply_type' => 'Supply Type',
            'supply_reverse_change' => 'Supply Reverse Change',
            'point_of_supply' => 'Point Of Supply',
            'is_invoice_paid_provisional' => 'Is Invoice Paid Provisional',
            'prefix' => 'Prefix',
            'proforma_invoice_no' => 'Proforma Invoice No',
            'suffix' => 'Suffix',
            'proforma_invoice_id' => 'Proforma Invoice ID',
            'date' => 'Date',
            'due_date' => 'Due Date',
            'terms_of_payment' => 'Terms Of Payment',
            'sub_total_price' => 'Sub Total Price',
            'total_price' => 'Total Price',
            'reminder' => 'Reminder',
            'reminder_for' => 'Reminder For',
            'reminder_through' => 'Reminder Through',
            'reminder_emails' => 'Reminder Emails',
            'reminder_content' => 'Reminder Content',
            'reminder_repeat_on' => 'Reminder Repeat On',
            'reminder_from_date' => 'Reminder From Date',
            'reminder_to_date' => 'Reminder To Date',
            'add_delivery_details' => 'Add Delivery Details',
            'delivery_note' => 'Delivery Note',
            'suppliers_reference' => 'Suppliers Reference',
            'other_reference' => 'Other Reference',
            'converted_to' => 'Converted To',
            'receipt_id' => 'Receipt ID',
            'terms' => 'Terms',
            'status' => 'Status',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }
    public function emailList($attribute, $params) {

        $validator = new EmailValidator;

        $emails = explode(',', $this->reminder_emails);

        foreach ($emails as $email) {

            $validator->validate($email)? : $this->addError($attribute, "'$email' is not a valid email.");

        }

    }


}
