<?php

namespace backend\modules\users\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use backend\modules\users\models\PurchaseBill;
use backend\modules\master\models\Vendor;
use backend\models\UserClient;
/**
 * PurchaseBillSearch represents the model behind the search form of `backend\modules\users\models\PurchaseBill`.
 */
class PurchaseBillSearch extends PurchaseBill
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'supply_attracts', 'is_ineligible_itc', 'round_off', 'payment', 'set_reminder', 'show_to_store_keeper', 'order_type'], 'integer'],
            [['vendor_id','setting_type', 'vendor_bill_no', 'bill_date', 'sender_gst_id', 'notes', 'bill_type', 'supply_type', 'point_of_supply', 'pricelist', 'prefix', 'bill_no', 'suffix', 'date', 'due_date', 'bill_notes', 'terms_of_payment', 'product_code', 'terms', 'created_at', 'updated_at'], 'safe'],
            [['subtotal', 'total', 'tcs'], 'number'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params,array $accessed=null)
    {

        if($accessed == null)
        {
            $query = PurchaseBill::find()->where(['is_deleted'=>0])->orderBy(['id' => SORT_DESC]);
        }
        else
        {
            $vendors = UserClient::find()->where(['user_id'=>$accessed])->andWhere(['client_type'=>'Vendor'])->all();
            $vendor_ids= [];
            $i = 0;
            foreach($vendors as $data)
            {
                $vendor_ids[$i] = $data->client_id;
                $i++;
            }
            $query = PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['in', 'vendor_id', $vendor_ids])->andWhere(['is_deleted' =>0]);


            // $query = PurchaseBill::find()->orderBy(['id' => SORT_DESC])->where(['is_deleted' =>0])->andWhere(['in', 'updated_by', $accessed]);
        }

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort'=> ['defaultOrder' => ['id' => SORT_DESC]],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            // 'vendor_id' => $this->vendor_id,
            'supply_attracts' => $this->supply_attracts,
            'is_ineligible_itc' => $this->is_ineligible_itc,
            'subtotal' => $this->subtotal,
            'total' => $this->total,
            'tcs' => $this->tcs,
            'round_off' => $this->round_off,
            'payment' => $this->payment,
            'set_reminder' => $this->set_reminder,
            'show_to_store_keeper' => $this->show_to_store_keeper,
            'order_type' => $this->order_type,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'setting_type', $this->setting_type])
            ->andFilterWhere(['like', 'vendor_bill_no', $this->vendor_bill_no])
            ->andFilterWhere(['like', 'bill_date', $this->bill_date])
            ->andFilterWhere(['like', 'sender_gst_id', $this->sender_gst_id])
            ->andFilterWhere(['like', 'notes', $this->notes])
            ->andFilterWhere(['like', 'bill_type', $this->bill_type])
            ->andFilterWhere(['like', 'supply_type', $this->supply_type])
            ->andFilterWhere(['like', 'point_of_supply', $this->point_of_supply])
            ->andFilterWhere(['like', 'pricelist', $this->pricelist])
            ->andFilterWhere(['like', 'prefix', $this->prefix])
            ->andFilterWhere(['like', 'bill_no', $this->bill_no])
            ->andFilterWhere(['like', 'suffix', $this->suffix])
            ->andFilterWhere(['like', 'date', $this->date])
            ->andFilterWhere(['like', 'due_date', $this->due_date])
            ->andFilterWhere(['like', 'bill_notes', $this->bill_notes])
            ->andFilterWhere(['like', 'terms_of_payment', $this->terms_of_payment])
            ->andFilterWhere(['like', 'product_code', $this->product_code])
            ->andFilterWhere(['like', 'terms', $this->terms]);
            if($this->vendor_id){
                $vendor=Vendor::find()->select('id')->where(['like','business_name',$this->vendor_id])->column();
                $query->andFilterWhere(['in', 'vendor_id', $vendor]);
            }
        return $dataProvider;
    }
}
