<?php

namespace backend\modules\users\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use backend\modules\users\models\PurchasePayment;
use backend\modules\master\models\Vendor;
use backend\models\UserClient;
/**
 * PurchasePaymentSearch represents the model behind the search form of `backend\modules\users\models\PurchasePayment`.
 */
class PurchasePaymentSearch extends PurchasePayment
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'updated_by', 'is_deleted'], 'integer'],
            [['vendor_id','setting_type', 'payment_type', 'payment_no', 'date', 'cheque_no', 'cheque_date', 'payment_mode', 'notes'], 'safe'],
            [['amount', 'tds_amount', 'total_amount'], 'number'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params, array $accessed=null)
    {

        if($accessed == null)
        {
            $query = PurchasePayment::find()->where(['is_deleted'=>0]);
        }
        else
        {
            $check_id = Yii::$app->user->identity->id;
            $customers = UserClient::find()->where(['user_id'=>$check_id])->andWhere(['client_type'=>'Vendor'])->all();
            $customer_ids = [];
            $j = 0;
            foreach($customers as $data)
            {
                $customer_ids[$j] = $data->client_id;
                $j++;
            }
            $query = PurchasePayment::find()->where(['is_deleted'=>0])->andWhere(['in', 'vendor_id', $customer_ids])->orderBy(['id' => SORT_DESC]);
        }
        

        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort'=> ['defaultOrder' => ['id' => SORT_DESC]],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'amount' => $this->amount,
            'tds_amount' => $this->tds_amount,
            'updated_by' => $this->updated_by,
            'is_deleted' => $this->is_deleted,
            'total_amount' => $this->total_amount,
        ]);

        $query->andFilterWhere(['like', 'setting_type', $this->setting_type])
            ->andFilterWhere(['like', 'payment_type', $this->payment_type])
            ->andFilterWhere(['like', 'payment_no', $this->payment_no])
            ->andFilterWhere(['like', 'date', $this->date])
            ->andFilterWhere(['like', 'cheque_no', $this->cheque_no])
            ->andFilterWhere(['like', 'cheque_date', $this->cheque_date])
            ->andFilterWhere(['like', 'payment_mode', $this->payment_mode])
            ->andFilterWhere(['like', 'notes', $this->notes]);
            if($this->vendor_id){
                $vendor=Vendor::find()->select('id')->where(['like','business_name',$this->vendor_id])->column();
                $query->andFilterWhere(['in', 'vendor_id', $vendor]);}
        return $dataProvider;
    }
}
